<?php
/**
 * Override Course list block.
 *
 * @package    block_prova_navigation
 * @copyright  1999 onwards Martin Dougiamas (http://dougiamas.com)
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

class block_prova_navigation extends block_list {

    function init() {
        $this->title = get_string('pluginname', 'block_prova_navigation');
    }

    function has_config() {
        return true;
    }

    function get_content() {
        global $CFG, $USER, $DB, $OUTPUT; 
        require_once($CFG->dirroot . '/mod/prova/managelib.php');

        $this->content = new stdClass;
        $this->content->items = array();
        $this->content->icons = array();

        $icon = $OUTPUT->pix_icon('i/course', get_string('course'));

        $adminseesall = true;
        if (has_capability('mod/prova:accesstoread', context_course::instance(SEOX_COURSE_ID))) {                
            $this->content->items[].=$icon.' '.html_writer::link(new moodle_url('/mod/prova/components/questions.php'), get_string('em_sublabel1', 'mod_prova'), array('class' => 'menuexames_link'));
            $this->content->items[].=$icon.' '.html_writer::link(new moodle_url('/mod/prova/components/provas.php'), get_string('em_sublabel2', 'mod_prova'), array('class' => 'menuexames_link'));
            $this->content->items[].=$icon.' '.html_writer::link(new moodle_url('/mod/prova/screens/buscaprova/view.php'), get_string('em_sublabel3', 'mod_prova'), array('class' => 'menuexames_link'));
            $this->content->items[].=$icon.' '.html_writer::link(new moodle_url('/mod/prova/screens/estatisticasquestao/view.php'), get_string('em_sublabel4', 'mod_prova'), array('class' => 'menuexames_link'));
            if (has_capability('mod/prova:accesstowrite', context_course::instance(SEOX_COURSE_ID))) {
                $this->content->items[].=$icon.' '.html_writer::link(new moodle_url('/mod/prova/screens/cad_area/view.php'), get_string('em_sublabel5', 'mod_prova'), array('class' => 'menuexames_link'));
                $this->content->items[].=$icon.' '.html_writer::link(new moodle_url('/mod/prova/screens/cad_subarea/view.php'), get_string('em_sublabel6', 'mod_prova'), array('class' => 'menuexames_link'));
                $this->content->items[].=$icon.' '.html_writer::link(new moodle_url('/mod/prova/screens/cad_assunto/view.php'), get_string('em_sublabel7', 'mod_prova'), array('class' => 'menuexames_link'));
                $this->content->items[].=$icon.' '.html_writer::link(new moodle_url('/user/index.php', array('id' =>SEOX_COURSE_ID)), get_string('em_sublabel8', 'mod_prova'), array('class' => 'menuexames_link'));
            }
        }

        $this->render_customstudentContent();

        return $this->content;
    }
    
    public function get_aria_role() {
        return 'navigation';
    }

    //block/prova_navigation:studentcontent
    public function render_customstudentContent() {
        global $DB, $USER;

        // - user_enrolments enrolid /
        if (!$userEnrols = $DB->get_records('user_enrolments', array('userid' => $USER->id), null, 'enrolid')) { return; }
        $userEnrols = array_keys($userEnrols);

        // - Get user courses from enrols
        $enrolsQuery = 'id IN ('.implode(',', $userEnrols).')';
        $enrolsSql = "SELECT courseid FROM {enrol} WHERE ".$enrolsQuery;
        if (!$userCourses = $DB->get_records_sql($enrolsSql)) { return; }
        $userCourses = array_keys($userCourses);

        // - all prova WHERE curse in list AND with valid time AND not correted
        $provasQuery = array(
            'prova.corrected = 0',
            '(prova.timeclose = 0 OR prova.timeclose >= '.time().')',
            '(prova.timeopen = 0 OR prova.timeopen <= '.time().')',
            'prova.course IN ('.implode(',', $userCourses).')',
        );
        $provasQuery = implode(' AND ', $provasQuery);
        $provaSql = "SELECT prova.id, prova.name, prova.course, prova.cmid FROM {prova} prova INNER JOIN {course_modules} cmod ON cmod.id = prova.cmid WHERE ".$provasQuery;
        if (!$provas = $DB->get_records_sql($provaSql)) { return; }
        $provasIds = array_keys($provas);

        // - all user attempts in these provas.
        // -- if has line AND its status is different to inprogress, do not show alert OR
        $attemptsQuery = array(
            'userid = '.$USER->id,
            'prova IN ('.implode(',', $provasIds).')',
            "state != 'inprogress'",
        );
        $attemptsQuery = implode(' AND ', $attemptsQuery);
        
        $attemptSql = "SELECT id, prova FROM {prova_attempts} WHERE ".$attemptsQuery;
        if ($attempts = $DB->get_records_sql($attemptSql)) {
            foreach ($attempts as $attempt) {
                unset($provas[$attempt->prova]);
            }
        }  

        // - Get course data to prepare msg
        $coursesQuery = array(
            'id IN ('.implode(',', $userCourses).')',
        );
        $coursesQuery = implode(' AND ', $coursesQuery);
        $coursesSql = "SELECT id, fullname FROM {course} WHERE ".$coursesQuery;
        if (!$coursesData = $DB->get_records_sql($coursesSql)) { return; }

        // - Prepare output
        if (!$provas or empty($provas)) { return; }

        $warnings = [];
        foreach ($provas as $prova) {
            if (!has_capability('mod/prova:attempt', context_course::instance($prova->course)) 
                OR has_capability('mod/prova:preview', context_course::instance($prova->course))) { continue; }

            $text = html_writer::tag('p', get_string('provastudent:label', 'block_prova_navigation'), array('class' => 'label-ps-warning'));
            $link = html_writer::link(
                new moodle_url('/mod/prova/view.php', array('id' => $prova->cmid)),
                $coursesData[$prova->course]->fullname . ': ' . $prova->name, 
                array('class' => 'menuexames_link')
            );
            $span = html_writer::tag('span', $text . $link, array('class' => 'text-ps-warning'));
            $warnings[] = html_writer::tag('div', $span, array('class' => 'container-ps-warnin'));
        }

        if (!empty($warnings)) {
            if (!empty($this->content->items)) {
                $this->content->items[] = '<div class="delimiter-ps-warning"></div>';
            }
            $this->content->items = array_merge($this->content->items, $warnings);
        }
    }
}


