<?php
/**
 * Implementaton of the provaaccess_delaybetweenattempts plugin.
 *
 * @package    provaaccess
 * @subpackage delaybetweenattempts
 */


defined('MOODLE_INTERNAL') || die();

require_once($CFG->dirroot . '/mod/prova/accessrule/accessrulebase.php');


/**
 * A rule imposing the delay between attempts settings.
 */
class provaaccess_delaybetweenattempts extends prova_access_rule_base {

    public static function make(prova $provaobj, $timenow, $canignoretimelimits) {
        if (empty($provaobj->get_prova()->delay1) && empty($provaobj->get_prova()->delay2)) {
            return null;
        }

        return new self($provaobj, $timenow);
    }

    public function prevent_new_attempt($numprevattempts, $lastattempt) {
        if ($this->prova->attempts > 0 && $numprevattempts >= $this->prova->attempts) {
            // No more attempts allowed anyway.
            return false;
        }
        if ($this->prova->timeclose != 0 && $this->timenow > $this->prova->timeclose) {
            // No more attempts allowed anyway.
            return false;
        }
        $nextstarttime = $this->compute_next_start_time($numprevattempts, $lastattempt);
        if ($this->timenow < $nextstarttime) {
            if ($this->prova->timeclose == 0 || $nextstarttime <= $this->prova->timeclose) {
                return get_string('youmustwait', 'provaaccess_delaybetweenattempts',
                        userdate($nextstarttime));
            } else {
                return get_string('youcannotwait', 'provaaccess_delaybetweenattempts');
            }
        }
        return false;
    }

    /**
     * Compute the next time a student would be allowed to start an attempt,
     * according to this rule.
     * @param int $numprevattempts number of previous attempts.
     * @param object $lastattempt information about the previous attempt.
     * @return number the time.
     */
    protected function compute_next_start_time($numprevattempts, $lastattempt) {
        if ($numprevattempts == 0) {
            return 0;
        }

        $lastattemptfinish = $lastattempt->timefinish;
        if ($this->prova->timelimit > 0) {
            $lastattemptfinish = min($lastattemptfinish,
                    $lastattempt->timestart + $this->prova->timelimit);
        }

        if ($numprevattempts == 1 && $this->prova->delay1) {
            return $lastattemptfinish + $this->prova->delay1;
        } else if ($numprevattempts > 1 && $this->prova->delay2) {
            return $lastattemptfinish + $this->prova->delay2;
        }
        return 0;
    }

    public function is_finished($numprevattempts, $lastattempt) {
        $nextstarttime = $this->compute_next_start_time($numprevattempts, $lastattempt);
        return $this->timenow <= $nextstarttime &&
        $this->prova->timeclose != 0 && $nextstarttime >= $this->prova->timeclose;
    }
}
