<?php
/**
 * Implementaton of the provaaccess_openclosedate plugin.
 *
 * @package    provaaccess
 * @subpackage openclosedate
 */


defined('MOODLE_INTERNAL') || die();

require_once($CFG->dirroot . '/mod/prova/accessrule/accessrulebase.php');


/**
 * A rule enforcing open and close dates.
 */
class provaaccess_openclosedate extends prova_access_rule_base {

    public static function make(prova $provaobj, $timenow, $canignoretimelimits) {
        // This rule is always used, even if the prova has no open or close date.
        return new self($provaobj, $timenow);
    }

    public function description() {
        $result = array();
        if ($this->timenow < $this->prova->timeopen) {
            $result[] = get_string('provanotavailable', 'provaaccess_openclosedate',
                    userdate($this->prova->timeopen));
            if ($this->prova->timeclose) {
                $result[] = get_string('provacloseson', 'prova', userdate($this->prova->timeclose));
            }

        } else if ($this->prova->timeclose && $this->timenow > $this->prova->timeclose) {
            $result[] = get_string('provaclosed', 'prova', userdate($this->prova->timeclose));

        } else {
            if ($this->prova->timeopen) {
                $result[] = get_string('provaopenedon', 'prova', userdate($this->prova->timeopen));
            }
            if ($this->prova->timeclose) {
                $result[] = get_string('provacloseson', 'prova', userdate($this->prova->timeclose));
            }
        }

        return $result;
    }

    public function prevent_access() {
        $message = get_string('notavailable', 'provaaccess_openclosedate');

        if ($this->timenow < $this->prova->timeopen) {
            return $message;
        }

        if (!$this->prova->timeclose) {
            return false;
        }

        if ($this->timenow <= $this->prova->timeclose) {
            return false;
        }

        if ($this->prova->overduehandling != 'graceperiod') {
            return $message;
        }

        if ($this->timenow <= $this->prova->timeclose + $this->prova->graceperiod) {
            return false;
        }

        return $message;
    }

    public function is_finished($numprevattempts, $lastattempt) {
        return $this->prova->timeclose && $this->timenow > $this->prova->timeclose;
    }

    public function end_time($attempt) {
        if ($this->prova->timeclose) {
            return $this->prova->timeclose;
        }
        return false;
    }

    public function time_left_display($attempt, $timenow) {
        // If this is a teacher preview after the close date, do not show
        // the time.
        if ($attempt->preview && $timenow > $this->prova->timeclose) {
            return false;
        }
        // Otherwise, return to the time left until the close date, providing that is
        // less than PROVA_SHOW_TIME_BEFORE_DEADLINE.
        $endtime = $this->end_time($attempt);
        if ($endtime !== false && $timenow > $endtime - PROVA_SHOW_TIME_BEFORE_DEADLINE) {
            return $endtime - $timenow;
        }
        return false;
    }
}
