<?php
/**
 * Unit tests for the provaaccess_openclosedate plugin.
 *
 * @package    provaaccess
 * @subpackage openclosedate
 * @category   phpunit
 */


defined('MOODLE_INTERNAL') || die();

global $CFG;
require_once($CFG->dirroot . '/mod/prova/accessrule/openclosedate/rule.php');


/**
 * Unit tests for the provaaccess_openclosedate plugin.
 */
class provaaccess_openclosedate_testcase extends basic_testcase {
    public function test_no_dates() {
        $prova = new stdClass();
        $prova->timeopen = 0;
        $prova->timeclose = 0;
        $prova->overduehandling = 'autosubmit';
        $cm = new stdClass();
        $cm->id = 0;
        $provaobj = new prova($prova, $cm, null);
        $attempt = new stdClass();
        $attempt->preview = 0;

        $rule = new provaaccess_openclosedate($provaobj, 10000);
        $this->assertEmpty($rule->description());
        $this->assertFalse($rule->prevent_access());
        $this->assertFalse($rule->prevent_new_attempt(0, $attempt));
        $this->assertFalse($rule->is_finished(0, $attempt));
        $this->assertFalse($rule->end_time($attempt));
        $this->assertFalse($rule->time_left_display($attempt, 10000));
        $this->assertFalse($rule->time_left_display($attempt, 0));

        $rule = new provaaccess_openclosedate($provaobj, 0);
        $this->assertEmpty($rule->description());
        $this->assertFalse($rule->prevent_access());
        $this->assertFalse($rule->prevent_new_attempt(0, $attempt));
        $this->assertFalse($rule->is_finished(0, $attempt));
        $this->assertFalse($rule->end_time($attempt));
        $this->assertFalse($rule->time_left_display($attempt, 0));
    }

    public function test_start_date() {
        $prova = new stdClass();
        $prova->timeopen = 10000;
        $prova->timeclose = 0;
        $prova->overduehandling = 'autosubmit';
        $cm = new stdClass();
        $cm->id = 0;
        $provaobj = new prova($prova, $cm, null);
        $attempt = new stdClass();
        $attempt->preview = 0;

        $rule = new provaaccess_openclosedate($provaobj, 9999);
        $this->assertEquals($rule->description(),
            array(get_string('provanotavailable', 'provaaccess_openclosedate', userdate(10000))));
        $this->assertEquals($rule->prevent_access(),
            get_string('notavailable', 'provaaccess_openclosedate'));
        $this->assertFalse($rule->prevent_new_attempt(0, $attempt));
        $this->assertFalse($rule->is_finished(0, $attempt));
        $this->assertFalse($rule->end_time($attempt));
        $this->assertFalse($rule->time_left_display($attempt, 0));

        $rule = new provaaccess_openclosedate($provaobj, 10000);
        $this->assertEquals($rule->description(),
            array(get_string('provaopenedon', 'prova', userdate(10000))));
        $this->assertFalse($rule->prevent_access());
        $this->assertFalse($rule->prevent_new_attempt(0, $attempt));
        $this->assertFalse($rule->is_finished(0, $attempt));
        $this->assertFalse($rule->end_time($attempt));
        $this->assertFalse($rule->time_left_display($attempt, 0));
    }

    public function test_close_date() {
        $prova = new stdClass();
        $prova->timeopen = 0;
        $prova->timeclose = 20000;
        $prova->overduehandling = 'autosubmit';
        $cm = new stdClass();
        $cm->id = 0;
        $provaobj = new prova($prova, $cm, null);
        $attempt = new stdClass();
        $attempt->preview = 0;

        $rule = new provaaccess_openclosedate($provaobj, 20000);
        $this->assertEquals($rule->description(),
            array(get_string('provacloseson', 'prova', userdate(20000))));
        $this->assertFalse($rule->prevent_access());
        $this->assertFalse($rule->prevent_new_attempt(0, $attempt));
        $this->assertFalse($rule->is_finished(0, $attempt));

        $this->assertEquals($rule->end_time($attempt), 20000);
        $this->assertFalse($rule->time_left_display($attempt, 20000 - prova_SHOW_TIME_BEFORE_DEADLINE));
        $this->assertEquals($rule->time_left_display($attempt, 19900), 100);
        $this->assertEquals($rule->time_left_display($attempt, 20000), 0);
        $this->assertEquals($rule->time_left_display($attempt, 20100), -100);

        $rule = new provaaccess_openclosedate($provaobj, 20001);
        $this->assertEquals($rule->description(),
            array(get_string('provaclosed', 'prova', userdate(20000))));
        $this->assertEquals($rule->prevent_access(),
            get_string('notavailable', 'provaaccess_openclosedate'));
        $this->assertFalse($rule->prevent_new_attempt(0, $attempt));
        $this->assertTrue($rule->is_finished(0, $attempt));
        $this->assertEquals($rule->end_time($attempt), 20000);
        $this->assertFalse($rule->time_left_display($attempt, 20000 - prova_SHOW_TIME_BEFORE_DEADLINE));
        $this->assertEquals($rule->time_left_display($attempt, 19900), 100);
        $this->assertEquals($rule->time_left_display($attempt, 20000), 0);
        $this->assertEquals($rule->time_left_display($attempt, 20100), -100);
    }

    public function test_both_dates() {
        $prova = new stdClass();
        $prova->timeopen = 10000;
        $prova->timeclose = 20000;
        $prova->overduehandling = 'autosubmit';
        $cm = new stdClass();
        $cm->id = 0;
        $provaobj = new prova($prova, $cm, null);
        $attempt = new stdClass();
        $attempt->preview = 0;

        $rule = new provaaccess_openclosedate($provaobj, 9999);
        $this->assertEquals($rule->description(),
            array(get_string('provanotavailable', 'provaaccess_openclosedate', userdate(10000)),
                    get_string('provacloseson', 'prova', userdate(20000))));
        $this->assertEquals($rule->prevent_access(),
            get_string('notavailable', 'provaaccess_openclosedate'));
        $this->assertFalse($rule->prevent_new_attempt(0, $attempt));
        $this->assertFalse($rule->is_finished(0, $attempt));

        $rule = new provaaccess_openclosedate($provaobj, 10000);
        $this->assertEquals($rule->description(),
            array(get_string('provaopenedon', 'prova', userdate(10000)),
                get_string('provacloseson', 'prova', userdate(20000))));
        $this->assertFalse($rule->prevent_access());
        $this->assertFalse($rule->prevent_new_attempt(0, $attempt));
        $this->assertFalse($rule->is_finished(0, $attempt));

        $rule = new provaaccess_openclosedate($provaobj, 20000);
        $this->assertEquals($rule->description(),
            array(get_string('provaopenedon', 'prova', userdate(10000)),
                get_string('provacloseson', 'prova', userdate(20000))));
        $this->assertFalse($rule->prevent_access());
        $this->assertFalse($rule->prevent_new_attempt(0, $attempt));
        $this->assertFalse($rule->is_finished(0, $attempt));

        $rule = new provaaccess_openclosedate($provaobj, 20001);
        $this->assertEquals($rule->description(),
            array(get_string('provaclosed', 'prova', userdate(20000))));
        $this->assertEquals($rule->prevent_access(),
            get_string('notavailable', 'provaaccess_openclosedate'));
        $this->assertFalse($rule->prevent_new_attempt(0, $attempt));
        $this->assertTrue($rule->is_finished(0, $attempt));

        $this->assertEquals($rule->end_time($attempt), 20000);
        $this->assertFalse($rule->time_left_display($attempt, 20000 - prova_SHOW_TIME_BEFORE_DEADLINE));
        $this->assertEquals($rule->time_left_display($attempt, 19900), 100);
        $this->assertEquals($rule->time_left_display($attempt, 20000), 0);
        $this->assertEquals($rule->time_left_display($attempt, 20100), -100);
    }

    public function test_close_date_with_overdue() {
        $prova = new stdClass();
        $prova->timeopen = 0;
        $prova->timeclose = 20000;
        $prova->overduehandling = 'graceperiod';
        $prova->graceperiod = 1000;
        $cm = new stdClass();
        $cm->id = 0;
        $provaobj = new prova($prova, $cm, null);
        $attempt = new stdClass();
        $attempt->preview = 0;

        $rule = new provaaccess_openclosedate($provaobj, 20000);
        $this->assertFalse($rule->prevent_access());

        $rule = new provaaccess_openclosedate($provaobj, 20001);
        $this->assertFalse($rule->prevent_access());

        $rule = new provaaccess_openclosedate($provaobj, 21000);
        $this->assertFalse($rule->prevent_access());

        $rule = new provaaccess_openclosedate($provaobj, 21001);
        $this->assertEquals($rule->prevent_access(),
                get_string('notavailable', 'provaaccess_openclosedate'));
    }
}
