<?php
/**
 * Implementaton of the provaaccess_password plugin.
 *
 * @package    provaaccess
 * @subpackage password
 */


defined('MOODLE_INTERNAL') || die();

require_once($CFG->dirroot . '/mod/prova/accessrule/accessrulebase.php');


/**
 * A rule implementing the password check.
 */
class provaaccess_password extends prova_access_rule_base {

    public static function make(prova $provaobj, $timenow, $canignoretimelimits) {
        if (empty($provaobj->get_prova()->password)) {
            return null;
        }

        return new self($provaobj, $timenow);
    }

    public function description() {
        return get_string('requirepasswordmessage', 'provaaccess_password');
    }

    public function is_preflight_check_required($attemptid) {
        global $SESSION;
        return empty($SESSION->passwordcheckedprovas[$this->prova->id]);
    }

    public function add_preflight_check_form_fields(mod_prova_preflight_check_form $provaform,
            MoodleQuickForm $mform, $attemptid) {

        $mform->addElement('header', 'passwordheader', get_string('password'));
        $mform->addElement('static', 'passwordmessage', '',
                get_string('requirepasswordmessage', 'provaaccess_password'));

        // Don't use the 'proper' field name of 'password' since that get's
        // Firefox's password auto-complete over-excited.
        $mform->addElement('password', 'provapassword',
                get_string('provapassword', 'provaaccess_password'), array('autofocus' => 'true'));
    }

    public function validate_preflight_check($data, $files, $errors, $attemptid) {

        $enteredpassword = $data['provapassword'];
        if (strcmp($this->prova->password, $enteredpassword) === 0) {
            return $errors; // Password is OK.

        } else if (isset($this->prova->extrapasswords)) {
            // Group overrides may have additional passwords.
            foreach ($this->prova->extrapasswords as $password) {
                if (strcmp($password, $enteredpassword) === 0) {
                    return $errors; // Password is OK.
                }
            }
        }

        $errors['provapassword'] = get_string('passworderror', 'provaaccess_password');
        return $errors;
    }

    public function notify_preflight_check_passed($attemptid) {
        global $SESSION;
        $SESSION->passwordcheckedprovas[$this->prova->id] = true;
    }

    public function current_attempt_finished() {
        global $SESSION;
        // Clear the flag in the session that says that the user has already
        // entered the password for this prova.
        if (!empty($SESSION->passwordcheckedprovas[$this->prova->id])) {
            unset($SESSION->passwordcheckedprovas[$this->prova->id]);
        }
    }
}
