<?php
/**
 * Implementaton of the provaaccess_safebrowser plugin.
 *
 * @package    provaaccess
 * @subpackage safebrowser
 */


defined('MOODLE_INTERNAL') || die();

require_once($CFG->dirroot . '/mod/prova/accessrule/accessrulebase.php');


/**
 * A rule representing the safe browser check.
 */
class provaaccess_safebrowser extends prova_access_rule_base {

    public static function make(prova $provaobj, $timenow, $canignoretimelimits) {

        // TODO: testar como funciona o safebrowser
        if ($provaobj->get_prova()->browsersecurity !== 'safebrowser') {
            return null;
        }

        return new self($provaobj, $timenow);
    }

    public function prevent_access() {
        if (!$this->check_safe_browser()) {
            return get_string('safebrowsererror', 'provaaccess_safebrowser');
        } else {
            return false;
        }
    }

    public function description() {
        return get_string('safebrowsernotice', 'provaaccess_safebrowser');
    }

    public function setup_attempt_page($page) {
        $page->set_title($this->provaobj->get_course()->shortname . ': ' . $page->title);
        $page->set_cacheable(false);
        $page->set_popup_notification_allowed(false); // Prevent message notifications.
        $page->set_heading($page->title);
        $page->set_pagelayout('secure');
    }

    /**
     * Checks if browser is safe browser
     *
     * @return true, if browser is safe browser else false
     */
    public function check_safe_browser() {
        return strpos($_SERVER['HTTP_USER_AGENT'], 'SEB') !== false;
    }

    /**
     * @return array key => lang string any choices to add to the prova Browser
     *      security settings menu.
     */
    public static function get_browser_security_choices() {
        global $CFG;

        if (empty($CFG->enablesafebrowserintegration)) {
            return array();
        }

        return array('safebrowser' =>
                get_string('requiresafeexambrowser', 'provaaccess_safebrowser'));
    }
}
