<?php
/**
 * Implementaton of the provaaccess_securewindow plugin.
 *
 * @package    provaaccess
 * @subpackage securewindow
 */


defined('MOODLE_INTERNAL') || die();

require_once($CFG->dirroot . '/mod/prova/accessrule/accessrulebase.php');


/**
 * A rule for ensuring that the prova is opened in a popup, with some JavaScript
 * to prevent copying and pasting, etc.
 */
class provaaccess_securewindow extends prova_access_rule_base {
    /** @var array options that should be used for opening the secure popup. */
    protected static $popupoptions = array(
        'left' => 0,
        'top' => 0,
        'fullscreen' => true,
        'scrollbars' => true,
        'resizeable' => false,
        'directories' => false,
        'toolbar' => false,
        'titlebar' => false,
        'location' => false,
        'status' => false,
        'menubar' => false,
    );

    public static function make(prova $provaobj, $timenow, $canignoretimelimits) {

        if ($provaobj->get_prova()->browsersecurity !== 'securewindow') {
            return null;
        }

        return new self($provaobj, $timenow);
    }

    public function attempt_must_be_in_popup() {
        return !$this->provaobj->is_preview_user();
    }

    public function get_popup_options() {
        return self::$popupoptions;
    }

    public function setup_attempt_page($page) {
        $page->set_popup_notification_allowed(false); // Prevent message notifications.
        $page->set_title($this->provaobj->get_course()->shortname . ': ' . $page->title);
        $page->set_cacheable(false);
        $page->set_pagelayout('secure');

        if ($this->provaobj->is_preview_user()) {
            return;
        }

        $page->add_body_class('prova-secure-window');
        $page->requires->js_init_call('M.mod_prova.secure_window.init',
                null, false, prova_get_js_module());
    }

    /**
     * @return array key => lang string any choices to add to the prova Browser
     *      security settings menu.
     */
    public static function get_browser_security_choices() {
        return array('securewindow' =>
                get_string('popupwithjavascriptsupport', 'provaaccess_securewindow'));
    }
}
