<?php
/**
 * Implementaton of the provaaccess_timelimit plugin.
 *
 * @package    provaaccess
 * @subpackage timelimit
 */


defined('MOODLE_INTERNAL') || die();

require_once($CFG->dirroot . '/mod/prova/accessrule/accessrulebase.php');


/**
 * A rule representing the time limit. It does not actually restrict access, but we use this
 * class to encapsulate some of the relevant code.
 */
class provaaccess_timelimit extends prova_access_rule_base {

    public static function make(prova $provaobj, $timenow, $canignoretimelimits) {

        if (empty($provaobj->get_prova()->timelimit) || $canignoretimelimits) {
            return null;
        }

        return new self($provaobj, $timenow);
    }

    public function description() {
        return get_string('provatimelimit', 'provaaccess_timelimit',
                format_time($this->prova->timelimit));
    }

    public function end_time($attempt) {
        $timedue = $attempt->timestart + $this->prova->timelimit;
        if ($this->prova->timeclose) {
            $timedue = min($timedue, $this->prova->timeclose);
        }
        return $timedue;
    }

    public function time_left_display($attempt, $timenow) {
        // If this is a teacher preview after the time limit expires, don't show the time_left
        $endtime = $this->end_time($attempt);
        if ($attempt->preview && $timenow > $endtime) {
            return false;
        }
        return $endtime - $timenow;
    }

    public function is_preflight_check_required($attemptid) {
        // Warning only required if the attempt is not already started.
        return $attemptid === null;
    }

    public function add_preflight_check_form_fields(mod_prova_preflight_check_form $provaform,
            MoodleQuickForm $mform, $attemptid) {
        $mform->addElement('header', 'honestycheckheader',
                get_string('confirmstartheader', 'provaaccess_timelimit'));
        $mform->addElement('static', 'honestycheckmessage', '',
                get_string('confirmstart', 'provaaccess_timelimit', format_time($this->prova->timelimit)));
    }
}
