<?php
/**
 * Unit tests for the provaaccess_timelimit plugin.
 *
 * @package    provaaccess
 * @subpackage timelimit
 */


defined('MOODLE_INTERNAL') || die();

global $CFG;
require_once($CFG->dirroot . '/mod/prova/accessrule/timelimit/rule.php');


/**
 * Unit tests for the provaaccess_timelimit plugin.
 */
class provaaccess_timelimit_testcase extends basic_testcase {
    public function test_time_limit_access_rule() {
        $prova = new stdClass();
        $prova->timeclose = 0;
        $prova->timelimit = 3600;
        $cm = new stdClass();
        $cm->id = 0;
        $provaobj = new prova($prova, $cm, null);
        $rule = new provaaccess_timelimit($provaobj, 10000);
        $attempt = new stdClass();

        $this->assertEquals($rule->description(),
            get_string('provatimelimit', 'provaaccess_timelimit', format_time(3600)));

        $attempt->timestart = 10000;
        $attempt->preview = 0;
        $this->assertEquals(13600, $rule->end_time($attempt));
        $this->assertEquals(3600, $rule->time_left_display($attempt, 10000));
        $this->assertEquals(1600, $rule->time_left_display($attempt, 12000));
        $this->assertEquals(-400, $rule->time_left_display($attempt, 14000));

        $this->assertFalse($rule->prevent_access());
        $this->assertFalse($rule->prevent_new_attempt(0, $attempt));
        $this->assertFalse($rule->is_finished(0, $attempt));
    }

    /**
     * Data provider for test_time_limit_access_rule_with_time_close.
     *
     * @return array of ($timetoclose, $timelimit, $displaylimit, $actuallimit)
     */
    public function time_limit_access_rule_with_time_close_provider() {
        return [
            'Close time is earlier than time limit' => [1800, 3600, 3600, 1800],
            'Close time is on time limit' => [3600, 3600, 3600, 3600],
            'Close time is later than time limit' => [3600, 1800, 1800, 1800]
        ];
    }

    /**
     * Test the time_left_display method of the provaaccess_timelimit class.
     *
     * @param int $timetoclose  The number of seconds that is left to the prova' closing time
     * @param int $timelimit    Time limit of the prova
     * @param int $displaylimit The limit that is displayed on the prova page
     * @param int $actuallimit  The actual limit that is being applied
     * @dataProvider time_limit_access_rule_with_time_close_provider
     */
    public function test_time_limit_access_rule_with_time_close($timetoclose, $timelimit, $displaylimit, $actuallimit) {
        $timenow = 10000;

        $prova = new stdClass();
        $prova->timeclose = $timenow + $timetoclose;
        $prova->timelimit = $timelimit;
        $cm = new stdClass();
        $cm->id = 0;
        $provaobj = new prova($prova, $cm, null);
        $rule = new provaaccess_timelimit($provaobj, $timenow);
        $attempt = new stdClass();

        $this->assertEquals($rule->description(),
            get_string('provatimelimit', 'provaaccess_timelimit', format_time($displaylimit)));

        $attempt->timestart = $timenow;
        $attempt->preview = 0;
        $this->assertEquals($timenow + $actuallimit, $rule->end_time($attempt));
        $this->assertEquals($actuallimit, $rule->time_left_display($attempt, $timenow));
        $this->assertEquals($actuallimit - 1000, $rule->time_left_display($attempt, $timenow + 1000));

        $this->assertFalse($rule->prevent_access());
        $this->assertFalse($rule->prevent_new_attempt(0, $attempt));
        $this->assertFalse($rule->is_finished(0, $attempt));
    }
}
