<?php
/**
 * The mod_prova attempt reviewed event.
 *
 * @package mod_prova
 */
namespace mod_prova\event;

defined('MOODLE_INTERNAL') || die();

/**
 * The mod_prova attempt reviewed event class.
 *
 * @property-read array $other {
 *      Extra information about event.
 *
 *      - int provaid: the id of the prova.
 * }
 *
 * @package mod_prova
 */
class attempt_reviewed extends \core\event\base {

    /**
     * Init method.
     */
    protected function init() {
        $this->data['objecttable'] = 'prova_attempts';
        $this->data['crud'] = 'r';
        $this->data['edulevel'] = self::LEVEL_TEACHING;
    }

    /**
     * Returns localised general event name.
     *
     * @return string
     */
    public static function get_name() {
        return get_string('eventattemptreviewed', 'mod_prova');
    }

    /**
     * Returns description of what happened.
     *
     * @return string
     */
    public function get_description() {
        return "The user with id '$this->userid' has had their attempt with id '$this->objectid' reviewed by the user " .
            "with id '$this->relateduserid' for the prova with course module id '$this->contextinstanceid'.";
    }

    /**
     * Returns relevant URL.
     *
     * @return \moodle_url
     */
    public function get_url() {
        return new \moodle_url('/mod/prova/screens/attempt/review.php', array('attempt' => $this->objectid));
    }

    /**
     * Return the legacy event log data.
     *
     * @return array
     */
    protected function get_legacy_logdata() {
        return array($this->courseid, 'prova', 'review', 'review.php?attempt=' . $this->objectid,
            $this->other['provaid'], $this->contextinstanceid);
    }

    /**
     * Custom validation.
     *
     * @throws \coding_exception
     * @return void
     */
    protected function validate_data() {
        parent::validate_data();

        if (!isset($this->relateduserid)) {
            throw new \coding_exception('The \'relateduserid\' must be set.');
        }

        if (!isset($this->other['provaid'])) {
            throw new \coding_exception('The \'provaid\' value must be set in other.');
        }
    }

    public static function get_objectid_mapping() {
        return array('db' => 'prova_attempts', 'restore' => 'prova_attempt');
    }

    public static function get_other_mapping() {
        $othermapped = array();
        $othermapped['provaid'] = array('db' => 'prova', 'restore' => 'prova');

        return $othermapped;
    }
}
