<?php
/**
 * The mod_prova question manually graded event.
 *
 * @package    core
 * @copyright  2014 Mark Nelson <markn@moodle.com>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
namespace mod_prova\event;

defined('MOODLE_INTERNAL') || die();

/**
 * The mod_prova question manually graded event class.
 *
 * @property-read array $other {
 *      Extra information about event.
 *
 *      - int provaid: the id of the prova.
 *      - int attemptid: the id of the attempt.
 *      - int slot: the question number in the attempt.
 * }
 *
 * @package    core
 * @since      Moodle 2.7
 * @copyright  2014 Mark Nelson <markn@moodle.com>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class question_manually_graded extends \core\event\base {

    /**
     * Init method.
     */
    protected function init() {
        $this->data['objecttable'] = 'question';
        $this->data['crud'] = 'c';
        $this->data['edulevel'] = self::LEVEL_TEACHING;
    }

    /**
     * Returns localised general event name.
     *
     * @return string
     */
    public static function get_name() {
        return get_string('eventquestionmanuallygraded', 'mod_prova');
    }

    /**
     * Returns description of what happened.
     *
     * @return string
     */
    public function get_description() {
        return "The user with id '$this->userid' manually graded the question with id '$this->objectid' for the attempt " .
            "with id '{$this->other['attemptid']}' for the prova with course module id '$this->contextinstanceid'.";
    }

    /**
     * Returns relevant URL.
     *
     * @return \moodle_url
     */
    public function get_url() {
        return new \moodle_url('/mod/prova/screens/report/comment.php', array('attempt' => $this->other['attemptid'],
            'slot' => $this->other['slot']));
    }

    /**
     * Return the legacy event log data.
     *
     * @return array
     */
    protected function get_legacy_logdata() {
        return array($this->courseid, 'prova', 'manualgrade', 'comment.php?attempt=' . $this->other['attemptid'] .
            '&slot=' . $this->other['slot'], $this->other['provaid'], $this->contextinstanceid);
    }

    /**
     * Custom validation.
     *
     * @throws \coding_exception
     * @return void
     */
    protected function validate_data() {
        parent::validate_data();

        if (!isset($this->other['provaid'])) {
            throw new \coding_exception('The \'provaid\' value must be set in other.');
        }

        if (!isset($this->other['attemptid'])) {
            throw new \coding_exception('The \'attemptid\' value must be set in other.');
        }

        if (!isset($this->other['slot'])) {
            throw new \coding_exception('The \'slot\' value must be set in other.');
        }
    }

    public static function get_objectid_mapping() {
        return array('db' => 'question', 'restore' => 'question');
    }

    public static function get_other_mapping() {
        $othermapped = array();
        $othermapped['provaid'] = array('db' => 'prova', 'restore' => 'prova');
        $othermapped['attemptid'] = array('db' => 'prova_attempts', 'restore' => 'prova_attempt');

        return $othermapped;
    }
}
