<?php
/**
 * Renderer outputting the quiz editing UI.
 *
 * @package mod_prova
 */

namespace mod_prova\output;

defined('MOODLE_INTERNAL') || die();

use \html_writer;

/**
 * Renderer outputting the quiz editing UI.
 *
 * @copyright 2013 The Open University.
 * @license http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 * @since Moodle 2.7
 */
class mod_form_renderer extends \plugin_renderer_base {

	/**
	 * Adiciona botoes de adiconar e buscar ao dropdown
	 */
	public function seox_edit_menu_actions() {
		global $PAGE;

		static $str;
		if (!isset($str)) {
			$str = get_strings(array('addprova', 'buscar_prova'), 'prova');
		}

		$returnurl = $PAGE->url;
		$params = ['returnurl' => $returnurl];

		$actions['addprova'] = new \action_menu_link_secondary(
			new \moodle_url('/mod/prova/components/addprovas/addprova.php', $params),
			new \pix_icon('t/add', $str->addprova, 'moodle', array('class' => 'iconsmall', 'title' => '')),
			$str->addprova, array('data-id' => 'seox_add_prova', 'class' => 'cm-edit-action addquestion', 'data-action' => 'addquestion')
		);

		$actions['buscarprova'] = new \action_menu_link_secondary(
			new \moodle_url('/mod/prova/components/addquestions/addquestions.php', $params),
			new \pix_icon('a/search', $str->buscar_prova, 'moodle', array('class' => 'iconsmall', 'title' => '')),
			$str->buscar_prova, array('data-id' => 'seox_busca_prova', 'class' => 'cm-edit-action buscar_prova', 'data-action' => 'buscar_prova')
		);

		$PAGE->requires->css('/mod/prova/provabank.css');

		// Global css hides the last element, so I created a fake action
		$actions['fakeaction'] = new \action_menu_link_secondary(new \moodle_url('my'), null, '', array('style' => 'display:none;'));
		return $actions;
	}

	public function seox_add_menu_actions() {

		$actions = $this->seox_edit_menu_actions();
		if (empty($actions)) {
			return '';
		}
		$menu = new \action_menu();
		$menu->set_alignment(\action_menu::TR, \action_menu::TR);
		$menu->set_constraint('.mod-quiz-edit-content');
		$trigger = html_writer::tag('span', get_string('add', 'prova'), array('class' => 'add-menu'));
		$menu->set_menu_trigger($trigger);
		// The menu appears within an absolutely positioned element causing width problems.
		// Make sure no-wrap is set so that we don't get a squashed menu.
		$menu->set_nowrap_on_items(true);

		// Disable the link if quiz has attempts.
		/* if (!$structure->can_be_edited()) {
	        return $this->render($menu);
*/

		foreach ($actions as $action) {
			if ($action instanceof \action_menu_link) {
				$action->add_class('add-menu');
			}
			$menu->add($action);
		}

		$menu->attributes['class'] .= 'page-add-actions commands';

		// Prioritise the menu ahead of all other actions.
		$menu->prioritise = true;

		return $this->render($menu);
	}

}
