<?php
/**
 * A {@link qubaid_condition} representing all the attempts by one user at a given prova.
 *
 * @package   mod_prova
 * @category  question
 */

namespace mod_prova\question;
defined('MOODLE_INTERNAL') || die();


/**
 * A {@link qubaid_condition} representing all the attempts by one user at a given prova.
 */
class qubaids_for_users_attempts extends \qubaid_join {
    /**
     * Constructor.
     *
     * This takes the same arguments as {@link prova_get_user_attempts()}.
     *
     * @param int $provaid the prova id.
     * @param int $userid the userid.
     * @param string $status 'all', 'finished' or 'unfinished' to control
     * @param bool $includepreviews defaults to false.
     */
    public function __construct($provaid, $userid, $status = 'finished', $includepreviews = false) {
        $where = 'provaa.prova = :provaavalue AND provaa.userid = :userid';
        $params = array('provaavalue' => $provaid, 'userid' => $userid);

        if (!$includepreviews) {
            $where .= ' AND preview = 0';
        }

        switch ($status) {
            case 'all':
                break;

            case 'finished':
                $where .= ' AND state IN (:state1, :state2)';
                $params['state1'] = \prova_attempt::FINISHED;
                $params['state2'] = \prova_attempt::ABANDONED;
                break;

            case 'unfinished':
                $where .= ' AND state IN (:state1, :state2)';
                $params['state1'] = \prova_attempt::IN_PROGRESS;
                $params['state2'] = \prova_attempt::OVERDUE;
                break;
        }

        parent::__construct('{prova_attempts} provaa', 'provaa.uniqueid', $where, $params);
    }
}
