<?php
/**
 * Classe que comporta funções clonadas/alteradas e funções de
 * escopo glonal essenciais à exeução da funcionalidade 
 * "Criar provas"
 */
namespace provacomp_addprovas\action;
defined('MOODLE_INTERNAL') || die();

require_once($CFG->dirroot . '/question/editlib.php');

use context_course;
use moodle_url;

class addprovas_actions {
    /**
     * Gera parametros necessários para criar uma prova
     *
     * @return os parâmetros necessários
     */
    static function generate_addprova_params($customCat = false) {
        global $DB, $CFG;
        require_once($CFG->dirroot . '/mod/prova/managelib.php');
        $returnurl = '/mod/prova/components/addprovas/edit.php';
        $courseid = SEOX_COURSE_ID;
        $contextid = context_course::instance(SEOX_COURSE_ID)->id;
        if (!$category = $DB->get_record('question_categories', array('name' => 'top', 'contextid' => $contextid))) {
            print_error('error_getquestioncategory', 'provacomp_addprovas');
        }
        $categoryid = $category->id;
        $params = array(
            'returnurl' => $returnurl,
            'courseid'  => $courseid,
            'category'  => $categoryid,
        );
        if ($customCat) {
            $params['category'] = $categoryid . ', ' . $contextid;
        }

        return $params;
    }

    /** 
     * Gera query customizada com base na busca feita pelo
     * usuário.
     *
     * @param search \Array lista de filtros da questão 
     * @return query \String 
     */
    static function build_search_query($search) {
        $data = array();

        if ($search['md_areas'] != 0) {
            $data[] = 'seox.id_area = '.$search['md_areas'];
        }
        if ($search['md_subareas'] != 0) {
            $data[] = 'seox.id_subarea = '.$search['md_subareas'];
        }
        if ($search['md_assuntos'] != 0) {
            $data[] = 'seox.id_assunto = '.$search['md_assuntos'];
        }
        if (!empty($search['md_dificuldades'])) {
            $str = 'seox.dificuldade IN (';
            $str .= implode(',', $search['md_dificuldades']);
            $str .= ')';
            $data[] = $str;
        }
        if (!empty($search['md_qtypes'])) {
            $str = "q.qtype IN ('";
            $str .= implode("','", $search['md_qtypes']);
            $str .= "')";
            $data[] =  $str;
        }
        if ($search['md_text'] and $search['md_text'] != '') {
            $data[] = "q.questiontext LIKE '%".$search['md_text']."%'";
        }

        return $data;
    } 
}