<?php 
/**
 * Classe baseada no renderizador do modulo Quiz
 * Original Source: /mod/quiz/classes/output/edit_renderer.php
 *
 * Objetivo é carregar somente omodulos específicos, 
 * sobreescrevendo alguns métodos específicos.
 */
namespace provacomp_addprovas\output;
defined('MOODLE_INTERNAL') || die();

use \provacomp_addprovas\structure;
use \html_writer;
use renderable;

class edit_renderer extends \plugin_renderer_base {

    /** @var \structure objeto modelo para listagem de questoes de uma prova */
    protected $structure = null;

    /** @var \moodle_url atual url da página */
    protected $currenturl = null;

    /**
     * @param currenturl \moodle_url
     */
    public function set_currenturl($currenturl) {
        $this->currenturl = $currenturl;
    }

    public function edit_page(\ProvaObj $provaobj, structure $structure,
            \context_course $contexts, \moodle_url $pageurl, array $pagevars) {

        $this->structure = $structure;

        $contexts = new \question_edit_contexts($contexts);

        $output = '';

        // Page title.
        $output .= $this->heading_with_help(
            get_string('editingprovax', 'provacomp_addprovas', format_string($provaobj->get_data()->name)), 
            '' // TODO: registrar descrição de ajuda
        );
        // TODO: Change code from above to bellow
        // $output .= $this->heading_with_help(get_string('editingprovax', 'prova',
        //         format_string($provaobj->get_prova_name())), 'editingprova', 'prova', '',
        //         get_string('basicideasofprova', 'prova'), 2);

        // Information at the top.
        // $output .= $this->prova_state_warnings($structure);

        $output .= html_writer::start_div('mod_prova-edit-top-controls');
        $output .= $this->prova_information($structure);

        // main_headerinfos: informações do valor total da prova e sua dificuldade
        $output .= html_writer::start_div('main_headerinfos');
        $output .= html_writer::start_div('sub_headerinfos');
        if ($structure->is_not_empty()) {
            $output .= $this->difficulty_sommatory($structure);
        }
        $output .= $this->total_marks($structure->get_prova());
        // $output .= $this->maximum_grade_input($structure, $pageurl);
        $output .= html_writer::end_div();
        $output .= html_writer::end_div();

        $output .= html_writer::start_div('mod_prova-edit-action-buttons btn-group edit-toolbar', ['role' => 'group']);
        $output .= $this->selectmultiple_button($structure);
        $output .= html_writer::end_tag('div');


        $output .= $this->selectmultiple_controls($structure);
        $output .= html_writer::end_tag('div');

        // Show the questions organised into sections and pages.
        $output .= html_writer::start_tag('ul', array('class' => 'slots'));
        $output .= $this->start_section($structure);
        $output .= $this->questions_in_section($structure, $contexts, $pagevars, $pageurl);

        // Last section
        $output .= \html_writer::start_div('last-add-menu');
        $output .= html_writer::tag('span', $this->add_menu_actions($structure, 0, $pageurl, null, $pagevars), array('class' => 'add-menu-outer'));
        $output .= \html_writer::end_div();

        // $output .= $this->end_section();
        $output .= html_writer::end_tag('ul');

        // Initialise the JavaScript.
        $this->initialise_editing_javascript($structure, $contexts, $pagevars, $pageurl);

        if ($structure->can_be_edited()) {
            $thiscontext = $contexts->lowest();
            // TODO: sobreescrever
            $this->page->requires->js_call_amd('provacomp_addprovas/provaquestionbank', 'init', [
                $thiscontext->id
            ]);

            $this->page->requires->js_call_amd('provacomp_addprovas/add_random_question', 'init', [
                $thiscontext->id
            ]);

            // Include the question chooser.
            $output .= $this->question_chooser();
        }

        return $output;
    }

    /**
     * Function from original edit_renderer.php
     */
    public function prova_information(structure $structure) {
        $output = html_writer::span(
                    get_string('numquestionsx', 'provacomp_addprovas', $structure->get_question_count()),
                    'numberofquestions');

        return html_writer::div($output, 'statusbar');
    }

    /**
     * Generate the controls that appear when the bulk action button is pressed.
     *
     * @param structure $structure the structure of the prova being edited.
     * @return string HTML to output.
     */
    protected function selectmultiple_controls(structure $structure) {
        $output = '';

        // Bulk action button delete and bulk action button cancel.
        $buttondeleteoptions = array(
            'type' => 'button',
            'id' => 'selectmultipledeletecommand',
            'value' => get_string('deleteselected', 'mod_prova'),
            'class' => 'btn btn-secondary'
        );
        $buttoncanceloptions = array(
            'type' => 'button',
            'id' => 'selectmultiplecancelcommand',
            'value' => get_string('cancel', 'moodle'),
            'class' => 'btn btn-secondary'
        );

        $groupoptions = array(
            'class' => 'btn-group selectmultiplecommand actions',
            'role' => 'group'
        );

        $output .= html_writer::tag('div',
                        html_writer::tag('button', get_string('deleteselected', 'mod_prova'), $buttondeleteoptions) .
                        " " .
                        html_writer::tag('button', get_string('cancel', 'moodle'),
                $buttoncanceloptions), $groupoptions);

        $toolbaroptions = array(
            'class' => 'btn-toolbar',
            'role' => 'toolbar',
            'aria-label' => get_string('selectmultipletoolbar', 'mod_prova'),
        );

        // Select all/deselect all questions.
        $buttonselectalloptions = array(
            'role' => 'button',
            'id' => 'questionselectall',
            'class' => 'btn btn-link'
        );
        $buttondeselectalloptions = array(
            'role' => 'button',
            'id' => 'questiondeselectall',
            'class' => 'btn btn-link'
        );
        $output .= html_writer::tag('div',
                html_writer::tag('div',
                        html_writer::link('#', get_string('selectall', 'mod_prova'), $buttonselectalloptions) .
                        html_writer::tag('span', "/", ['class' => 'separator']) .
                        html_writer::link('#', get_string('selectnone', 'mod_prova'), $buttondeselectalloptions),
                        array('class' => 'btn-group selectmultiplecommandbuttons')),
                $toolbaroptions);
        return $output;
    }

    /**
     * Function from original edit_renderer.php
     */
    public function total_marks($prova) {
        $totallabel = html_writer::label(get_string('title_valortotalprova', 'provacomp_addprovas'), '');
        $totalmark = html_writer::span(prova_format_grade($prova, $prova->sumgrades), 'seox_prova_summarks');
        return html_writer::tag('span',
                $totallabel . $totalmark,
                array('class' => 'seox_totalpoints'));
    }

    /**
     * Imprime na tela a dificuldade da prova
     */
    public function difficulty_sommatory($structure) {

        list($class, $text) = generate_output_to_difficulty($structure->get_provadifficulty());
        
        $output = '';
        $output .= html_writer::start_div('difficulty_maincontainer');

        $output .= html_writer::tag('label', get_string('title_dificuldadeprova', 'provacomp_addprovas'), array('for' => 'cell'));

        $output .= html_writer::start_div('provadifficulty_div containerdifficulty ' . $class);

        $output .= html_writer::span($text, 'cell provadifficulty', array('title' => $text));

        $output .= html_writer::end_div();
        $output .= html_writer::end_div();
        return $output;
    }

    /**
     * Function from original edit_renderer.php
     */
    protected function start_section($structure) {

        $output = '';

        $sectionstyle = '';
        if ($structure->is_only_one_slot_in_section()) {
            $sectionstyle = ' only-has-one-slot';
        }

        $output .= html_writer::start_tag('li', array(
            'class' => 'section main clearfix'.$sectionstyle, 'role' => 'region',
            'aria-label' => ''));

        $output .= html_writer::start_div('content');

        return $output;
    }

    /**
     * Function from original edit_renderer.php
     */
    public function questions_in_section(structure $structure,
            $contexts, $pagevars, $pageurl) {

        $output = '';
        foreach ($structure->get_slots_order() as $slot) {
            $output .= $this->question_row($structure, $slot, $pagevars, $pageurl);
        }
        return html_writer::tag('ul', $output, array('class' => 'section img-text'));
    }

    /**
     * Function from original edit_renderer.php
     * Gera a linha de cada questão
     */
    public function question_row(structure $structure, $slot, $pagevars, $pageurl) {
        $output = '';

        $output .= $this->page_row($structure, $slot, null, $pagevars, $pageurl);

        // Page split/join icon.
        $joinhtml = '';

        // Question HTML.
        $questionhtml = $this->question($structure, $slot, $pageurl);

        $qtype = $structure->get_question_type_for_slot($slot);
        $questionclasses = 'activity ' . $qtype . ' qtype_' . $qtype . ' slot';

        $output .= html_writer::tag('li', $questionhtml . $joinhtml,
                array('class' => $questionclasses, 'id' => 'slot-' . $structure->get_slot_id_for_slot($slot)));

        return $output;
    }

    /**
     * Function from original edit_renderer.php
     * Prepara estrutura de páginas
     */
    public function page_row(structure $structure, $slot, $contexts, $pagevars, $pageurl) {
        $output = '';

        // Put page in a heading for accessibility and styling.
        if ($structure->is_first_slot_on_page($slot)) {
            // Add the add-menu at the page level.
            $pagenumber = $structure->get_page_number_for_slot($slot);
            $addmenu = html_writer::tag('span', $this->add_menu_actions($structure,
                    $pagenumber, $pageurl, $contexts, $pagevars),
                    array('class' => 'add-menu-outer'));


            $output .= html_writer::tag('li', $addmenu /* . $addquestionform*/,
                    array(
                        'class' => 'pagenumber activity yui3-dd-drop page', 'id' => 'page-' . $pagenumber,
                        'style' => 'min-height: 25px'
                    ));
        }

        return $output;
    }

    /**
     * Generate the bulk action button.
     *
     * @param structure $structure the structure of the prova being edited.
     * @return string HTML to output.
     */
    protected function selectmultiple_button(structure $structure) {
        if (!$structure->can_be_edited()) { return ''; }
        $buttonoptions = array(
            'type'  => 'button',
            'name'  => 'selectmultiple',
            'id'    => 'selectmultiplecommand',
            'value' => get_string('selectmultipleitems', 'prova'),
            'class' => 'btn btn-secondary mb-1'
        );
        if (!$structure->can_be_edited()) {
            $buttonoptions['disabled'] = 'disabled';
        }

        return html_writer::tag('button', get_string('selectmultipleitems', 'prova'), $buttonoptions);
    }

    /**
     * Function from original edit_renderer.php
     * Func_1: gera o corpo da linha de cada questão 
     */
    public function question(structure $structure, $slot, \moodle_url $pageurl) {
        $output = '';
        $output .= html_writer::start_tag('div');

        if ($structure->can_be_edited()) {
            $output .= $this->question_move_icon($structure, $slot);
        }

        $output .= html_writer::start_div('mod-indent-outer');
        $output .= html_writer::tag('input', '', array('id' => 'selectquestion-' .
                $structure->get_displayed_number_for_slot($slot), 'name' => 'selectquestion[]',
               'type' => 'checkbox', 'class' => 'select-multiple-checkbox',
               'value' => $structure->get_displayed_number_for_slot($slot)));
        $output .= $this->question_number($structure->get_displayed_number_for_slot($slot));

        // This div is used to indent the content.
        $output .= html_writer::div('', 'mod-indent');

        // Display the link to the question (or do nothing if question has no url).
        if ($structure->get_question_type_for_slot($slot) == 'random') {
            $questionname = $this->random_question($structure, $slot, $pageurl);
        } else {
            $questionname = $this->question_name($structure, $slot, $pageurl);
        }

        // Start the div for the activity title, excluding the edit icons.
        $output .= html_writer::start_div('activityinstance');
        $output .= $questionname;

        // Closing the tag which contains everything but edit icons. Content part of the module should not be part of this.
        $output .= html_writer::end_tag('div'); // .activityinstance.

        // Action icons.
        $questionicons = '';
        $questionicons .= $this->gera_question_preview($structure->get_question_in_slot($slot)->id, $structure->get_courseid());
        if ($structure->can_be_edited()) {
            $questionicons .= $this->question_remove_icon($structure, $slot, $pageurl);
        }
        $questionicons .= $this->marked_out_of_field($structure, $slot);
        $questionicons .= $this->display_difficulty_markers($structure, $slot);
        $output .= html_writer::span($questionicons, 'actions'); // Required to add js spinner icon.

        // End of indentation div.
        $output .= html_writer::end_tag('div');
        $output .= html_writer::end_tag('div');

        return $output;
    }

    /**
     * Function from original edit_renderer.php
     * Função editada para imprimir informações 
     */
    public function question_name(structure $structure, $slot, $pageurl) {
        global $CFG;
        // require necessário para utilização da função prova_question_tostring
        require_once($CFG->dirroot . '/mod/prova/locallib.php');

        $output = '';

        $question = $structure->get_question_in_slot($slot);
        $returnurl = new \moodle_url($pageurl->out_as_local_url(), array(
            'provaid' => $structure->get_provaid(),
        ));
        $editurl = new \moodle_url('/mod/prova/components/addquestions/question.php', array(
            'returnurl' => $returnurl->out_as_local_url(),
            'id' => $question->id,
            'courseid' => $structure->get_courseid()
        ));

        $instancename = prova_question_tostring($question);

        $qtype = \question_bank::get_qtype($question->qtype, false);
        $namestr = $qtype->local_name();

        $icon = $this->pix_icon('icon', $namestr, $qtype->plugin_name(), array('title' => $namestr, 'class' => 'icon activityicon', 'alt' => ' ', 'role' => 'presentation'));
        
        $editicon = $this->pix_icon('t/edit', '', 'moodle', array('title' => ''));

        // Need plain question name without html tags for link title.
        $title = shorten_text(format_string($question->name), 100);

        // Define se exibe ou nao o botão de edição
        
        $editicon = html_writer::tag('span', $editicon, array('style' => 'visibility:hidden'));
        $activitylink = $icon . html_writer::tag('span', $editicon . $instancename, array('class' => 'instancename'));
        $output .= html_writer::tag('p', $activitylink, array('title' => get_string('editquestion', 'provacomp_addprovas').' '.$title));

        return $output;
    }

    /**
     * Gera um link e icone de preview de questão
     *
     * @param $qid \int id da questão
     * @param $courseid \int id do curso
     * @return \String html do elemento de preview
     */
    protected function gera_question_preview($qid, $courseid) {
        $previewurl = new \moodle_url('/question/preview.php', array(
            'courseid' => $courseid,
            'id' => $qid,
        ));
        $previewimage = $this->pix_icon('t/preview', get_string('preview'), '', array('class' => 'iconsmall'));
        $previewaction = new \popup_action('click', $previewurl, 'questionpreview', array(
            'height' => 600,
            'width' => 800,
        ));
        return $this->action_link($previewurl, $previewimage, $previewaction, array());
    } 

    public function marked_out_of_field(structure $structure, $slot) {

        $output = html_writer::span($structure->formatted_question_grade($slot),
                'instancemaxmark decimalplaces_' . $structure->get_decimal_places_for_question_marks(),
                array('title' => get_string('maxmark', 'provacomp_addprovas')));

        if ($structure->can_be_edited()) {
            $output .= html_writer::span(
                html_writer::link(
                    new \moodle_url('#'),
                    $this->pix_icon('t/editstring', '', 'moodle', array('class' => 'editicon visibleifjs', 'title' => '')),
                    array(
                        'class' => 'editing_maxmark',
                        'data-action' => 'editmaxmark',
                        'title' => get_string('editmaxmark', 'provacomp_addprovas'),
                    )
                )
            );
        } else {
            $output .= html_writer::span('', 'editing_maxmark');
        }
        
        return html_writer::span($output, 'instancemaxmarkcontainer');
    }

    /** 
     * Mostra em cada linha a dificuldade da questão específica
     *
     * @param structure \structure
     * @param slot \stdClass
     * @return \String difficulty marker 
     */
    protected function display_difficulty_markers($structure, $slot) {
        $question = $structure->get_question_in_slot($slot);

        list($class, $text) = generate_output_to_difficulty($question->dificuldade);

        $output = html_writer::span($text, 'instancedifficulty', array('title' => $text));

        return html_writer::span($output, 'containerdifficulty ' . $class);
    }

    /**
     * Function from original edit_renderer.php
     * Adiciona botões de adicionar questões específicas ou aleatórias
     */
    public function add_menu_actions(structure $structure, $page, \moodle_url $pageurl,
            \question_edit_contexts $contexts=null, array $pagevars) {

        $actions = $this->edit_menu_actions($structure, $page, $pageurl, $pagevars);
        if (empty($actions)) {
            return '';
        }
        $menu = new \action_menu();
        $menu->set_alignment(\action_menu::TR, \action_menu::TR);
        $menu->set_constraint('.mod-quiz-edit-content');
        $trigger = html_writer::tag('span', get_string('add', 'provacomp_addprovas'), array('class' => 'add-menu'));
        $menu->set_menu_trigger($trigger);
        // The menu appears within an absolutely positioned element causing width problems.
        // Make sure no-wrap is set so that we don't get a squashed menu.
        $menu->set_nowrap_on_items(true);

        // Disable the link if quiz has attempts.
        if (!$structure->can_be_edited()) {
            return $this->render($menu);
        }

        foreach ($actions as $action) {
            if ($action instanceof \action_menu_link) {
                $action->add_class('add-menu');
            }
            $menu->add($action);
        }
        $menu->attributes['class'] .= ' page-add-actions commands';

        // Prioritise the menu ahead of all other actions.
        $menu->prioritise = true;

        return $this->render($menu);
    }

    /**
     * Function from original edit_renderer.php
     * Prepara os elementos HTML dos botões de adicionar questão
     */
    public function edit_menu_actions(structure $structure, $page,
            \moodle_url $pageurl, array $pagevars) {

        static $str;
        if (!isset($str)) {
            $str = get_strings(array('addasection', 'addaquestion', 'addarandomquestion',
                    'addarandomselectedquestion', 'questionbank'), 'provacomp_addprovas');
        }

        // Get section, page, slotnumber and maxmark.
        $actions = array();

        // Add a new question to the quiz.
        $returnurl = new \moodle_url($pageurl, array(
            'addonpage' => $page,
            'provaid' => $structure->get_provaid(),
        ));
        $params = $structure->get_course_params($returnurl->out_as_local_url(false));

        $params = array_merge($params, array('addonpage' => $page, 'appendqnumstring' => 'addquestion'));

        $actions['addaquestion'] = new \action_menu_link_secondary(
            new \moodle_url('/mod/prova/components/addquestions/addquestions.php', $params),
            new \pix_icon('t/add', $str->addaquestion, 'moodle', array('class' => 'iconsmall', 'title' => '')),
            $str->addaquestion, array('class' => 'cm-edit-action addquestion', 'data-action' => 'addquestion')
        );

        // Call question bank.
        $icon = new \pix_icon('t/add', $str->questionbank, 'moodle', array('class' => 'iconsmall', 'title' => ''));
        if ($page) {
            $title = get_string('addquestionfrombanktopage', 'provacomp_addprovas', $page);
        } else {
            $title = get_string('addquestionfrombankatend', 'provacomp_addprovas');
        }
        $attributes = array('class' => 'cm-edit-action questionbank', 'data-header' => $title, 'data-action' => 'questionbank', 'data-addonpage' => $page);
        $actions['questionbank'] = new \action_menu_link_secondary($pageurl, $icon, $str->questionbank, $attributes);

        // Add a random question.
        $returnurl = $this->currenturl;
        $returnurl->param('data-addonpage', $page);
        $params = ['returnurl' => $returnurl, 'appendqnumstring' => 'addarandomquestion'];
        $url = $this->currenturl;//new \moodle_url('/mod/prova/components/addprovas/addrandom.php', $params);
        $icon = new \pix_icon('t/add', $str->addarandomquestion, 'moodle', array('class' => 'iconsmall', 'title' => ''));
        $attributes = array('class' => 'cm-edit-action addarandomquestion', 'data-action' => 'addarandomquestion');
        if ($page) {
            $title = get_string('addrandomquestiontopage', 'provacomp_addprovas', $page);
        } else {
            $title = get_string('addrandomquestionatend', 'provacomp_addprovas');
        }
        $attributes = array_merge(array('data-header' => $title, 'data-addonpage' => $page), $attributes);
        $actions['addarandomquestion'] = new \action_menu_link_secondary($url, $icon, $str->addarandomquestion, $attributes);

        // Global css hides the last element, so I created a fake action
        $actions['fakeaction'] = new \action_menu_link_secondary(new \moodle_url('my'), null, '', array('style' => 'display:none;'));
        return $actions;
    }

    /**
     * Render the move icon.
     *
     * @param structure $structure object containing the structure of the prova.
     * @param int $slot the first slot on the page we are outputting.
     * @return string The markup for the move action.
     */
    public function question_move_icon(structure $structure, $slot) {
        return html_writer::link(new \moodle_url('#'),
            $this->pix_icon('i/dragdrop', get_string('move'), 'moodle', array('class' => 'iconsmall', 'title' => '')),
            array('class' => 'editing_move', 'data-action' => 'move')
        );
    }

    /**
     * Output the question number.
     * @param string $number The number, or 'i'.
     * @return string HTML to output.
     */
    public function question_number($number) {
        if (is_numeric($number)) {
            $number = html_writer::span(get_string('question'), 'accesshide') . ' ' . $number;
        }
        return html_writer::tag('span', $number, array('class' => 'slotnumber'));
    }

    /**
     * Render an icon to remove a question from the prova.
     *
     * @param structure $structure object containing the structure of the prova.
     * @param int $slot the first slot on the page we are outputting.
     * @param \moodle_url $pageurl the canonical URL of the edit page.
     * @return string HTML to output.
     */
    public function question_remove_icon(structure $structure, $slot, $pageurl) {
        $url = new \moodle_url($pageurl, array('sesskey' => sesskey(), 'remove' => $slot));
        $strdelete = get_string('delete');

        $image = $this->pix_icon('t/delete', $strdelete);

        return $this->action_link($url, $image, null, array('title' => $strdelete,
                    'class' => 'cm-edit-action editing_delete', 'data-action' => 'delete'));
    }

    /**
     * Renders the question chooser.
     *
     * @param renderable
     * @return string
     */
    public function render_question_chooser(renderable $chooser) {
        return $this->render_from_template('provacomp_addprovas/question_chooser', $chooser->export_for_template($this));
    }

    /**
     * Render the question type chooser dialogue.
     * @return string HTML to output.
     */
    public function question_chooser() {
        $chooser = \provacomp_addprovas\output\question_chooser::get($this->page->course, [], null);
        $container = html_writer::div($this->render($chooser), '', array('id' => 'qtypechoicecontainer'));
        return html_writer::div($container, 'createnewquestion');
    }

    /**
     * Function from original edit_renderer.php
     * Carrega JS na página 
     */
    protected function initialise_editing_javascript(structure $structure,
            \question_edit_contexts $contexts, array $pagevars, \moodle_url $pageurl) {

        $config = new \stdClass();
        $config->resourceurl = '/mod/prova/components/addprovas/edit_rest.php';
        $config->sectionurl = '/mod/prova/components/addprovas/edit_rest.php';
        $config->pageparams = array();
        $config->questiondecimalpoints = $structure->get_decimal_places_for_question_marks();

        $this->page->requires->strings_for_js(array('areyousureremoveselected'), 'provacomp_addprovas');
        $this->page->requires->yui_module('moodle-provacomp_addprovas-toolboxes',
                'M.provacomp_addprovas.init_section_toolbox',
                array(array(
                        'courseid' => $structure,
                        'provaid' => $structure->get_provaid(),
                        'ajaxurl' => $config->sectionurl,
                        'config' => $config,
                ))
        );

        $this->page->requires->yui_module('moodle-provacomp_addprovas-toolboxes',
                'M.provacomp_addprovas.init_resource_toolbox',
                array(array(
                        'courseid' => $structure->get_courseid(),
                        'provaid' => $structure->get_provaid(),
                        'ajaxurl' => $config->resourceurl,
                        'config' => $config,
                ))
        );
        unset($config->pagehtml);
        unset($config->addpageiconhtml);

        $this->page->requires->yui_module('moodle-provacomp_addprovas-dragdrop', 'M.provacomp_addprovas.init_resource_dragdrop',
                array(array(
                        'courseid' => $structure,
                        'provaid' => $structure->get_provaid(),
                        'ajaxurl' => $config->resourceurl,
                        'config' => $config,
                )), null, true);

        // Js que alterar referencia da url de criacao de questoes
        $addquestionUrl = new \moodle_url('/mod/prova/components/addquestions/addquestions.php');
        $this->page->requires->yui_module(
            'moodle-provacomp_addquestions-btn_addquestion',
            'M.provacomp_addquestions.btn_addquestion.init',
            array(strval($addquestionUrl))
        );

        $this->page->requires->yui_module(
            'moodle-provacomp_addprovas-question_control',
            'M.provacomp_addprovas.question_control.init'
        );

        // Require various strings for the command toolbox.
        $this->page->requires->strings_for_js(array(
                'clicktohideshow',
                'deletechecktype',
                'deletechecktypename',
                'edittitle',
                'edittitleinstructions',
                'emptydragdropregion',
                'hide',
                'markedthistopic',
                'markthistopic',
                'move',
                'movecontent',
                'moveleft',
                'movesection',
                'page',
                'question',
                'selectall',
                'show',
                'tocontent',
        ), 'moodle');

        $this->page->requires->strings_for_js(array(
                'addpagebreak',
                'cannotremoveallsectionslots',
                'cannotremoveslots',
                'confirmremovesectionheading',
                'confirmremovequestion',
                'dragtoafter',
                'dragtostart',
                'numquestionsx',
                'sectionheadingedit',
                'sectionheadingremove',
                'removepagebreak',
                'questiondependencyadd',
                'questiondependencyfree',
                'questiondependencyremove',
                'questiondependsonprevious',
        ), 'provacomp_addprovas');

        foreach (\question_bank::get_all_qtypes() as $qtype => $notused) {
            $this->page->requires->string_for_js('pluginname', 'qtype_' . $qtype);
        }

        return true;
    }

    /**
     * Function from original edit_renderer.php
     * Carrega conteudo do modal do questionbank 
     */
    public function question_bank_contents(\provacomp_addprovas\question\bank\custom_view $questionbank, array $pagevars) {

        $str_targets = array('md_areas', 'md_subareas', 'md_assuntos', 'md_text');
        $array_targets = array('md_dificuldades', 'md_qtypes');

        $search = array();

        foreach ($pagevars as $key => $value) {
            if (in_array($key, $str_targets)) {
                $search[$key] = $value;
            } elseif (in_array($key, $array_targets) and $value !== '') {
                $search[$key] = explode('-', $value);
            }
        }

        $pagevars = array(
            'qpage'         => (array_key_exists('qpage', $pagevars)) ? $pagevars['qpage'] : 0,
            'qperpage'      => (array_key_exists('qperpage', $pagevars)) ? $pagevars['qperpage'] : 15, 
            'cat'           => \provacomp_addprovas\structure::get_cquestion_category_and_context_id($pagevars['courseid']),
            'recurse'       => 1,
            'showhidden'    => false,
            'qbshowtext'    => true,
            'search'        => $search,
        );

        $qbank = $questionbank->render('editq', $pagevars['qpage'], $pagevars['qperpage'],
                $pagevars['cat'], $pagevars['recurse'], $pagevars['showhidden'], $pagevars['qbshowtext'], $pagevars['search']);
        return html_writer::div(html_writer::div($qbank, 'bd'), 'questionbankformforpopup');
    }
}
?>