<?php
/**
 * Define a tela de banco de questões da view do Edit Prova
 *
 * @package   provacomp_addprovas
 */

namespace provacomp_addprovas\question\bank;
defined('MOODLE_INTERNAL') || die();

/**
 * Subclass to customise the view of the question bank for the prova editing screen.
 */
class custom_view extends \core_question\bank\view {
	/** @var bool whether the prova this is used by has been attemptd. */
    protected $provahasattempts = false;
    /** @var int The maximum displayed length of the category info. */
    const MAX_TEXT_LENGTH = 200;

	/** @var \stdClass the prova settings. */
	protected $prova = false;

	/** @var \array parametros de busca. */
	protected $search = null;

	/** @var \bool verifica se é addrandom. */
	protected $is_random = null;

	public function __construct($contexts, $pageurl, $course, $cm = null, $prova, $is_random) {
		global $CFG, $PAGE;

		$this->prova = $prova;
		$this->is_random = $is_random;
		parent::__construct($contexts, $pageurl, $course, $cm);
	}

	protected function wanted_columns() {
		global $CFG;

		if (empty($CFG->provaquestionbankcolumns)) {
			if ($this->is_random) {
				$provaquestionbankcolumns = array(
					'question_type_column',
					'question_name_text_column',
					'preview_action_column',
				);
			} else {
				$provaquestionbankcolumns = array(
					'add_action_column',
					'checkbox_column',
					'question_type_column',
					'question_name_text_column',
					'preview_action_column',
				);
			}
		} else {
			$provaquestionbankcolumns = explode(',', $CFG->provaquestionbankcolumns);
		}

		foreach ($provaquestionbankcolumns as $fullname) {
			if (!class_exists($fullname)) {
				if (class_exists('provacomp_addprovas\\question\\bank\\' . $fullname)) {
					$fullname = 'provacomp_addprovas\\question\\bank\\' . $fullname;
				} else if (class_exists('core_question\\bank\\' . $fullname)) {
					$fullname = 'core_question\\bank\\' . $fullname;
				} else if (class_exists('question_bank_' . $fullname)) {
					debugging('Legacy question bank column class question_bank_' .
						$fullname . ' should be renamed to provacomp_addprovas\\question\\bank\\' .
						$fullname, DEBUG_DEVELOPER);
					$fullname = 'question_bank_' . $fullname;
				} else {
					throw new coding_exception("No such class exists: $fullname");
				}
			}
			$this->requiredcolumns[$fullname] = new $fullname($this);
		}
		return $this->requiredcolumns;
	}

	/**
     * Specify the column heading
     *
     * @return string Column name for the heading
     */
    protected function heading_column() {
        return 'provacomp_addprovas\\question\\bank\\question_name_text_column';
    }

    protected function default_sort() {
        return array(
            'core_question\\bank\\question_type_column' => 1,
            'provacomp_addprovas\\question\\bank\\question_name_text_column' => 1,
        );
    }

	/** Overwritten */
	public function render($tabname, $page, $perpage, $cat, $recurse, $showhidden,
		$showquestiontext, $search, $tagids = []) {
		$this->search = $search;

		ob_start();
		$this->display($tabname, $page, $perpage, $cat, $recurse, $showhidden, $showquestiontext, $tagids);
		$out = ob_get_contents();
		ob_end_clean();
		return $out;
	}

	/** Overwritten */
	public function display($tabname, $page, $perpage, $cat, $recurse, $showhidden, $showquestiontext, $tagids = []) {
		global $PAGE, $CFG;

		$this->display_question_bank_header();

		// Mostra formulário de pesquisa
		$this->display_buscaquestoes_form();

		// Continues with list of questions.
		$this->display_question_list(null, $this->baseurl, $cat, null, $recurse, $page, $perpage);
	}

	/**
	 * Mostra formulário de busca no modal
	 */
	protected function display_buscaquestoes_form() {
		global $PAGE;

		if ($this->is_random) {
			$advancedsearch = 'advancedsearch_random';
			$thisform = new \provacomp_addprovas\form\addrandom_form();
			$params = [[
				'default' => $thisform->get_filtros_areas_content(),
				'is_random' => true,
			]];
		} else {
			$advancedsearch = 'advancedsearch';
			$thisform = new \provacomp_addprovas\form\buscaquestoes_form();
			$params = $thisform->get_filtros_areas_content();
		}

		print_collapsible_region_start('', $advancedsearch, get_string('advancedsearchoptions', 'question'), 'question_bank_advanced_search');

		if (!empty($this->search)) {
			$thisform->populate_elements($this->search);
		}
		$thisform->display();

		$PAGE->requires->yui_module(
			'moodle-provacomp_buscaquestoes-filtro_searchquestions',
			'M.provacomp_buscaquestoes.filtro_searchquestions.init',
			$params
		);

		print_collapsible_region_end();
	}

	public function add_to_prova_url($questionid) {
		global $CFG;
		$params = $this->baseurl->params();
		$params['provaid'] = $this->prova;
		$params['addquestion'] = $questionid;
		$params['sesskey'] = sesskey();
		return new \moodle_url('/mod/prova/components/addprovas/edit.php', $params);
	}

	/** Overwritten */
	protected function display_question_list($contexts, $pageurl, $categoryandcontext, $cm = null, $recurse = 1, $page = 0, $perpage = 20, $showhidden = false, $showquestiontext = false, $addcontexts = array()) {
		global $CFG, $DB, $OUTPUT, $PAGE;

		// This function can be moderately slow with large question counts and may time out.
		// We probably do not want to raise it to unlimited, so randomly picking 5 minutes.
		// Note: We do not call this in the loop because prova ob_ captures this function (see raise() PHP doc).
		\core_php_time_limit::raise(300);

		$category = $this->get_current_category($categoryandcontext);

		$strselectall = get_string('selectall');
		$strselectnone = get_string('deselectall');

		list($categoryid, $contextid) = explode(',', $categoryandcontext);
		$catcontext = \context::instance_by_id($contextid);

		$canadd = has_capability('moodle/question:add', $catcontext);

		$this->create_new_question_form($category, $canadd);

		$this->build_query();
		$totalnumber = $this->get_question_count();
		if ($totalnumber == 0) {
			return;
		}
		$questions = $this->load_page_questions($page, $perpage);
		echo '<div class="categorypagingbarcontainer">';
		$pageingurl = new \moodle_url('edit.php');
		$this->reprepare_url($pageingurl);
		$r = $pageingurl->params($pageurl->params());
		$pagingbar = new \paging_bar($totalnumber, $page, $perpage, $pageingurl);
		$pagingbar->pagevar = 'qpage';
		echo $OUTPUT->render($pagingbar);
		echo '</div>';

		echo '<form method="post" action="edit.php">';
		echo '<input type="hidden" name="provaid" value="' . $this->prova . '" />';
		echo '<fieldset class="invisiblefieldset" style="display: block;">';
		echo '<input type="hidden" name="sesskey" value="' . sesskey() . '" />';
		echo \html_writer::input_hidden_params($this->baseurl);

		echo '<div class="categoryquestionscontainer" id="questionscontainer">';
		$this->start_table();
		$rowcount = 0;
		foreach ($questions as $question) {
			$this->print_table_row($question, $rowcount);
			$rowcount += 1;
		}
		$questions->close();
		$this->end_table();

		echo '<div class="categorypagingbarcontainer pagingbottom">';
		echo $OUTPUT->render($pagingbar);
		if ($totalnumber > DEFAULT_QUESTIONS_PER_PAGE) {
			if ($perpage == DEFAULT_QUESTIONS_PER_PAGE) {
				$url = new \moodle_url('edit.php', array_merge($pageurl->params(), array('qperpage' => MAXIMUM_QUESTIONS_PER_PAGE)));
				// $url->param('provaid', $this->prova);
				$this->reprepare_url($url);
				if ($totalnumber > MAXIMUM_QUESTIONS_PER_PAGE) {
					$showall = '<a href="' . $url . '">' . get_string('showperpage', 'moodle', MAXIMUM_QUESTIONS_PER_PAGE) . '</a>';
				} else {
					$showall = '<a href="' . $url . '">' . get_string('showall', 'moodle', $totalnumber) . '</a>';
				}
			} else {
				$url = new \moodle_url('edit.php', array_merge($pageurl->params(), array('qperpage' => DEFAULT_QUESTIONS_PER_PAGE)));
				// $url->param('provaid', $this->prova);
				$this->reprepare_url($url);
				$showall = '<a href="' . $url . '">' . get_string('showperpage', 'moodle', DEFAULT_QUESTIONS_PER_PAGE) . '</a>';
			}
			echo "<div class='paging'>{$showall}</div>";
		}
		echo '</div>';

		$PAGE->requires->js_call_amd('core_question/qbankmanager', 'init');

		if ($this->is_random) {
			$this->display_form_random_questions($totalnumber);
		} else {
			$this->display_bottom_controls($totalnumber, $recurse, $category, $catcontext, $addcontexts);
		}

		echo '</fieldset>';
		echo "</form>\n";
	}

	/** Overwritten */
	protected function print_table_headers() {
		echo "<tr>\n";
		foreach ($this->visiblecolumns as $key => $column) {
			if (strrpos($key, 'preview_action_column') !== false) {
				echo '<th class="header iconcol" scope="col" style="width: 72px !important;"></th>';
			}
			$column->display_header();
		}
		echo "</tr>\n";
	}

	/** Overwritten */
	protected function print_table_row($question, $rowcount) {
		$rowclasses = implode(' ', $this->get_row_classes($question, $rowcount));
		if ($rowclasses) {
			echo '<tr class="' . $rowclasses . '">' . "\n";
		} else {
			echo "<tr>\n";
		}
		foreach ($this->visiblecolumns as $key => $column) {
			if (strrpos($key, 'preview_action_column') !== false) {
				$this->display_difficulty($question);
			}
			$column->display($question, $rowclasses);
		}
		echo "</tr>\n";
		foreach ($this->extrarows as $row) {
			$row->display($question, $rowclasses);
		}
	}

	/**
	 * Mostra a dificuldade de cada questão
	 */
	public function display_difficulty($question) {
		global $CFG;
		require_once $CFG->dirroot . '/mod/prova/components/addprovas/locallib.php';
		list($class, $text) = generate_output_to_difficulty($question->dificuldade);
		$output = \html_writer::span($text, 'instancedifficulty', array('title' => $text));
		$output = \html_writer::span($output, 'containerdifficulty ' . $class);
		echo \html_writer::tag('td', $output);
	}

	/**
	 * Usado para repopular a URL de busca com base nos parametros
	 * vindos da requisicao
	 */
	protected function reprepare_url($url) {

		$search = $this->search;

		$url->param('provaid', $this->prova);

		if ($search['md_areas'] != 0) {
			$url->param('md_areas', $search['md_areas']);
		}
		if ($search['md_subareas'] != 0) {
			$url->param('md_subareas', $search['md_subareas']);
		}
		if ($search['md_assuntos'] != 0) {
			$url->param('md_assuntos', $search['md_assuntos']);
		}
		if (!empty($search['md_dificuldades'])) {
			$url->param('md_dificuldades', implode('-', $search['md_dificuldades']));
		}
		if (!empty($search['md_qtypes'])) {
			$url->param('md_qtypes', implode('-', $search['md_qtypes']));
		}
		if ($search['md_text'] and $search['md_text'] != '') {
			$url->param('md_text', $search['md_text']);
		}
	}

	/** Overwritten */
	protected function build_query() {
		global $DB;

		// Get the required tables and fields.
		$joins = array();
		$fields = array('q.hidden', 'q.category');
		foreach ($this->requiredcolumns as $column) {
			$extrajoins = $column->get_extra_joins();
			foreach ($extrajoins as $prefix => $join) {
				if (isset($joins[$prefix]) && $joins[$prefix] != $join) {
					throw new \coding_exception('Join ' . $join . ' conflicts with previous join ' . $joins[$prefix]);
				}
				$joins[$prefix] = $join;
			}
			$fields = array_merge($fields, $column->get_required_fields());
		}
		$fields = array_unique($fields);

		// Build the order by clause.
		$sorts = array();
		foreach ($this->sort as $sort => $order) {
			list($colname, $subsort) = $this->parse_subsort($sort);
			$sorts[] = $this->requiredcolumns[$colname]->sort_expression($order < 0, $subsort);
		}

		$prevjoin = array('INNER JOIN {seox_questions} seox ON q.id = seox.id_question');
		$joins = array_merge($prevjoin, $joins);
		$fields[] = 'seox.dificuldade as dificuldade';

		// Build the where clause.
		$tests = array_merge(array('q.parent = 0'), \provacomp_addprovas\action\addprovas_actions::build_search_query($this->search));

		$tests[] = "NOT EXISTS (SELECT * FROM {seox_prova_slots} WHERE provaid = " . $this->prova . " AND questionid = q.id)";

		// Build the SQL.
		$sql = ' FROM {question} q ' . implode(' ', $joins);
		$sql .= " WHERE " . implode(' AND ', $tests);
		$this->countsql = 'SELECT count(1)' . $sql;
		$this->loadsql = 'SELECT ' . implode(', ', $fields) . $sql . ' ORDER BY ' . implode(', ', $sorts);
	}

	/** Overwritten */
	protected function display_bottom_controls($totalnumber, $recurse, $category, \context $catcontext, array $addcontexts) {
		$cmoptions = new \stdClass();
		$cmoptions->hasattempts = !empty($this->provahasattempts);

		$canuseall = has_capability('moodle/question:useall', $catcontext);

		echo '<div class="modulespecificbuttonscontainer">';
		if ($canuseall) {

			// Add selected questions to the prova.
			$params = array(
				'type' => 'submit',
				'name' => 'add',
				'class' => 'btn btn-primary',
				'value' => get_string('addbankquestions_button', 'provacomp_addprovas'),
			);
			if ($cmoptions->hasattempts) {
				$params['disabled'] = 'disabled';
			}
			echo \html_writer::empty_tag('input', $params);
		}
		echo "</div>\n";
	}

	/**
	 * Mostra o formulário de questões
	 */
	protected function display_form_random_questions($questions) {
		$questions = ($questions > 100) ? 100 : $questions;
		$questions = range(1, $questions);

		echo '<p style="margin-top:30px">' . get_string('pre_addrandomquestion_button', 'provacomp_addprovas') . '</p>';
		echo '<div style="display:flex">';
		foreach ($this->search as $key => $value) {
			if ($key === 'md_dificuldades' or $key === 'md_qtypes') {
				echo \html_writer::empty_tag('input', array('type' => 'hidden', 'name' => $key, 'value' => implode('-', $value)));
			} else {
				echo \html_writer::empty_tag('input', array('type' => 'hidden', 'name' => $key, 'value' => $value));
			}
		}
		echo \html_writer::select($questions, 'questionnumber', '', false, array('id' => 'random_question_qtd'));
		$params = array(
			'type' => 'submit',
			'name' => 'addrandom',
			'class' => 'btn btn-primary',
			'value' => get_string('selectrandomquestions_button', 'provacomp_addprovas'),
			'style' => 'margin-left: 10px',
		);
		echo \html_writer::empty_tag('input', $params);
		echo '</div>';
	}
}