<?php
/**
 * Arquivo baseado na página de edição de questões de uma
 * página.
 * Original Source: /mod/quiz/edit.php
 */
require_once(__DIR__ . '/../config.php');
require_once($CFG->dirroot . '/mod/prova/managelib.php');
require_once($CFG->dirroot . '/question/editlib.php');
require_once('locallib.php');
require_once('provalib.php');

global $USER, $DB;

\mod_prova\manager::has_permission_to_access();

$course_id = SEOX_COURSE_ID;
$prova_id = required_param('provaid', PARAM_INT);
$originalreturnurl = optional_param('returnurl', 0, PARAM_LOCALURL);

$thispageurl = new moodle_url('/mod/prova/components/addprovas/edit.php', array('provaid' => $prova_id));
if ($originalreturnurl !== 0) {
    $thispageurl->param('returnurl', $originalreturnurl);
}
$afteractionurl = $thispageurl;
$originalreturnurl = new moodle_url(($originalreturnurl !== 0) ? $originalreturnurl : '/mod/prova/components/provas.php'); 
$originalreturnurl->param('provaid', $prova_id);

$pagename = get_string('questionstructureprova_title', 'provacomp_addprovas');
$pagevars = array('courseid' => $course_id);
$context = context_course::instance($course_id);
$prova = new ProvaObj($prova_id);
$structure = $prova->get_structure();

$PAGE->set_context(context_system::instance());
$PAGE->set_url($thispageurl);
$PAGE->set_title($pagename);
$PAGE->set_heading($pagename);

// if ($scrollpos) {
//     $afteractionurl->param('scrollpos', $scrollpos);
// }

if (($addquestion = optional_param('addquestion', false, PARAM_INT) or $addquestion = optional_param('lastchanged', false, PARAM_INT)) && confirm_sesskey()) {
    require_capability('mod/prova:accesstoread', $context);

    // Add a single question to the current quiz.
    $structure->check_can_be_edited();
    $question = $DB->get_record('question', array('id' => $addquestion), '*', MUST_EXIST);
    $addonpage = 1;
    prova_add_prova_question($addquestion, $prova->get_data(), $addonpage);
    // quiz_delete_previews($quiz);
    prova_update_sumgrades($prova->get_data());
    redirect($afteractionurl);
}

if (optional_param('add', false, PARAM_BOOL) && confirm_sesskey()) {
    require_capability('mod/prova:accesstoread', $context);

    $structure->check_can_be_edited();
    $addonpage = 1;
    // Add selected questions to the current quiz.
    $rawdata = (array) data_submitted();
    foreach ($rawdata as $key => $value) { // Parse input for question ids.
        if (preg_match('!^q([0-9]+)$!', $key, $matches)) {
            $key = $matches[1];
            prova_add_prova_question($key, $prova->get_data(), $addonpage);
        }
    }
    // quiz_delete_previews($quiz);
    prova_update_sumgrades($prova->get_data());
    redirect($afteractionurl);
}

if ((optional_param('addrandom', false, PARAM_BOOL)) && confirm_sesskey()) {
    require_capability('mod/prova:accesstoread', $context);

    $randomcount = required_param('questionnumber', PARAM_INT);
    $params = array(
        'md_areas' => optional_param('md_areas', 0, PARAM_INT),
        'md_subareas' => optional_param('md_subareas', 0, PARAM_INT),
        'md_assuntos' => optional_param('md_assuntos', 0, PARAM_INT),
        'md_dificuldades' => optional_param('md_dificuldades', '', PARAM_ALPHANUMEXT),
        'md_qtypes' => optional_param('md_qtypes', '', PARAM_ALPHANUMEXT),
        'md_text' => optional_param('md_text', '', PARAM_ALPHANUMEXT),
    );

    $params['md_dificuldades'] = ($params['md_dificuldades'] and $params['md_dificuldades'] != '') ? explode('-', $params['md_dificuldades']) : [];
    $params['md_qtypes'] = ($params['md_qtypes'] and $params['md_qtypes'] != '') ? explode('-', $params['md_qtypes']) : [];

    // Add random questions to the quiz.
    $structure->check_can_be_edited();
    $addonpage = 1;
    $ids = prova_add_random_questions($prova_id, $randomcount, $params);
    foreach ($ids as $id) {
        prova_add_prova_question($id, $prova->get_data(), $addonpage);
    }

    prova_update_sumgrades($prova->get_data());
    redirect($afteractionurl);
}

$PAGE->set_pagelayout('incourse');
$PAGE->set_pagetype('mod-quiz-edit');
$PAGE->requires->css('/mod/prova/components/addprovas/edit_style.css');

$output = $PAGE->get_renderer('provacomp_addprovas', 'edit');
$output->set_currenturl($thispageurl);

$provaobj = $DB->get_record('seox_prova', array('id' => $prova_id));

echo $OUTPUT->header();

if($USER->id != $provaobj->createdby){
   echo $OUTPUT->notification(get_string('err_provaedit', 'provacomp_addprovas'), 'notifyerros');
} else {
    echo html_writer::start_tag('div', array('class' => 'mod-quiz-edit-content'));
    echo $output->edit_page($prova, $structure, $context, $thispageurl, $pagevars);
    echo html_writer::end_tag('div');

    if ($structure->can_be_edited()) {
        echo $OUTPUT->single_button($originalreturnurl, get_string('conclude_prova_edition', 'provacomp_addprovas'), 'get', array('disabled'=>true, 'class'=>'returnurl-div'));
    } else {
        echo $OUTPUT->single_button($originalreturnurl, get_string('conclude_viewprova_edition', 'provacomp_addprovas'), 'get', array('class'=>'returnurl-div'));
    }
}

echo $OUTPUT->footer();