<?php
/**
 * Arquivo baseado na REST do renderizador do modulo Quiz
 * Original Source: /mod/quiz/edit_rest.php
 *
 * Objetivo é atualizar o banco conforme as ações dos 
 * usuários
 */

if (!defined('AJAX_SCRIPT')) {
    define('AJAX_SCRIPT', true);
}

require_once(__DIR__ . '/../config.php');
require_once($CFG->dirroot . '/mod/prova/managelib.php');
require_once('provalib.php');
require_once('locallib.php');

// Initialise ALL the incoming parameters here, up front.
$provaid    = required_param('provaid', PARAM_INT);
$class      = required_param('class', PARAM_ALPHA);
$field      = optional_param('field', '', PARAM_ALPHA);
$instanceid = optional_param('instanceId', 0, PARAM_INT);
$sectionid  = optional_param('sectionId', 0, PARAM_INT);
$previousid = optional_param('previousid', 0, PARAM_INT);
$value      = optional_param('value', 0, PARAM_INT);
$column     = optional_param('column', 0, PARAM_ALPHA);
$id         = optional_param('id', 0, PARAM_INT);
$summary    = optional_param('summary', '', PARAM_RAW);
$sequence   = optional_param('sequence', '', PARAM_SEQUENCE);
$visible    = optional_param('visible', 0, PARAM_INT);
$pageaction = optional_param('action', '', PARAM_ALPHA); // Used to simulate a DELETE command.
$maxmark    = optional_param('maxmark', '', PARAM_FLOAT);
$newheading = optional_param('newheading', '', PARAM_TEXT);
$shuffle    = optional_param('newshuffle', 0, PARAM_INT);
$page       = optional_param('page', '', PARAM_INT);
$ids        = optional_param('ids', '', PARAM_SEQUENCE);
$PAGE->set_url('/mod/prova/components/addprovas/edit-rest.php', array('provaid' => $provaid, 'class' => $class));

require_sesskey();
$prova = $DB->get_record('seox_prova', array('id' => $provaid), '*', MUST_EXIST);
$course = $DB->get_record('course', array('id' => SEOX_COURSE_ID), '*', MUST_EXIST);
require_login($course, false, $cm);

$provaobj = new ProvaObj($prova);
$structure = $provaobj->get_structure();
if (!$structure->can_be_edited()) {
    print_error('err_provaedited', 'provacomp_addprovas');
}
$maincontext = context_course::instance(SEOX_COURSE_ID);

echo $OUTPUT->header(); // Send headers.

// All these AJAX actions should be logically atomic.
$transaction = $DB->start_delegated_transaction();

// OK, now let's process the parameters and do stuff
// MDL-10221 the DELETE method is not allowed on some web servers,
// so we simulate it with the action URL param.
$requestmethod = $_SERVER['REQUEST_METHOD'];
if ($pageaction == 'DELETE') {
    $requestmethod = 'DELETE';
}

$result = null;

switch($requestmethod) {
    case 'POST':
    case 'GET': // For debugging.
        switch ($class) {
            case 'resource':
                switch ($field) {
                    case 'move':
                        require_capability('mod/prova:accesstoread', $maincontext);
                        $structure->move_slot($id, $previousid, $page);
                        $result = array('visible' => true);
                        break;

                    case 'getmaxmark':
                        require_capability('mod/prova:accesstoread', $maincontext);
                        $slot = $DB->get_record('seox_prova_slots', array('id' => $id), '*', MUST_EXIST);
                        $result = array('instancemaxmark' => prova_format_question_grade($slot->maxmark));
                        break;

                    case 'updatemaxmark':
                        require_capability('mod/prova:accesstoread', $maincontext);
                        $slot = $structure->get_slot_by_id($id);
                        if ($structure->update_slot_maxmark($slot, $maxmark)) {
                            // Grade has really changed.
                            prova_update_sumgrades($prova);
                        }
                        $result = array('instancemaxmark' => prova_format_question_grade($prova, $maxmark), 'newsummarks' => prova_format_grade($prova, $prova->sumgrades));
                        break;

                    case 'updatepagebreak':
                        require_capability('mod/prova:accesstoread', $maincontext);
                        $slots = $structure->update_page_break($id, $value);
                        $json = array();
                        foreach ($slots as $slot) {
                            $json[$slot->slot] = array('id' => $slot->id, 'slot' => $slot->slot, 'page' => $slot->page);
                        }
                        $result = array('slots' => $json);
                        break;

                    case 'deletemultiple':
                        require_capability('mod/prova:accesstoread', $maincontext);
                        $ids = explode(',', $ids);
                        if ($structure->get_pages_count() <= count($ids)) {
                            print_error('err_invalid_delete_order', 'provacomp_addprovas');
                            break;
                        }
                        foreach ($ids as $id) {
                            $slot = $DB->get_record('seox_prova_slots', array('provaid' => $prova->id, 'id' => $id), '*', MUST_EXIST);
                            if (prova_has_question_use($prova, $slot->slot)) {
                                $structure->remove_slot($slot->slot);
                            }
                        }
                        prova_update_sumgrades($prova);
                        $dificuldade = difficulty_ajax_response($prova->id, $prova->dificuldade);

                        $result = array(
                            'newsummarks' => prova_format_grade($prova, $prova->sumgrades),
                            'deleted' => true,
                            'newnumquestions' => $structure->get_question_count(),
                            'difficulty' => $dificuldade,
                        );
                        break;

                    case 'updatedependency':
                        require_capability('mod/prova:accesstoread', $maincontext);
                        $slot = $structure->get_slot_by_id($id);
                        $value = (bool) $value;
                        $structure->update_question_dependency($slot->id, $value);
                        $result = array('requireprevious' => $value);
                        break;
                }
                break;
        }
        break;

    case 'DELETE':
        switch ($class) {
            case 'resource':
                require_capability('mod/prova:accesstoread', $maincontext);

                if ($structure->get_pages_count() <= 1) {
                    print_error('err_invalid_delete_order', 'provacomp_addprovas');
                    break;
                }

                if (!$slot = $DB->get_record('seox_prova_slots', array('provaid' => $prova->id, 'id' => $id))) {
                    throw new moodle_exception('AJAX commands.php: Bad slot ID '.$id);
                }
                $structure->remove_slot($slot->slot);
                prova_update_sumgrades($prova);
                $dificuldade = difficulty_ajax_response($prova->id, $prova->dificuldade);

                $result = array(
                    'newsummarks' => prova_format_grade($prova, $prova->sumgrades),
                    'deleted' => true,
                    'newnumquestions' => $structure->get_question_count(),
                    'difficulty' => $dificuldade,
                );
                break;
        }
        break;
}

$transaction->allow_commit();
echo json_encode($result);
