<?php 
/**
 * Library of functions used by the seox_prova block.
 */
defined('MOODLE_INTERNAL') || die();

require_once($CFG->dirroot . '/mod/prova/components/addprovas/lib.php');
require_once($CFG->dirroot . '/mod/prova/components/addprovas/provalib.php');
require_once($CFG->libdir . '/questionlib.php');


/** 
 * Função extraída do arquivo /mod/quiz/locallib.php
 * Original name: quiz_add_quiz_question()
 *
 * Função adiciona uma questão a um slot da prova
 */
function prova_add_prova_question($questionid, $prova, $page = 0, $maxmark = null) {
    global $DB;

    // Make sue the question is not of the "random" type.
    $questiontype = $DB->get_field('question', 'qtype', array('id' => $questionid));
    if ($questiontype == 'random') {
        throw new coding_exception(
                'Adding "random" questions via prova_add_prova_question() is deprecated. Please use prova_add_random_questions().'
        );
    }
    $slots = $DB->get_records('seox_prova_slots', array('provaid' => $prova->id), 'slot', 'questionid, slot, page, id');
    if (array_key_exists($questionid, $slots)) {
        return false;
    }

    $trans = $DB->start_delegated_transaction();

    $maxpage = 1;
    $numonlastpage = 0;
    foreach ($slots as $slot) {
        if ($slot->page > $maxpage) {
            $maxpage = $slot->page;
            $numonlastpage = 1;
        } else {
            $numonlastpage += 1;
        }
    }

    // Add the new question instance.
    $slot = new stdClass();
    $slot->provaid = $prova->id;
    $slot->questionid = $questionid;

    if ($maxmark !== null) {
        $slot->maxmark = $maxmark;
    } else {
        $slot->maxmark = $DB->get_field('question', 'defaultmark', array('id' => $questionid));
    }

    if (is_int($page) && $page >= 1) {
        // Adding on a given page.
        $lastslotbefore = 0;
        foreach (array_reverse($slots) as $otherslot) {
            if ($otherslot->page > $page) {
                $DB->set_field('seox_prova_slots', 'slot', $otherslot->slot + 1, array('id' => $otherslot->id));
            } else {
                $lastslotbefore = $otherslot->slot;
                break;
            }
        }
        $slot->slot = $lastslotbefore + 1;
        $slot->page = min($page, $maxpage + 1);

    } else {
        $lastslot = end($slots);
        if ($lastslot) {
            $slot->slot = $lastslot->slot + 1;
        } else {
            $slot->slot = 1;
        }
    	// TODO: add $prova->questionsperpage
        // if ($numonlastpage >= 5) {
        $slot->page = $maxpage + 1;
        // } else {
        //     $slot->page = $maxpage;
        // }
    }

    $DB->insert_record('seox_prova_slots', $slot);
    
    $DB->execute('UPDATE {seox_questions} SET vezes_usada = vezes_usada + 1 WHERE id_question = ?', [$questionid]);
    $trans->allow_commit();
}

/**
 * Função extraída do arquivo /mod/quiz/locallib.php
 * Original name: quiz_update_sumgrades()
 */
function prova_update_sumgrades($prova) {
    global $DB;

    $sql = 'UPDATE {seox_prova}
            SET sumgrades = COALESCE((
                SELECT SUM(maxmark)
                FROM {seox_prova_slots}
                WHERE provaid = {seox_prova}.id
            ), 0)
            WHERE id = ?';
    $DB->execute($sql, array($prova->id));
    $prova->sumgrades = $DB->get_field('seox_prova', 'sumgrades', array('id' => $prova->id));
}

/**
 * Função extraída do arquivo /mod/quiz/locallib.php
 * Original name: quiz_has_question_use()
 *
 * Verifica se questão existe
 */
function prova_has_question_use($prova, $slot) {
    global $DB;
    $question = $DB->get_record_sql("
            SELECT q.*
              FROM {seox_prova_slots} slot
              JOIN {question} q ON q.id = slot.questionid
             WHERE slot.provaid = ? AND slot.slot = ?", array($prova->id, $slot));
    if (!$question) {
        return false;
    }
    // return question_has_capability_on($question, 'use');
    return true;
}

/**
 * Função extraída do arquivo from /mod/quiz/locallib.php
 * Original name: quiz_has_question_use()
 */
function prova_format_grade($prova, $grade) {
    if (is_null($grade)) {
        return get_string('notyetgraded', 'provacomp_addprovas');
    }
    return format_float($grade, 2);
}

/**
 * Irá recalcular a dificuldade após requisicoes de ajax
 */
function difficulty_ajax_response($provaid, $old_difficulty) {
    global $DB;
    $sql = 'SELECT q.* FROM {seox_prova_slots} seox 
        INNER JOIN {seox_questions} q ON seox.questionid = q.id_question
        WHERE seox.provaid = :provaid';

    if ($questions = $DB->get_records_sql($sql, array('provaid' => $provaid))) {

        if ($dificuldade = difficulty_calculator($provaid, $questions, $old_difficulty)) {

            list($class, $text) = generate_output_to_difficulty($dificuldade['result']);
            return array('classe' => $class, 'text' => $text);
        }
        return false;
    }
    return array('hide' => true);
}

/**
 * Calculadora de dificuldade da prova
 */
function difficulty_calculator($provaid, $questions, $old_difficulty) {
    global $DB;

    $base = count($questions);
    if ($base <= 0) { return false; }

    $sumgrades = 0.0;
    foreach ($questions as $key => $question) {
        $sumgrades += $question->dificuldade + 1;
    }
    $result = $sumgrades / $base;

    if ($result <= 1.5) {
        $new_result = 0;
    } elseif ($result <= 2.2) {
        $new_result = 1;
    } else {
        $new_result = 2;
    }
    // echo '<script>console.log("'.$old_difficulty.'");</script>';

    if ($old_difficulty != $new_result) {
        $trans = $DB->start_delegated_transaction();

        $DB->set_field('seox_prova', 'dificuldade', $new_result, array('id' => $provaid));

        $trans->allow_commit();
        return array('result' => $new_result);
    }
    return false;
}

/**
 * Gera outpu de dificuldade
 */
function generate_output_to_difficulty($dificuldade) {
    $class = '';
    $text = '';
    switch ($dificuldade) {
        case 1:
            $class = 'medio';
            $text = get_string('title_medio', 'provacomp_addprovas');
            break;
        case 2:
            $class = 'dificil';
            $text = get_string('title_dificil', 'provacomp_addprovas');
            break;
        default:
            $class = 'facil';
            $text = get_string('title_facil', 'provacomp_addprovas');
            break;
    }
    return array($class, $text);
}

/**
 * Seleciona uma questão randomica do banco de dados
 * e adiona no quiz
 */
function prova_add_random_questions($provaid, $randomcount, $params) {
    global $DB;

    $randomcount++;

    $filtros = provacomp_addprovas\action\addprovas_actions::build_search_query($params);
    $filtros = implode(' AND ', $filtros);
    $filtros = ($filtros != '') ? ' AND ' . $filtros : '';
    $limit = $randomcount;

    $questions = $DB->get_fieldset_sql('SELECT id_question FROM {seox_questions} seox INNER JOIN {question} q ON seox.id_question = q.id WHERE NOT EXISTS (SELECT * FROM {seox_prova_slots} WHERE provaid = '.$provaid.' AND questionid = seox.id_question)' . $filtros);

    $keys = ($limit <= 1) ? [array_rand($questions, $limit)] : array_rand($questions, $limit);
    $selected = [];
    foreach ($keys as $k) {
        $selected[] = $questions[$k];
    }
    return $selected;
}

/**
 * Função extraída do arquivo from /mod/quiz/locallib.php
 * Original function: quiz_get_review_options
 *
 * Gera as permissões de usuários para ver o preview de uma prova
 *
 * @return mod_quiz_display_options
 */
function prova_get_preview_options() {
    $options = new question_display_options();

    $options->readonly = true;

    return $options;
}

?>