<?php
/**
 * Defines the renderer for the block seox_addprovas.
 */

defined('MOODLE_INTERNAL') || die();

// require_once($CFG->dirroot . '/mod/prova/renderer.php');

class provacomp_addprovas_renderer extends plugin_renderer_base {

	/**
	 * Cria estrutura visual do preview da prova
	 *
	 * @param attempt \prova_attempt_preview objeto da prova
	 * @param page \int representa o número slot que será exibido
	 * @param accessmanager TODO
	 * @param messages TODO
	 * @param slots \array(int) contém todos os ids dos slots de uma prova
	 * @param id TODO
	 * @param newxpage \int contém o número da próxima página que será exibida
	 * @return \String html da tela de preview
	 */
	public function preview_page($attempt, $page, $messages, $slots,
            $nextpage) {
        $this->page->set_pagelayout('popup');
        $this->page->requires->css('/mod/prova/components/addprovas/preview.scss');
        $output = '';
        $output .= $this->header();
        $output .= $this->preview_form($attempt, $page, $slots, $nextpage);
        $output .= $attempt->seox_get_navigation_panel($this, 'addprova_attempt_preview_nav_panel', $page);
        $output .= $this->footer();
        return $output;
    }

    /**
	 * Cria estrutura visual do preview da prova
	 *
	 * @param attempt \prova_attempt_preview objeto da prova
	 * @param page \int representa o número slot que será exibido
	 * @param slots \array(int) contém todos os ids dos slots de uma prova
	 * @param newxpage \int contém o número da próxima página que será exibida
	 * @return \String $output
	 */
    public function preview_form($attempt, $page, $slots, $nextpage) {
    	$prova = $attempt->get_prova();

    	$output = '';

        // Start the form.
        $output .= html_writer::start_tag(
        	'form',
            array(
            	'action' => $attempt->processpreview_url(),
            	'method' => 'post',
				'enctype' => 'multipart/form-data',
				'accept-charset' => 'utf-8',
				'id' => 'responseform'
			)
        );
        $output .= html_writer::start_tag('div');

        // Print all the questions.
        foreach ($slots as $slot) {
            $output .= $attempt->seox_render_question($slot, false, $this, $attempt->preview_url($prova->id, $page), $this);
        }

        $output .= $this->attempt_navigation_buttons($page, $attempt->is_last_page($page));

        // Some hidden fields to trach what is going on.
        $output .= html_writer::empty_tag('input', array('type' => 'hidden', 'name' => 'provaid',
                'value' => $prova->id));
        $output .= html_writer::empty_tag('input', array('type' => 'hidden', 'name' => 'thispage',
                'value' => $page, 'id' => 'followingpage'));
        $output .= html_writer::empty_tag('input', array('type' => 'hidden', 'name' => 'nextpage',
                'value' => $nextpage));
        $output .= html_writer::empty_tag('input', array('type' => 'hidden', 'name' => 'timeup',
                'value' => '0', 'id' => 'timeup'));
        $output .= html_writer::empty_tag('input', array('type' => 'hidden', 'name' => 'sesskey',
                'value' => sesskey()));
        $output .= html_writer::empty_tag('input', array('type' => 'hidden', 'name' => 'scrollpos',
                'value' => '', 'id' => 'scrollpos'));

        $output .= html_writer::end_tag('div');
        $output .= html_writer::end_tag('form');

        return $output;

    }

    /***/
    protected function attempt_navigation_buttons($page, $lastpage, $navmethod = 'free') {
        $output = '';

        $output .= html_writer::start_tag('div', array('class' => 'submitbtns'));
        if ($page > 0 && $navmethod == 'free') {
            $output .= html_writer::empty_tag('input', array('type' => 'submit', 'name' => 'previous',
                    'value' => get_string('navigateprevious', 'prova'), 'class' => 'mod_quiz-prev-nav btn btn-secondary'));
        }
        if ($lastpage) {
            $nextlabel = get_string('endtest', 'prova');
        } else {
            $nextlabel = get_string('navigatenext', 'prova');
        }
        $output .= html_writer::empty_tag('input', array('type' => 'submit', 'name' => 'next',
                'value' => $nextlabel, 'class' => 'mod_quiz-next-nav btn btn-primary'));
        $output .= html_writer::end_tag('div');

        return $output;
    }

    /**
     * Outputs the navigation block panel
     *
     * @param addprova_nav_panel_base $panel instance of addprova_nav_panel_base
     */
    public function navigation_panel(addprova_nav_panel_base $panel, $title) {

        $output = '</div></section>';

        $output .= '<section data-region="blocks-column-seox" class="d-print-none" aria-label="Blocks"><div class="card-body p-3">';

        $output .= $title; 

        $userpicture = $panel->user_picture();
        if ($userpicture) {
            $fullname = fullname($userpicture->user);
            if ($userpicture->size === true) {
                $fullname = html_writer::div($fullname);
            }
            $output .= html_writer::tag('div', $this->render($userpicture) . $fullname,
                    array('id' => 'user-picture', 'class' => 'clearfix'));
        }
        $output .= $panel->render_before_button_bits($this);

        $bcc = $panel->get_button_container_class();
        $output .= html_writer::start_tag('div', array('class' => "qn_buttons clearfix $bcc"));

        foreach ($panel->get_question_buttons() as $button) {
            $output .= $this->render($button);
        }
        $output .= html_writer::end_tag('div');

        $output .= html_writer::tag('div', $panel->render_end_bits($this),
                array('class' => 'othernav'));

        $this->page->requires->js_init_call('M.mod_prova.nav.init', null, false, prova_get_js_module());

        return $output;
    }

    /**
     * Function from /mod/prova/renderer.php
     *
     * Create a preview link
     *
     * @param $url contains a url to the given page
     */
    public function restart_preview_button($url) {
        return $this->single_button($url, get_string('startnewpreview', 'prova'));
    }

    /**
     * Display a prova navigation button.
     *
     * @param prova_nav_question_button $button
     * @return string HTML fragment.
     */
    protected function render_addprova_nav_question_button(addprova_nav_question_button $button) {
        $classes = array('qnbutton', $button->stateclass, $button->navmethod, 'btn', 'btn-secondary');
        $extrainfo = array();

        if ($button->currentpage) {
            $classes[] = 'thispage';
            $extrainfo[] = get_string('onthispage', 'prova');
        }

        // Flagged?
        if ($button->flagged) {
            $classes[] = 'flagged';
            $flaglabel = get_string('flagged', 'question');
        } else {
            $flaglabel = '';
        }
        $extrainfo[] = html_writer::tag('span', $flaglabel, array('class' => 'flagstate'));

        if (is_numeric($button->number)) {
            $qnostring = 'questionnonav';
        } else {
            $qnostring = 'questionnonavinfo';
        }

        $a = new stdClass();
        $a->number = $button->number;
        $a->attributes = implode(' ', $extrainfo);
        $tagcontents = html_writer::tag('span', '', array('class' => 'thispageholder')) .
                        html_writer::tag('span', '', array('class' => 'trafficlight')) .
                        get_string($qnostring, 'prova', $a);
        $tagattributes = array('class' => implode(' ', $classes), 'id' => $button->id,
                                  'title' => $button->statestring, 'data-quiz-page' => $button->page);

        if ($button->url) {
            return html_writer::link($button->url, $tagcontents, $tagattributes);
        } else {
            return html_writer::tag('span', $tagcontents, $tagattributes);
        }
    }
}