<?php
/**
 * Gera o form que será apresentado após o cadastro de uma questão.
 */
require_once('../config.php');
require_once($CFG->dirroot . '/mod/prova/managelib.php');

global $DB, $OUTPUT, $PAGE;

\mod_prova\manager::has_permission_to_access();

$originalreturnurl = optional_param('returnurl', null, PARAM_LOCALURL);
$lastchanged = optional_param('lastchanged', null, PARAM_INT);

$url_add = new moodle_url('/mod/prova/components/addquestions/view.php', array('lastchanged' => $lastchanged, 'returnurl' => $originalreturnurl));
$PAGE->set_url($url_add);
$PAGE->set_pagelayout('admin');
$PAGE->set_context(context_course::instance(SEOX_COURSE_ID));
$PAGE->set_heading(get_string('addquestions:finalize_question_title', 'provacomp_addquestions'));

/* new provacomp_addquestions\form\addquestions_form();*/

$thisblock_form = new mod_prova\form\seox_check_areas_form();

if($thisblock_form->is_cancelled()) {
	if ($originalreturnurl) {
		die(redirect($originalreturnurl));
	} 
    die(redirect(new moodle_url('/mod/prova/components/questions.php')));
} elseif ($fromform = $thisblock_form->get_data()) {
	$isAllClean = true;
	if ($fromform->md_areas == 0) {
	    print_error('addquestions:invalidarea', 'provacomp_addquestions', $questionid);
	    $isAllClean = false;
	}
	if ($fromform->md_subareas == 0) {
	    print_error('addquestions:invalidsubarea', 'provacomp_addquestions', $questionid);
	    $isAllClean = false;
	}
	if ($fromform->md_assuntos == 0) {
	    print_error('addquestions:invalidassunto', 'provacomp_addquestions', $questionid);
	    $isAllClean = false;
	}

	$data = array(
		'id_question' => $fromform->questionid,
		'id_area' => $fromform->md_areas,
		'id_subarea' => $fromform->md_subareas,
		'id_assunto' => $fromform->md_assuntos,
		'dificuldade' => $fromform->md_dificuldades,
		// TODO: add user, createdAt e updatedAt
	);

	if ($isAllClean) {
		if (array_key_exists('update', $fromform) and $fromform->update) {
			if ($data['id_area'] != $fromform->id_area
				or $data['id_subarea'] != $fromform->id_subarea
				or $data['id_assunto'] != $fromform->id_assunto
				or $data['dificuldade'] != $fromform->dificuldade) {
				$data['id'] = $fromform->seoxid;
				if (!$resultid = $DB->update_record_raw('seox_questions', $data)) {
			    	print_error('addquestions:addquestion_dberror_insert', 'provacomp_addquestions');
				}
			} 
			$resultid = $fromform->seoxid;
		} else {	
			if (!$resultid = $DB->insert_record_raw('seox_questions', $data)) {
			    print_error('addquestions:addquestion_dberror_insert', 'provacomp_addquestions');
			}
		}
		if (array_key_exists('returnurl', $fromform) and $fromform->returnurl and $fromform->returnurl != '') {
			$url = new moodle_url($fromform->returnurl);
			$url->param('sesskey', sesskey());
			redirect($url);
		} else {
    		redirect(new moodle_url('/mod/prova/components/questions.php', array('qid' => $resultid, 'sesskey' => sesskey())));
		}
		exit;
	}

} else {
	// ---- Pre exibition
	$questionid = required_param('lastchanged', PARAM_INT);
	$toform = array();
	if (!$question = $DB->get_record('question', array('id' => $questionid))) {
	    print_error('addquestions:invalidquestion', 'provacomp_addquestions', $questionid);
	}

	// Adiciona hidden params
	$toform['questionid'] = $questionid;
	if ($originalreturnurl) {
		$toform['returnurl'] = $originalreturnurl;
	}
	// Verifica se é insert ou edit
	if ($seox_question = $DB->get_record('seox_questions', array('id_question' => $questionid))) {
		$toform['update'] = true;//$seox_question;
		$toform['seoxid'] = $seox_question->id;
		$toform['id_area'] = $seox_question->id_area;
		$toform['id_subarea'] = $seox_question->id_subarea;
		$toform['id_assunto'] = $seox_question->id_assunto;
		$toform['dificuldade'] = $seox_question->dificuldade;

		$defaults = array();
		$defaults['md_areas'] = $seox_question->id_area;
		$defaults['md_subareas'] = $seox_question->id_subarea;
		$defaults['md_assuntos'] = $seox_question->id_assunto;
		$defaults['md_dificuldades'] = $seox_question->dificuldade;

		$thisblock_form->populate_elements($defaults);
	}
	$thisblock_form->set_data($toform);

	// ----
	// Exibição padrão
    echo $OUTPUT->header();

    echo html_writer::tag('h3', get_string('addquestions:finalize_question_details_1', 'provacomp_addquestions'), null);
	echo '<br>';

	$thisblock_form->display();

	$PAGE->requires->yui_module(
		'moodle-provacomp_buscaquestoes-filtro_searchquestions',
		'M.provacomp_buscaquestoes.filtro_searchquestions.init',
		$thisblock_form->get_filtros_areas_content()
	);

	echo $OUTPUT->footer();
}