<?php
defined('MOODLE_INTERNAL') || die;

function xmldb_prova_install() {
    global $DB, $CFG;

    $file = dirname(__DIR__) . '/styles.css';

    if (!is_writable($file)) {
        purge_caches();
        print_error('install_permission_error', 'mod_prova', new \moodle_url('/admin/plugins.php', array('sesskey' => sesskey(), 'uninstall' => 'mod_prova', 'confirm' => 1)));
        return false;
    }

    //Dados do novo curso
    $data = (Object) array(
        'category' => 1,
        'fullname' => 'Gestão de Exames',
        'shortname' => 'gestao_exames',
        'visible' => 0,
        'visibleold' => 0,
    );
    
    if (!$course = create_course($data)) {
        print_error('install_error', 'mod_prova');
        return false;
    }
    require_once($CFG->libdir . '/questionlib.php');
    question_make_default_categories(array(context_course::instance($course->id)));


    $trans = $DB->start_delegated_transaction();
    $DB->execute("UPDATE {course} SET format = 'site', sortorder = 9999999999 WHERE id = ?", [$course->id]);
    $trans->allow_commit();


    $css = '[data-courseid="'.$course->id.'"], .type_course [data-node-key="'.$course->id.'"], .list-group-item[data-key="'.$course->id.'"] { display: none !important; }';
    $start = 'gestao_exames_start';
    $end = 'gestao_exames_end';

    $lines  = file($file);
    $result = [];
    $permission = true;
    $has_ended = false;
    foreach($lines as $line) {
        if ($has_ended) {
            $result[] = $line;
            continue;
        }

        if ($permission) {
            if(stripos($line, $start) !== false) {
                $permission = false;
            }
            $result[] = $line;
        } else {
            if(stripos($line, $end) !== false) {
                $result[] = $css . "\r\n";
                $result[] = $line;
                $has_ended = true;
            }
        }
    }
    file_put_contents($file, $result);

    purge_caches();
    return true;
}