<?php
/****************************************************
 ******************** MENUEXAMES ********************
 ****************************************************/
global $DB;
if (defined('SEOX_COURSE_ID')) {
	global $DB, $PAGE;
	$contextid = context_course::instance(SEOX_COURSE_ID);
	$editingteacherrole = $DB->get_record('role', array('shortname' => 'editingteacher'));
	assign_capability('moodle/question:editall', CAP_PROHIBIT, $editingteacherrole->id, $contextid, true);
	assign_capability('moodle/question:editmine', CAP_PROHIBIT, $editingteacherrole->id, $contextid, true);
	$editingteacherrole = $DB->get_records('role');
	foreach ($editingteacherrole as $key => $value) {
		assign_capability('moodle/block:view', CAP_PROHIBIT, $value->id, $contextid, true);
	}
}

/****************************************************
 *********** External fragment Functions ************
 ****************************************************/
/**
 * Function from /mod/quiz/lib.php
 * Based on mod_quiz_output_fragment_quiz_question_bank()
 *
 * Generates the question bank in a fragment output. This allows
 * the question bank to be displayed in a modal.
 */
function mod_prova_output_fragment_prova_bank($args) {

	\mod_prova\manager::has_permission_to_access();

	global $CFG, $DB, $PAGE;

	$querystring = preg_replace('/^\?/', '', $args['querystring']);
	$params = [];
	parse_str($querystring, $params);

    $pagevars = array(
        'qpage'         => (array_key_exists('qpage', $params)) ? $params['qpage'] : 0,
        'qperpage'      => (array_key_exists('qperpage', $params)) ? $params['qperpage'] : 5, 
    );

    unset($params['qpage']);
    unset($params['qperpage']);

	$provabank = new mod_prova\prova\bank_view($thispageurl, $params);

	return $provabank->render($pagevars['qpage'], $pagevars['qperpage']);
}

/****************************************************
 *************** Prova View Functions ***************
 ****************************************************/
/**
 * Mark the activity completed (if required) and trigger the course_module_viewed event.
 *
 * @param  stdClass $prova       prova object
 * @param  stdClass $course     course object
 * @param  stdClass $cm         course module object
 * @param  stdClass $context    context object
 * @since Moodle 3.1
 */
function prova_view($prova, $course, $cm, $context) {

	$params = array(
		'objectid' => $prova->id,
		'context' => $context,
	);

	$event = \mod_prova\event\course_module_viewed::create($params);
	$event->add_record_snapshot('prova', $prova);
	$event->trigger();

	// Completion.
	$completion = new completion_info($course);
	$completion->set_module_viewed($cm);
}

/****************************************************
 ************* Prova Attempt Functions **************
 ****************************************************/
/**#@+
 * Option controlling what options are offered on the prova settings form.
 */
define('PROVA_MAX_ATTEMPT_OPTION', 10);
define('PROVA_MAX_QPP_OPTION', 50);
define('PROVA_MAX_DECIMAL_OPTION', 5);
define('PROVA_MAX_Q_DECIMAL_OPTION', 7);
/**#@-*/

/**#@+
 * Options determining how the grades from individual attempts are combined to give
 * the overall grade for a user
 */
define('PROVA_GRADEHIGHEST', '1');
define('PROVA_GRADEAVERAGE', '2');
define('PROVA_ATTEMPTFIRST', '3');
define('PROVA_ATTEMPTLAST', '4');
/**#@-*/

/**
 * @var int If start and end date for the prova are more than this many seconds apart
 * they will be represented by two separate events in the calendar
 */
define('PROVA_MAX_EVENT_LENGTH', 5 * 24 * 60 * 60); // 5 days.

/**#@+
 * Options for navigation method within provazes.
 */
define('PROVA_NAVMETHOD_FREE', 'free');
define('PROVA_NAVMETHOD_SEQ', 'sequential');
/**#@-*/

/**
 * Event types.
 */
define('PROVA_EVENT_TYPE_OPEN', 'open');
define('PROVA_EVENT_TYPE_CLOSE', 'close');

/** 
 * Definĩção 
 */
class mod_prova_root_display_options {
    /**#@+
     * @var integer bits used to indicate various times in relation to a
     * prova attempt.
     */
    const DURING =            0x10000;
    const IMMEDIATELY_AFTER = 0x01000;
    const LATER_WHILE_OPEN =  0x00100;
    const AFTER_CLOSE =       0x00010;
    /**#@-*/
}

function prova_add_instance($prova) {

	global $DB, $USER;

	$cmid = $prova->coursemodule;
	$userid = $USER->id;

	// unset($prova->competency_rule);

	$provaobj = new stdClass();
	$provaobj->userid = $userid;
	$provaobj->provaid = $prova->provaid;
	$provaobj->course = $prova->course;
	$provaobj->cmid = $cmid;
	$provaobj->name = $prova->name;
	$provaobj->intro = $prova->intro;
	$provaobj->introformat = $prova->introformat;
	$provaobj->timeopen = $prova->timeopen;
	$provaobj->timeclose = $prova->timeclose;
	$provaobj->timelimit = $prova->timelimit;
	$provaobj->password = $prova->password;
	$provaobj->timecreated = time();
	$provaobj->provacontrol = (array_key_exists('provacontrol', $prova) && $prova->provacontrol) ? 1 : 0;
	verify_permissions_by_timeclose($provaobj, $provaobj->timeclose);

	$prova->id = $DB->insert_record('prova', $provaobj);

	//Atualização do campo vezes_usada da tabela seox_prova
	$DB->execute('UPDATE {seox_prova} SET vezes_usada = vezes_usada + 1 WHERE id = ?', [$provaobj->provaid]);

	return $prova->id;
}

function prova_update_instance($prova) {

	global $CFG, $DB;

	$disable = false;
	$disableduration = false;

	if ($prova_attempts = $DB->get_records('prova_attempts', array('prova' => $prova->instance))) {
		$disable = true;
		$oldprova = $DB->get_record('prova', array('id' => $prova->instance));
		if ($oldprova->timeclose !=  $prova->timeclose) {
			foreach ($prova_attempts as $attempt) {
				unlock_prova($attempt->id, $attempt->prova, $attempt->userid);
			}
			$disableduration = true;
		}
	}

	$cmid = $prova->coursemodule;

	$provaobj = new stdClass();
	if (!$disable) {
		$provaobj->course = $prova->course;
		$provaobj->provaid = $prova->provaid;
		$provaobj->cmid = $cmid;
		$provaobj->name = $prova->name;
	}
	$provaobj->id = $prova->instance;
	$provaobj->intro = $prova->intro;
	$provaobj->introformat = $prova->introformat;
	$provaobj->timeopen = $prova->timeopen;
	$provaobj->timeclose = $prova->timeclose;
	$provaobj->timelimit = ($disableduration) ? 0 : $prova->timelimit;
	$provaobj->password = $prova->password;
	$provaobj->timemodified = time();
	$provaobj->provacontrol = (array_key_exists('provacontrol', $prova) && $prova->provacontrol) ? 1 : 0;
	verify_permissions_by_timeclose($provaobj, $provaobj->timeclose);

	$prova->id = $DB->update_record('prova', $provaobj);

	$completiontimeexpected = !empty($data->completionexpected) ? $data->completionexpected : null;
	\core_completion\api::update_completion_date_event($cmid, 'prova', $provaobj->id, $completiontimeexpected);

	return true;
}

function prova_delete_instance($id) {

	global $DB;

	$prova = $DB->get_record('prova', array('id' => $id), '*', MUST_EXIST);

	prova_delete_all_attempts($prova);

	//Atualização do campo vezes_usada da tabela seox_prova
	$DB->execute('UPDATE {seox_prova} SET vezes_usada = vezes_usada - 1 WHERE id = ?', [$prova->provaid]);

	$DB->delete_records('prova', array('id' => $prova->id));

	return true;
}

/**
 * Delete all the attempts belonging to a prova.
 *
 * @param object $prova The prova object.
 */
function prova_delete_all_attempts($prova) {
    global $CFG, $DB;
    require_once($CFG->dirroot . '/mod/prova/locallib.php');
    $DB->delete_records('prova_attempts', array('prova' => $prova->id));
    $DB->delete_records('prova_grades', array('prova' => $prova->id));
}

/**
 * Function from /mod/quiz/lib.php
 * Original name: quiz_get_user_attempts
 */
function prova_get_user_attempts($provaids, $userid, $status = 'finished', $includepreviews = false) {
	global $DB, $CFG;
	require_once $CFG->dirroot . '/mod/prova/locallib.php';

	$params = array();
	switch ($status) {
	case 'all':
		$statuscondition = '';
		break;

	case 'finished':
		$statuscondition = ' AND state IN (:state1, :state2)';
		$params['state1'] = prova_attempt::FINISHED;
		$params['state2'] = prova_attempt::ABANDONED;
		break;

	case 'unfinished':
		$statuscondition = ' AND state IN (:state1, :state2)';
		$params['state1'] = prova_attempt::IN_PROGRESS;
		$params['state2'] = prova_attempt::OVERDUE;
		break;
	}

	$provaids = (array) $provaids;
	list($insql, $inparams) = $DB->get_in_or_equal($provaids, SQL_PARAMS_NAMED);
	$params += $inparams;
	$params['userid'] = $userid;

	$previewclause = '';
	if (!$includepreviews) {
		$previewclause = ' AND preview = 0';
	}

	return $DB->get_records_select('prova_attempts',
		"prova $insql AND userid = :userid" . $previewclause . $statuscondition,
		$params, 'prova, attempt ASC');
}

/**
 * Garante que provas com só exibiram resultados se o tempo limite extourar.
 * Se não houver tempo limite, estudantes não poderão revisar
 *
 * @param \alias-object, alias do objeto da prova que será salvo
 * @param \int, tempo em que prova fecha
 */
function verify_permissions_by_timeclose(&$prova, $timeclose) {
	$code = mod_prova_root_display_options::AFTER_CLOSE;

	if ($timeclose > 0) {
		$prova->reviewcorrectness = $code;
		$prova->reviewmarks = $code;
		$prova->reviewspecificfeedback = $code;
		$prova->reviewgeneralfeedback = $code;
		$prova->reviewrightanswer = $code;
		$prova->reviewoverallfeedback = $code;
	}

}

/**
 * Is this a graded prova? If this method returns true, you can assume that
 * $prova->grade and $prova->sumgrades are non-zero (for example, if you want to
 * divide by them).
 *
 * @param object $prova a row from the prova table.
 * @return bool whether this is a graded prova.
 */
function prova_has_grades($prova) {
	return $prova->grade >= 0.000000 && $prova->sumgrades >= 0.000005;
}

/**
 * Round a grade to to the correct number of decimal places, and format it for display.
 *
 * @param object $prova The prova table row.
 * @param float $grade The grade to round.
 * @return float
 */
function modprova_format_grade($prova, $grade) {
	if (is_null($grade)) {
		return get_string('notyetgraded', 'prova');
	}
	return format_float($grade, $prova->decimalpoints);
}

/**
 * Determine the correct number of decimal places required to format a grade.
 *
 * @param object $prova The prova table row, only $prova->decimalpoints is used.
 * @return integer
 */
function prova_get_grade_format($prova) {
	if (empty($prova->questiondecimalpoints)) {
		$prova->questiondecimalpoints = -1;
	}

	if ($prova->questiondecimalpoints == -1) {
		return $prova->decimalpoints;
	}

	return $prova->questiondecimalpoints;
}

/**
 * Round a grade to the correct number of decimal places, and format it for display.
 *
 * @param object $prova The prova table row, only $prova->decimalpoints is used.
 * @param float $grade The grade to round.
 * @return float
 */
function modprova_format_question_grade($prova, $grade) {
	return format_float($grade, prova_get_grade_format($prova));
}

/**
 * Get the best current grade for a particular user in a prova.
 *
 * @param object $prova the prova settings.
 * @param int $userid the id of the user.
 * @return float the user's current grade for this prova, or null if this user does
 * not have a grade on this prova.
 */
function prova_get_best_grade($prova, $userid) {
	global $DB;
	$grade = $DB->get_field('prova_grades', 'grade',
		array('prova' => $prova->id, 'userid' => $userid));

	// Need to detect errors/no result, without catching 0 grades.
	if ($grade === false) {
		return null;
	}

	return $grade + 0; // Convert to number.
}

/**
 * Return a textual summary of the number of attempts that have been made at a particular prova,
 * returns '' if no attempts have been made yet, unless $returnzero is passed as true.
 *
 * @param object $prova the prova object. Only $prova->id is used at the moment.
 * @param object $cm the cm object. Only $cm->course, $cm->groupmode and
 *      $cm->groupingid fields are used at the moment.
 * @param bool $returnzero if false (default), when no attempts have been
 *      made '' is returned instead of 'Attempts: 0'.
 * @param int $currentgroup if there is a concept of current group where this method is being called
 *         (e.g. a report) pass it in here. Default 0 which means no current group.
 * @return string a string like "Attempts: 123", "Attemtps 123 (45 from your groups)" or
 *          "Attemtps 123 (45 from this group)".
 */
function prova_num_attempt_summary($prova, $cm, $returnzero = false, $currentgroup = 0) {
    global $DB, $USER;
    $numattempts = $DB->count_records('prova_attempts', array('prova'=> $prova->id, 'preview'=>0));
    if ($numattempts || $returnzero) {
        if (groups_get_activity_groupmode($cm)) {
            $a = new stdClass();
            $a->total = $numattempts;
            if ($currentgroup) {
                $a->group = $DB->count_records_sql('SELECT COUNT(DISTINCT qa.id) FROM ' .
                        '{prova_attempts} qa JOIN ' .
                        '{groups_members} gm ON qa.userid = gm.userid ' .
                        'WHERE prova = ? AND preview = 0 AND groupid = ?',
                        array($prova->id, $currentgroup));
                return get_string('attemptsnumthisgroup', 'prova', $a);
            } else if ($groups = groups_get_all_groups($cm->course, $USER->id, $cm->groupingid)) {
                list($usql, $params) = $DB->get_in_or_equal(array_keys($groups));
                $a->group = $DB->count_records_sql('SELECT COUNT(DISTINCT qa.id) FROM ' .
                        '{prova_attempts} qa JOIN ' .
                        '{groups_members} gm ON qa.userid = gm.userid ' .
                        'WHERE prova = ? AND preview = 0 AND ' .
                        "groupid $usql", array_merge(array($prova->id), $params));
                return get_string('attemptsnumyourgroups', 'prova', $a);
            }
        }
        return get_string('attemptsnum', 'prova', $numattempts);
    }
    return '';
}

/**
 * Does this prova allow multiple tries?
 *
 * @return bool
 */
function prova_allows_multiple_tries($prova) {
    $bt = question_engine::get_behaviour_type($prova->preferredbehaviour);
    return $bt->allows_multiple_submitted_responses();
}


/**
 * Update grades in central gradebook
 *
 * @category grade
 * @param object $prova the prova settings.
 * @param int $userid specific user only, 0 means all users.
 * @param bool $nullifnone If a single user is specified and $nullifnone is true a grade item with a null rawgrade will be inserted
 */
function prova_update_grades($prova, $userid = 0, $nullifnone = true) {
    global $CFG, $DB;
    require_once($CFG->libdir . '/gradelib.php');

    if ($prova->grade == 0) {
        prova_grade_item_update($prova);

    } else if ($grades = quiz_get_user_grades($prova, $userid)) {
        prova_grade_item_update($prova, $grades);

    } else if ($userid && $nullifnone) {
        $grade = new stdClass();
        $grade->userid = $userid;
        $grade->rawgrade = null;
        prova_grade_item_update($prova, $grade);

    } else {
        prova_grade_item_update($prova);
    }
}

/**
 * Create or update the grade item for given quiz
 *
 * @category grade
 * @param object $prova object with extra cmidnumber
 * @param mixed $grades optional array/object of grade(s); 'reset' means reset grades in gradebook
 * @return int 0 if ok, error code otherwise
 */
function prova_grade_item_update($prova, $grades = null) {
    global $CFG, $OUTPUT;
    require_once($CFG->dirroot . '/mod/prova/locallib.php');
    require_once($CFG->libdir . '/gradelib.php');

    if (array_key_exists('cmidnumber', $prova)) { // May not be always present.
        $params = array('itemname' => $prova->name, 'idnumber' => $prova->cmidnumber);
    } else {
        $params = array('itemname' => $prova->name);
    }

    if ($prova->grade > 0) {
        $params['gradetype'] = GRADE_TYPE_VALUE;
        $params['grademax']  = $prova->grade;
        $params['grademin']  = 0;

    } else {
        $params['gradetype'] = GRADE_TYPE_NONE;
    }

    // What this is trying to do:
    // 1. If the prova is set to not show grades while the prova is still open,
    //    and is set to show grades after the prova is closed, then create the
    //    grade_item with a show-after date that is the prova close date.
    // 2. If the prova is set to not show grades at either of those times,
    //    create the grade_item as hidden.
    // 3. If the prova is set to show grades, create the grade_item visible.
    $openreviewoptions = mod_prova_display_options::make_from_prova($prova,
            mod_prova_display_options::LATER_WHILE_OPEN);
    $closedreviewoptions = mod_prova_display_options::make_from_prova($prova,
            mod_prova_display_options::AFTER_CLOSE);
    if ($openreviewoptions->marks < question_display_options::MARK_AND_MAX &&
            $closedreviewoptions->marks < question_display_options::MARK_AND_MAX) {
        $params['hidden'] = 1;

    } else if ($openreviewoptions->marks < question_display_options::MARK_AND_MAX &&
            $closedreviewoptions->marks >= question_display_options::MARK_AND_MAX) {
        if ($quiz->timeclose) {
            $params['hidden'] = $quiz->timeclose;
        } else {
            $params['hidden'] = 1;
        }

    } else {
        // Either
        // a) both open and closed enabled
        // b) open enabled, closed disabled - we can not "hide after",
        //    grades are kept visible even after closing.
        $params['hidden'] = 0;
    }

    if (!$params['hidden']) {
        // If the grade item is not hidden by the prova logic, then we need to
        // hide it if the quiz is hidden from students.
        if (property_exists($prova, 'visible')) {
            // Saving the quiz form, and cm not yet updated in the database.
            $params['hidden'] = !$prova->visible;
        } else {
            $cm = get_coursemodule_from_instance('prova', $prova->id);
            $params['hidden'] = !$cm->visible;
        }
    }

    if ($grades  === 'reset') {
        $params['reset'] = true;
        $grades = null;
    }

    $gradebook_grades = grade_get_grades($prova->course, 'mod', 'prova', $prova->id);
    if (!empty($gradebook_grades->items)) {
        $grade_item = $gradebook_grades->items[0];
        if ($grade_item->locked) {
            // NOTE: this is an extremely nasty hack! It is not a bug if this confirmation fails badly. --skodak.
            $confirm_regrade = optional_param('confirm_regrade', 0, PARAM_INT);
            if (!$confirm_regrade) {
                if (!AJAX_SCRIPT) {
                    $message = get_string('gradeitemislocked', 'grades');
                    $back_link = $CFG->wwwroot . '/mod/prova/src/report.php?p=' . $prova->id .
                            '&amp;mode=overview';
                    $regrade_link = qualified_me() . '&amp;confirm_regrade=1';
                    echo $OUTPUT->box_start('generalbox', 'notice');
                    echo '<p>'. $message .'</p>';
                    echo $OUTPUT->container_start('buttons');
                    echo $OUTPUT->single_button($regrade_link, get_string('regradeanyway', 'grades'));
                    echo $OUTPUT->single_button($back_link,  get_string('cancel'));
                    echo $OUTPUT->container_end();
                    echo $OUTPUT->box_end();
                }
                return GRADE_UPDATE_ITEM_LOCKED;
            }
        }
    }

    return grade_update('mod/prova', $prova->course, 'mod', 'prova', $prova->id, 0, $grades, $params);
}

/**
 * Return grade for given user or all users.
 *
 * @param int $provaid id of prova
 * @param int $userid optional user id, 0 means all users
 * @return array array of grades, false if none. These are raw grades. They should
 * be processed with quiz_format_grade for display.
 */
function prova_get_user_grades($prova, $userid = 0) {
    global $CFG, $DB;

    $params = array($prova->id);
    $usertest = '';
    if ($userid) {
        $params[] = $userid;
        $usertest = 'AND u.id = ?';
    }
    return $DB->get_records_sql("
            SELECT
                u.id,
                u.id AS userid,
                qg.grade AS rawgrade,
                qg.timemodified AS dategraded,
                MAX(qa.timefinish) AS datesubmitted

            FROM {user} u
            JOIN {prova_grades} pg ON u.id = pg.userid
            JOIN {prova_attempts} pa ON pa.prova = pg.prova AND pa.userid = u.id

            WHERE pg.prova = ?
            $usertest
            GROUP BY u.id, pg.grade, pg.timemodified", $params);
}


// Default moodle overrides ================================================================
/**
 * Get icon mapping for font-awesome.
 */
function mod_prova_get_fontawesome_icon_map() {
    return [
        'mod_prova:icon-correcao' => 'fa-flag',
    ];
}

/**
 * This function extends the settings navigation block for the site.
 *
 * It is safe to rely on PAGE here as we will only ever be within the module
 * context when this is called
 *
 * @param settings_navigation $settings
 * @param navigation_node $provanode
 * @return void
 */
function prova_extend_settings_navigation($settings, $provanode) {
    global $PAGE, $CFG, $DB;

    // // Require {@link questionlib.php}
    // // Included here as we only ever want to include this file if we really need to.
    // require_once($CFG->libdir . '/questionlib.php');

    // We want to add these new nodes after the Edit settings node, and before the
    // Locally assigned roles node. Of course, both of those are controlled by capabilities.
    $keys = $provanode->get_children_key_list();
    $beforekey = null;
    $i = array_search('modedit', $keys);
    if ($i === false and array_key_exists(0, $keys)) {
        $beforekey = $keys[0];
    } else if (array_key_exists($i + 1, $keys)) {
        $beforekey = $keys[$i + 1];
    }

    $returnurl = new moodle_url('/mod/prova/view.php', array('id' => $PAGE->cm->id));

    if (has_capability('mod/prova:manage', $PAGE->cm->context)) {
    	if ($provaobj = $DB->get_record('prova', array('id' => $PAGE->cm->instance))) {
	    	$url = new moodle_url('/mod/prova/components/addprovas/addprova.php', array('provaid' => $provaobj->provaid, 'returnurl' => $returnurl));
	        $node = navigation_node::create(
				get_string('editprova', 'prova'),
				$url,
				navigation_node::TYPE_SETTING, 
				null, 
				'provacomp_addprovas_edit',
				new pix_icon('t/edit', '')
			);
	        $provanode->add_node($node, $beforekey);
        }
    }

    if (has_capability('mod/prova:manage', $PAGE->cm->context)) {
        $url = new moodle_url('/mod/prova/viewattempts.php', array('appid'=>$provaobj->id, 'cm' => $PAGE->cm->id));
        $node = navigation_node::create(
			get_string('printprovacmalias', 'prova'), 
			$url, 
			navigation_node::TYPE_SETTING,
			null, 
			null, 
			new pix_icon('t/print', ''));
        $provanode->add_node($node, $beforekey);
    }

    if (has_capability('mod/prova:grade', $PAGE->cm->context)) {
        $url = new moodle_url('/mod/prova/screens/report/report.php', array('id'=>$PAGE->cm->id, 'mode' => 'responses'));
        $node = navigation_node::create(
			get_string('viewtablecorrectbtn', 'prova'), 
			$url, 
			navigation_node::TYPE_SETTING,
			null, 
			null, 
			new pix_icon('correcticon', '', 'mod_prova'));
        $provanode->add_node($node, $beforekey);
    }

    $provaobj = $DB->get_record('prova', array('id' => $PAGE->cm->instance));

    if($provaobj->timelimit){
        if (has_capability('mod/prova:grade', $PAGE->cm->context)) {
            $url = new moodle_url('/mod/prova/screens/time_prova/view.php', array('id'=>$PAGE->cm->id));
            $node = navigation_node::create(
            	get_string('timeincrementprova', 'prova'), 
            	$url, 
            	navigation_node::TYPE_SETTING,
				null, 
				null, 
				new pix_icon('time_icon', '', 'mod_prova'));
            $provanode->add_node($node, $beforekey);
        }
    }
    
    if (has_capability('mod/prova:grade', $PAGE->cm->context)) {
        $url = new moodle_url('/mod/prova/screens/liberarprova/view.php', array('appid'=>$provaobj->id));
        $node = navigation_node::create(
        	get_string('releaseprova', 'prova'), 
        	$url, 
        	navigation_node::TYPE_SETTING,
			null, 
			null, 
			new pix_icon('lock-open-icon', '', 'mod_prova'));
        $provanode->add_node($node, $beforekey);
    }

}

/**
 * Serves the prova files.
 *
 * @package  mod_prova
 * @category files
 * @param stdClass $course course object
 * @param stdClass $cm course module object
 * @param stdClass $context context object
 * @param string $filearea file area
 * @param array $args extra arguments
 * @param bool $forcedownload whether or not force download
 * @param array $options additional options affecting the file serving
 * @return bool false if file not found, does not return if found - justsend the file
 */
function prova_pluginfile($course, $cm, $context, $filearea, $args, $forcedownload, array $options=array()) {
    global $CFG, $DB;

    if ($context->contextlevel != CONTEXT_MODULE) {
        return false;
    }

    require_login($course, false, $cm);

    if (!$prova = $DB->get_record('prova', array('id'=>$cm->instance))) {
        return false;
    }

    // The 'intro' area is served by pluginfile.php.
    $fileareas = array('feedback');
    if (!in_array($filearea, $fileareas)) {
        return false;
    }

    $feedbackid = (int)array_shift($args);
    if (!$feedback = $DB->get_record('prova_feedback', array('id'=>$feedbackid))) {
        return false;
    }

    $fs = get_file_storage();
    $relativepath = implode('/', $args);
    $fullpath = "/$context->id/mod_prova/$filearea/$feedbackid/$relativepath";
    if (!$file = $fs->get_file_by_hash(sha1($fullpath)) or $file->is_directory()) {
        return false;
    }
    send_stored_file($file, 0, 0, true, $options);
}

/**
 * Called via pluginfile.php -> question_pluginfile to serve files belonging to
 * a question in a question_attempt when that attempt is a prova attempt.
 *
 * @package  mod_prova
 * @category files
 * @param stdClass $course course settings object
 * @param stdClass $context context object
 * @param string $component the name of the component we are serving files for.
 * @param string $filearea the name of the file area.
 * @param int $qubaid the attempt usage id.
 * @param int $slot the id of a question in this prova attempt.
 * @param array $args the remaining bits of the file path.
 * @param bool $forcedownload whether the user must be forced to download the file.
 * @param array $options additional options affecting the file serving
 * @return bool false if file not found, does not return if found - justsend the file
 */
function prova_question_pluginfile($course, $context, $component,
        $filearea, $qubaid, $slot, $args, $forcedownload, array $options=array()) {
    global $CFG;
    require_once($CFG->dirroot . '/mod/prova/locallib.php');

    $attemptobj = prova_attempt::create_from_usage_id($qubaid);
    require_login($attemptobj->get_course(), false, $attemptobj->get_cm());

    if ($attemptobj->is_own_attempt() && !$attemptobj->is_finished()) {
        // In the middle of an attempt.
        if (!$attemptobj->is_preview_user()) {
            $attemptobj->require_capability('mod/prova:attempt');
        }
        $isreviewing = false;

    } else {
        // Reviewing an attempt.
        $attemptobj->check_review_capability();
        $isreviewing = true;
    }

    if (!$attemptobj->check_file_access($slot, $isreviewing, $context->id,
            $component, $filearea, $args, $forcedownload)) {
        send_file_not_found();
    }

    $fs = get_file_storage();
    $relativepath = implode('/', $args);
    $fullpath = "/$context->id/$component/$filearea/$relativepath";
    if (!$file = $fs->get_file_by_hash(sha1($fullpath)) or $file->is_directory()) {
        send_file_not_found();
    }

    send_stored_file($file, 0, 0, $forcedownload, $options);
}

/**
 * Libera provas que já foram finalizadas por alunos
 */
function unlock_prova($p_attid, $appid, $userid){
    global $DB;

    $transaction = $DB->start_delegated_transaction();

    $timenow = time();

    //Pega campo uniqueid
    $prova_attempt = $DB->get_record('prova_attempts', array('id' => $p_attid, 'prova' => $appid, 'userid' => $userid), 'uniqueid', MUST_EXIST);
    
    //Atualizar o campo state
    $query_update = $DB->execute("UPDATE {prova_attempts} SET state = 'inprogress', timecheckstate = ? WHERE id = ? and prova = ? and userid = ?", [$timenow, $p_attid, $appid, $userid]);

    $return = ($query_update) ? 1 : 2;

    //Pegar os IDs da tentativas da questões da prova a ser liberada
    $sql = "  SELECT id 
                FROM {question_attempts}
                WHERE questionusageid = $prova_attempt->uniqueid ";

    $question_attempt = $DB->get_records_sql($sql);

    $return = ($question_attempt) ? 1 : 2;

    $array_question = [];
    foreach ($question_attempt as $value) {
        $array_question[] = $value->id;
    }


    //Pegar o ID da question_attempt_steps
    $array_qstep_id = [];
    foreach ($array_question as $id_question) {

        $sql = "  SELECT id, questionattemptid, sequencenumber, state
        FROM {question_attempt_steps}
        WHERE questionattemptid = $id_question
            AND state in ('gradedright', 'gradedwrong', 'needsgrading')";

        $question_attempt_steps = $DB->get_records_sql($sql);

        $return = ($question_attempt_steps) ? 1 : 2;

        foreach ($question_attempt_steps as $value) {
            $array_qstep_id[] = $value->id;
        }

        $sqlextrarow = "SELECT s.id
        	FROM {question_attempt_steps} s
        	JOIN ( $sql ) targ
        		ON s.questionattemptid = targ.questionattemptid AND s.id != targ.id
        	WHERE s.sequencenumber >= targ.sequencenumber";
        
        $sqlextrarow_question_attempt_steps = $DB->get_records_sql($sqlextrarow);
     
        foreach ($sqlextrarow_question_attempt_steps as $value) {
            $array_qstep_id[] = $value->id;
        }

    }


    //Deletar as tentativas de uma questão da prova
    foreach ($array_qstep_id as $qstep_id) {
       
        //Deletar linha de um determinado attemptstepid
        $delete_step_data = $DB->delete_records('question_attempt_step_data', array('attemptstepid' => $qstep_id));

        $return = ($delete_step_data) ? 1 : 2;

        //Deletar a linha de um determinado ID da tabela mdl_question_attempt_steps 
        $delete_attempt_steps = $DB->delete_records('question_attempt_steps', array('id' => $qstep_id));
        
        $return = ($delete_attempt_steps) ? 1 : 2;
    }

    $update_gaveup_sql = "UPDATE {question_attempt_steps} SET state = 'todo' WHERE state = 'gaveup' AND questionattemptid IN (" . implode(', ', $array_question) . ")";
    $DB->execute($update_gaveup_sql);

    $transaction->allow_commit();

    return $return;
}