<?php

require_once $CFG->dirroot . '/course/moodleform_mod.php';
require_once $CFG->dirroot . '/mod/prova/lib.php';

$return_url = optional_param('returnurl', 0, PARAM_URL);
$PAGE->navbar->add(get_string('form_return_title', 'prova'), new moodle_url($return_url));

class mod_prova_mod_form extends moodleform_mod {

	public static $datefieldoptions = array('optional' => true);

	public function definition() {

		global $COURSE, $PAGE, $DB, $OUTPUT;
		
		$mform = $this->_form;

		$provaid = optional_param('provaid', null, PARAM_INT);
		$cmid_update = optional_param('update', null, PARAM_INT);
		$cmid_update = ($cmid_update) ? $DB->get_record('prova', array('cmid' => $cmid_update), 'id, provaid') : null;

//-------------------------------------------------------------------------------
		$mform->addElement('header', 'newmodulefieldset', get_string('fieldset', 'prova'));

		$output = $PAGE->get_renderer('mod_prova', 'mod_form');
		// Verifica se prova já tem algum attempt
		$disable = $this->disable_buttons($cmid_update);
		// --- Lista de provas
		if ($disable !== null) {
			// Warning para adicionar provas no ínicio
			$warningtext = html_writer::tag('p', get_string('warningselectprova', 'prova'));
			$warningbody = html_writer::tag('span', $warningtext, array('id' => 'warningselectprova'));
			$mform->addElement('html', $warningbody);

			// Botao de adicionar provas na lista
			$button = $output->seox_add_menu_actions();
			$button_add = html_writer::tag('span', $button, array('class' => 'add-menu-outer'));
			$mform->addElement('html', $button_add);
		}
		$mform->addElement('html', $this->print_table_seox($provaid, $cmid_update, $disable));
		
		$PAGE->requires->js_call_amd('mod_prova/prova_table_functions', 'init', []);

		$PAGE->requires->js_call_amd('mod_prova/provabank', 'init', [
			$PAGE->context->id,
			$PAGE->url->get_path()
		]);
		
		$mform->addElement('hidden', 'provaid', '');
		$mform->setType('provaid', PARAM_INT);
		$mform->addRule('provaid', get_string('required'), 'required', null, 'client');

//-------------------------------------------------------------------------------
		/// Adding the "general" fieldset, where all the common settings are showed
		$mform->addElement('header', 'general', get_string('general', 'form'));

		/// Adding the standard "name" field
		$mform->addElement('text', 'name', get_string('name', 'prova'), array('size' => '64'));
		$mform->setType('name', PARAM_TEXT);
		$mform->addRule('name', null, 'required', null, 'client');
		$mform->addRule('name', get_string('maximumchars', '', 255), 'maxlength', 255, 'client');

		/// Adding the required "intro" field to hold the description of the instance
		$this->standard_intro_elements(get_string('intro', 'prova'));
		
// -------------------------------------------------------------------------------
		// Botão de controle de prova

		$mform->addElement('header', 'newmodulefieldset', get_string('provacontrol_header', 'prova'));

		$mform->addElement('checkbox', 'provacontrol', get_string('provacontrol', 'prova'));
		$mform->setType('provacontrol', PARAM_INT);

		$mform->addHelpButton('provacontrol', 'provacontrol', 'prova');

// -------------------------------------------------------------------------------
		$mform->addElement('header', 'newmodulefieldset', get_string('timing', 'prova'));

		// Open and close dates.
		$mform->addElement('date_time_selector', 'timeopen', get_string('provaopen', 'prova'),
			self::$datefieldoptions);
		$mform->addHelpButton('timeopen', 'provaopenclose', 'prova');

		$mform->addElement('date_time_selector', 'timeclose', get_string('provaclose', 'prova'),
			self::$datefieldoptions);

		// Time limit.
		$mform->addElement('duration', 'timelimit', get_string('timelimit', 'prova'),
			array('optional' => true));
		//$mform->addHelpButton('timelimit', 'timelimit', 'prova');

//-------------------------------------------------------------------------------
		$mform->addElement('header', 'security', get_string('provapassword', 'prova'));

		$mform->addElement('passwordunmask', 'password', get_string('requirepassword', 'prova'));
		$mform->setType('password', PARAM_TEXT);

//-------------------------------------------------------------------------------
		// add standard elements, common to all modules
		$this->standard_coursemodule_elements();

//-------------------------------------------------------------------------------
		// add standard buttons, common to all modules
		$this->add_action_buttons();

		// $mform->disabledIf('submitbutton2', $disable, 'eq', null);
		// $mform->disabledIf('submitbutton', $disable, 'eq', null);
		// $mform->disabledIf('name', $disable, 'eq', null);

		// $mform->disabledIf('timeopen', $disable, 'eq', null);
		// $mform->disabledIf('timeclose', $disable, 'eq', null);
		$mform->disabledIf('timelimit', $disable, 'eq', null);
		// $mform->disabledIf('password', $disable, 'eq', null);
		$mform->disabledIf('visible', $disable, 'eq', null);
		$mform->disabledIf('cmidnumber', $disable, 'eq', null);
		$mform->disabledIf('completion', $disable, 'eq', null);
		$mform->disabledIf('completionexpected', $disable, 'eq', null);
		$mform->disabledIf('competency_rule', $disable, 'eq', null);
		$mform->disabledIf('#id_introeditoreditable', $disable, 'eq', null);
	}

	public function disable_buttons($cmid_update) {
		global $DB;

		$var = 1;
		if ($cmid_update) {
			$prova_attempts = $DB->record_exists('prova_attempts', array('prova' => $cmid_update->id));

			if (!$prova_attempts) {
				$var = 0;
			} else {
				$var = null;
			}
		}
		return $var;
	}

	public function print_table_seox($provaid, $cmid_update, $disable) {
		global $OUTPUT, $PAGE;

		$pageurl = $PAGE->url;
		$queries = $PAGE->url->params();

		$selected = [];
		if ($disable !== null) {
			if ($provaid) {
				$selected = $this->buil_prova_view_query($provaid, true);
			} elseif($cmid_update and $cmid_update->provaid) {
				$selected = $this->buil_prova_view_query($cmid_update->provaid, true);
			}
		}

		$lastfive = [];
		if ($disable === null) {
			$lastfive = $this->buil_prova_view_query($cmid_update->provaid, true);
		} elseif ($provaid) {
			$lastfive = $this->buil_prova_view_query($provaid);
		} elseif ($cmid_update and $cmid_update->provaid) {
			$lastfive = $this->buil_prova_view_query($cmid_update->provaid);
		} else {
			$lastfive = $this->buil_prova_view_query();
		}
		$results = array_merge($selected, $lastfive);

		if (!empty($results)) {
			$table = new html_table();

			$table->head = array('Nome', 'Área', 'Subárea', 'Assunto', 'Dificuldade', 'Vezes Utilizada', 'Qtd de Questões', 'Ações', 'Selecionar');
			$table->attributes["class"] = "generaltable bankprovatable";

			// Gera o body da tabela
			foreach ($results as $prova) {
				$vezes_usada = $prova->vezes_usada;

				$row = new html_table_row();
				$row = new \html_table_row();

				$row->attributes = $this->set_attributes($prova, $provaid, $cmid_update);
				$row->cells[] = $this->prepare_name($prova->nome, $queries);
				$row->cells[] = $prova->area;
				$row->cells[] = $prova->subarea;
				$row->cells[] = $prova->assunto;
				$row->cells[] = \provacomp_buscaprovas\form\buscaprovas_form::$dificuldadeslist[$prova->dificuldade];
				$row->cells[] = intval($vezes_usada);
				$row->cells[] = intval($prova->nro_questions);
				$row->cells[] = $this->prepare_acoes($prova->provaid, $pageurl, $vezes_usada);
				$row->cells[] = $this->choose_row($prova, $provaid, $cmid_update, $selected);

				$table->data[] = $row;
			}
			$table_form = html_writer::table($table);
		} else {
			$table_form = html_writer::tag('p', get_string('noresults_at_search', 'provacomp_buscaprovas'));
		}

		return $table_form;
	}

	protected function buil_prova_view_query($provaid = false, $selected = false) {
		global $DB;

		$sql = 'SELECT  prova.id AS provaid,
                        prova.name AS nome,
                        area.name AS area,
                        subarea.name AS subarea,
                        assunto.name AS assunto,
                        prova.dificuldade AS dificuldade,
                        prova.vezes_usada AS vezes_usada,
                        COUNT(slots.id) AS nro_questions
                FROM {seox_prova} prova
                INNER JOIN {seox_subarea} subarea
                ON subarea.id = prova.id_subarea
                INNER JOIN {seox_assunto} assunto
                ON assunto.id = prova.id_assunto
                INNER JOIN {seox_prova_slots} slots
                ON prova.id = slots.provaid
                INNER JOIN {seox_area} area
                ON prova.id_area = area.id';

        if ($provaid) {
	        if ($selected) {
	        	$sql .= ' WHERE prova.id = ' . $provaid;
	        } else {
	        	$sql .= ' WHERE prova.id != ' . $provaid;
	        }
	    }

		$sql .= ' GROUP BY prova.id, area.id, subarea.id, assunto.id ORDER BY prova.id DESC LIMIT 5;';

		return $DB->get_records_sql($sql);
	}

	protected function prepare_name($field, $queries) {
		$field = strip_tags($field);
		$begin = 0;
		$pos = ($begin + 60 < strlen($field) - 1) ? '...' : '';
		return substr($field, $begin, 60) . $pos;
	}

	protected function prepare_acoes($provaid, $pageurl, $vezes_usada) {
		global $OUTPUT;
		// Actions Field
		// -- Preview
		$last_column = '<div style="display:inline-block; width: 100px">';
		$previewurl = new moodle_url('/mod/prova/components/addprovas/preview.php', array(
			'provaid' => $provaid,
		));
		$previewimage = $OUTPUT->pix_icon('t/preview', get_string('preview'), '', array('class' => 'iconsmall'));
		$previewaction = new popup_action('click', $previewurl, 'questionpreview', array(
			'height' => 600,
			'width' => 800,
		));
		$last_column .= $OUTPUT->action_link($previewurl, $previewimage, $previewaction, array());

		if ($vezes_usada <= 0) {
			$returnurl = $pageurl->out_as_local_url(false);

			// -- Edit
			$editurl = new moodle_url('/mod/prova/components/addprovas/addprova.php', array(
				'provaid' => $provaid,
				'returnurl' => $returnurl,
			));
			$editaction = $OUTPUT->action_icon($editurl, new pix_icon('t/edit', get_string('edit')), null, array('class' => 'seox_leanblocks-action'));
			$last_column .= $editaction;

			// -- Delete
			$deleteurl = new moodle_url('/mod/prova/components/buscaprovas/deleteprova.php', array(
				'deleteselected' => $provaid,
				'sesskey' => sesskey(),
				'returnurl' => $returnurl,
			));
			$deleteaction = $OUTPUT->action_icon($deleteurl, new pix_icon('t/delete', get_string('delete')), null, array('class' => 'seox_leanblocks-action'));
			$last_column .= $deleteaction;
		}
		return $last_column . '</div>';
	}

	protected function set_attributes($prova, $provaid, $cmid_update) {
		global $DB;
		$attrs = ['class' => 'provarow'];
		if ($provaid) {
			if ($prova->provaid == $provaid) {
				$attrs['checked'] = 'checked';
			}
		} elseif ($cmid_update) {
			if ($prova->provaid == $cmid_update->provaid) {
				$attrs['checked'] = 'checked';
			}
		}
		return $attrs;
	}

	protected function choose_row($prova, $provaid, &$cmid_update) {
		global $DB;
		$params = array('type' => 'radio', 'name' => 'provaid', 'value' => $prova->provaid, 'required' => 'true');
		if ($provaid) {
			if ($prova->provaid == $provaid) {
				$params['checked'] = true;
			}
		} elseif ($cmid_update) {
			if ($prova->provaid == $cmid_update->provaid) {
				$params['checked'] = true;
			}
		} 

		return html_writer::empty_tag('input', $params);
	}
}
