<?php
/**
 * Defines the renderer for the block seox_addprovas.
 */

defined('MOODLE_INTERNAL') || die();


class mod_prova_renderer extends plugin_renderer_base {

    /**
     * Renders the review question pop-up.
     *
     * @param prova_attempt $attemptobj an instance of prova_attempt.
     * @param int $slot which question to display.
     * @param int $seq which step of the question attempt to show. null = latest.
     * @param mod_prova_display_options $displayoptions instance of mod_prova_display_options.
     * @param array $summarydata contains all table data
     * @return $output containing html data.
     */
    public function review_question_page(prova_attempt $attemptobj, $slot, $seq,
            mod_prova_display_options $displayoptions, $summarydata, $slots, $page) {

        $output = '';
        $output .= $this->header();
        $output .= $this->review_summary_table($summarydata, 0);

        if (!is_null($seq)) {
            $output .= $attemptobj->render_question_at_step($slot, $seq, true, $this);
        } else {
            $output .= $attemptobj->render_question($slot, true, $this);
        }

        if ($slots && $page >= 0) {
            $output .= $this->review_question_next_navigation($attemptobj, $slot, $slots, $page);
        } else {
            $output .= $this->close_window_button();
        }
        $output .= $this->get_navigation_panel_reviewquestion($attemptobj, $slot, $slots);
        $output .= $this->footer();
        return $output;
    }

    /**
     * Creates the navigation links/buttons at the bottom of the reivew attempt page.
     *
     * Note, the name of this function is no longer accurate, but when the design
     * changed, it was decided to keep the old name for backwards compatibility.
     *
     * @param prova_attempt $attemptobj instance of prova_attempt
     * @param int $page the current page
     * @param bool $lastpage if true current page is the last page
     *
     * @return string HTML fragment.
     */
    public function review_question_next_navigation(prova_attempt $attemptobj, $slot, $strslot, $page) {

        $slots = explode('-', $strslot);

        $nav = '';
        if ($page > 0) {
            $reviewparams = array('attempt' => $attemptobj->get_attemptid(), 'slot' => $slots[$page-1], 'slots' => $strslot, 'page' => $page-1);
            $url = new moodle_url('/mod/prova/screens/report/reviewquestion.php', $reviewparams);
            $nav .= link_arrow_left(get_string('navigateprevious', 'prova'),
                    $url, false, 'mod_prova-prev-nav');
        }
        if ($slot == end($slots)) {
            $nav .= $this->finish_review_question_link($attemptobj->get_report_url());
        } else {
            $reviewparams = array('attempt' => $attemptobj->get_attemptid(), 'slot' => $slots[$page+1], 'slots' => $strslot, 'page' => $page+1);
            $url = new moodle_url('/mod/prova/screens/report/reviewquestion.php', $reviewparams);
            $nav .= link_arrow_right(get_string('navigatenext', 'prova'),
                    $url, false, 'mod_prova-next-nav');
        }
        $output = html_writer::start_div('printarea', array('id' => 'printarea'));
        $output .= html_writer::tag('div', $nav, array('class' => 'submitbtns', 'style' => 'display:flex;justify-content:center;'));
        $output .= html_writer::end_div();
        return $output;
    }

    public function finish_review_question_link($reteurnurl) {
        $this->page->requires->js_init_call('M.mod_prova.secure_window.init_close_button', array($reteurnurl), false, prova_get_js_module());
        return html_writer::empty_tag('input', array('type' => 'button',
                'value' => get_string('finishreview', 'prova'),
                'id' => 'secureclosebutton',
                'class' => 'mod_prova-next-nav btn btn-primary'));
    }


    /**
     * Return the HTML of the prova timer.
     * @return string HTML content.
     */
    public function countdown_timer(prova_attempt $attemptobj, $timenow) {

        $timeleft = $attemptobj->get_time_left_display($timenow);
        if ($timeleft !== false) {
            $ispreview = $attemptobj->is_preview();
            $timerstartvalue = $timeleft;
            if (!$ispreview) {
                // Make sure the timer starts just above zero. If $timeleft was <= 0, then
                // this will just have the effect of causing the prova to be submitted immediately.
                $timerstartvalue = max($timerstartvalue, 1);
            }
            $this->initialise_timer($timerstartvalue, $ispreview);
        }

        return html_writer::tag('div', get_string('timeleft', 'prova') . ' ' .
                html_writer::tag('span', '', array('id' => 'quiz-time-left')),
                array('id' => 'quiz-timer', 'role' => 'timer',
                    'aria-atomic' => 'true', 'aria-relevant' => 'text'));
    }

    


    /**
     * Review Page
     */
    /**
     * Builds the review page
     *
     * @param prova_attempt $attemptobj an instance of prova_attempt.
     * @param array $slots an array of intgers relating to questions.
     * @param int $page the current page number
     * @param bool $showall whether to show entire attempt on one page.
     * @param bool $lastpage if true the current page is the last page.
     * @param mod_prova_display_options $displayoptions instance of mod_prova_display_options.
     * @param array $summarydata contains all table data
     * @return $output containing html data.
     */
    public function review_page(prova_attempt $attemptobj, $slots, $page, $showall,
                                $lastpage, mod_prova_display_options $displayoptions,
                                $summarydata, $printter) {
        $output = '';
        $output .= $this->header();
        $output .= $this->review_usersign_field($summarydata, $printter);
        $output .= $this->review_summary_table($summarydata, $page);
        $output .= $this->review_form($page, $showall, $displayoptions, $this->questions($attemptobj, true, $slots, $page, $showall, $displayoptions), $attemptobj);
        $output .= $this->review_next_navigation($attemptobj, $page, $lastpage, $showall, $printter);
        $output .= $this->footer();
        return $output;
    }

    /**
     * Outputs the table containing data from summary data array
     *
     * @param array $summarydata contains row data for table
     * @param int $page contains the current page number
     */
    public function review_summary_table($summarydata, $page) {

        $summarydata = $this->filter_review_summary_table($summarydata, $page);
        if (empty($summarydata)) {
            return '';
        }

        $output = '';

        $output .= html_writer::start_tag('table', array('class' => 'generaltable generalbox quizreviewsummary'));
        $output .= html_writer::start_tag('tbody');
        foreach ($summarydata as $rowdata) {
            if ($rowdata['title'] instanceof renderable) {
                $title = $this->render($rowdata['title']);
            } else {
                $title = $rowdata['title'];
            }

            if ($rowdata['content'] instanceof renderable) {
                $content = $this->render($rowdata['content']);
            } else {
                $content = $rowdata['content'];
            }

            $output .= html_writer::tag('tr', html_writer::tag('th', $title, array('class' => 'cell', 'scope' => 'row')) . html_writer::tag('td', $content, array('class' => 'cell')));
        }

        $output .= html_writer::end_tag('tbody');
        $output .= html_writer::end_tag('table');
        return $output;
    }

    /**
     * Renders the main bit of the review page.
     *
     * @param array $summarydata contain row data for table
     * @param int $page current page number
     * @param mod_prova_display_options $displayoptions instance of mod_prova_display_options
     * @param $content contains each question
     * @param prova_attempt $attemptobj instance of prova_attempt
     * @param bool $showall if true display attempt on one page
     */
    public function review_form($page, $showall, $displayoptions, $content, $attemptobj) {
        if ($displayoptions->flags != question_display_options::EDITABLE) {
            return $content;
        }

        // TODO: trazer js
        $this->page->requires->js_init_call('M.mod_prova.init_review_form', null, false,
                prova_get_js_module());

        $output = '';
        $output .= html_writer::start_tag('form', array('action' => $attemptobj->review_url(null, $page, $showall), 'method' => 'post', 'class' => 'questionflagsaveform'));
        $output .= html_writer::start_tag('div');
        $output .= $content;
        $output .= html_writer::empty_tag('input', array('type' => 'hidden', 'name' => 'sesskey',
                'value' => sesskey()));
        $output .= html_writer::start_tag('div', array('class' => 'submitbtns'));
        $output .= html_writer::empty_tag('input', array('type' => 'submit',
                'class' => 'questionflagsavebutton btn btn-secondary', 'name' => 'savingflags',
                'value' => get_string('saveflags', 'question')));
        $output .= html_writer::end_tag('div');
        $output .= html_writer::end_tag('div');
        $output .= html_writer::end_tag('form');

        return $output;
    }

    /**
     * Renders each question
     *
     * @param prova_attempt $attemptobj instance of prova_attempt
     * @param bool $reviewing
     * @param array $slots array of intgers relating to questions
     * @param int $page current page number
     * @param bool $showall if true shows attempt on single page
     * @param mod_prova_display_options $displayoptions instance of mod_prova_display_options
     */
    public function questions(prova_attempt $attemptobj, $reviewing, $slots, $page, $showall,
                              mod_prova_display_options $displayoptions) {
        $output = '';
        foreach ($slots as $slot) {
            $output .= $attemptobj->render_question($slot, $reviewing, $this,
                    $attemptobj->review_url($slot, $page, $showall));
        }
        return $output;
    }

    /**
     * Creates the navigation links/buttons at the bottom of the reivew attempt page.
     *
     * Note, the name of this function is no longer accurate, but when the design
     * changed, it was decided to keep the old name for backwards compatibility.
     *
     * @param prova_attempt $attemptobj instance of prova_attempt
     * @param int $page the current page
     * @param bool $lastpage if true current page is the last page
     * @param bool|null $showall if true, the URL will be to review the entire attempt on one page,
     *      and $page will be ignored. If null, a sensible default will be chosen.
     *
     * @return string HTML fragment.
     */
    public function review_next_navigation(prova_attempt $attemptobj, $page, $lastpage,  $showall = null, $printter=false) {
        // TODO: inserir chamada de JS para imprimir e botão
        $nav = '';
        if ($page > 0) {
            $nav .= link_arrow_left(get_string('navigateprevious', 'prova'),
                    $attemptobj->review_url(null, $page - 1, $showall), false, 'mod_prova-prev-nav');
        }
        if ($lastpage) {
            if (!$printter) {
                $nav .= $this->finish_review_link($attemptobj);
            } else {
                $nav .= $this->finish_print_btn($attemptobj);
            }
        } else {
            $nav .= link_arrow_right(get_string('navigatenext', 'prova'),
                    $attemptobj->review_url(null, $page + 1, $showall), false, 'mod_prova-next-nav');
        }
        $output = html_writer::start_div('printarea', array('id' => 'printarea'));
        $output .= html_writer::tag('div', $nav, array('class' => 'submitbtns', 'style' => 'display:flex;justify-content:center;'));
        $output .= html_writer::end_div();
        return $output;
    }


    /**
     * Filters the summarydata array.
     *
     * @param array $summarydata contains row data for table
     * @param int $page the current page number
     * @return $summarydata containing filtered row data
     */
    protected function filter_review_summary_table($summarydata, $page) {
        if ($page == 0) {
            return $summarydata;
        }

        // Only show some of summary table on subsequent pages.
        foreach ($summarydata as $key => $rowdata) {
            if (!in_array($key, array('user', 'attemptlist'))) {
                unset($summarydata[$key]);
            }
        }

        return $summarydata;
    }

    /**
     * Output either a link to the review page for an attempt, or a button to
     * open the review in a popup window.
     *
     * @param moodle_url $url of the target page.
     * @param bool $reviewinpopup whether a pop-up is required.
     * @param array $popupoptions options to pass to the popup_action constructor.
     * @return string HTML to output.
     */
    public function review_link($url, $reviewinpopup, $popupoptions, $corrected=null) {
        if ($reviewinpopup) {

            $correctness = ($corrected) ? 'corrected' : '';
            $title = ($corrected) ? get_string('reviewbtn_corrected', 'prova') : get_string('reviewbtn', 'prova');

            $button = new single_button($url, $title);
            $button->add_action(new popup_action('click', $url, 'quizpopup', $popupoptions));

            $output = '<span class="reviewbtn '.$correctness.'">';
            $output .= $this->render($button);
            $output .= '</span>';

            return $output;

        } else {
            return html_writer::link($url, get_string('reviewbtn', 'prova'),
                    array('title' => get_string('reviewthisattempt', 'prova')));
        }
    }

    /**
     * Output either a link to the printter page for an attempt, or a button to
     * open the review in a popup window.
     *
     * @param moodle_url $url of the target page.
     * @param bool $reviewinpopup whether a pop-up is required.
     * @param array $popupoptions options to pass to the popup_action constructor.
     * @return string HTML to output.
     */
    public function printter_link($url, $reviewinpopup, $popupoptions) {
        if ($reviewinpopup) {
            $printterimage = $this->pix_icon('t/print', get_string('printicon', 'prova'), '', array('class' => 'iconsmall'));
            $printteraction = new popup_action('click', $url, 'quizpopup', $popupoptions);
            return $this->action_link($url, $printterimage, $printteraction,  array());

        } else {
            return html_writer::link($url, get_string('review', 'prova'),
                    array('title' => get_string('reviewthisattempt', 'prova')));
        }
    }

    /**
     * Returns either a liink or button
     *
     * @param prova_attempt $attemptobj instance of prova_attempt
     */
    public function finish_review_link(prova_attempt $attemptobj) {
        $url = $attemptobj->view_url();
        $urlparam = optional_param('returnurl', 0, PARAM_URL);
        $returnurl = new moodle_url($urlparam);

        if($returnurl){
            $url = $returnurl;
        }

        if ($attemptobj->get_access_manager(time())->attempt_must_be_in_popup()) {
            $this->page->requires->js_init_call('M.mod_prova.secure_window.init_close_button',
                    array($url), false, prova_get_js_module());
            return html_writer::empty_tag('input', array('type' => 'button',
                    'value' => get_string('finishreview', 'prova'),
                    'id' => 'secureclosebutton',
                    'class' => 'mod_prova-next-nav btn btn-primary'));

        } else {
            return html_writer::link($url, get_string('finishreview', 'prova'),
                    array('class' => 'mod_prova-next-nav'));
        }
    }

    /**
     * Gera campo para assinatura de usuário
     *
     * @param array $summarydata contains row data for table
     * @return str html content
     */
    public function review_usersign_field($data, $printter) {
        if (!$printter) { return ''; }

        $userdata = $data['usertitle']['title'] . ' ' . $data['usertitle']['content'] . ', ' . date("d/m/Y");
        $output = '';
        $output .= html_writer::start_tag('span', ['class' => 'printusersign']);
        $output .= html_writer::tag('div', '', ['id' => 'signarea']);
        $output .= html_writer::tag('p', $userdata);
        $output .= html_writer::end_tag('span');
        return $output;
    }

    /**
     * Returns either a liink or button
     *
     * @param prova_attempt $attemptobj instance of prova_attempt
     */
    public function finish_print_btn(prova_attempt $attemptobj) {
        $url = $attemptobj->view_url();

        $this->page->requires->js_init_call('M.mod_prova.secure_window.init_printter_button',
                array($url), false, prova_get_js_module());

        return html_writer::empty_tag('input', array('type' => 'button',
                'value' => get_string('printter', 'prova'),
                'id' => 'secureprintbutton',
                'class' => 'mod_prova-next-nav btn btn-primary'));

        
    }

    /**
     * outputs the link the other attempts.
     *
     * @param mod_prova_links_to_other_attempts $links
     */
    protected function render_mod_prova_links_to_other_attempts(
            mod_prova_links_to_other_attempts $links) {
        $attemptlinks = array();
        foreach ($links->links as $attempt => $url) {
            if (!$url) {
                $attemptlinks[] = html_writer::tag('strong', $attempt);
            } else if ($url instanceof renderable) {
                $attemptlinks[] = $this->render($url);
            } else {
                $attemptlinks[] = html_writer::link($url, $attempt);
            }
        }
        return implode(', ', $attemptlinks);
    }



    /*
     * Summary Page
     */
    /**
     * Create the summary page
     *
     * @param prova_attempt $attemptobj
     * @param mod_prova_display_options $displayoptions
     */
    public function summary_page($attemptobj, $displayoptions) {
        $output = '';
        $output .= $this->header();
        $output .= $this->heading(format_string($attemptobj->get_prova_name()));
        $output .= $this->heading(get_string('summaryofattempt', 'prova'), 3);
        $output .= $this->provacontrol_form($attemptobj, -1);
        $output .= $this->summary_table($attemptobj, $displayoptions);
        $output .= $this->summary_page_controls($attemptobj);
        $output .= $this->get_navigation_panel($attemptobj, 'prova_attempt_nav_panel', -1);
        $output .= $this->footer();
        return $output;
    }

    /**
     * Generates the table of summarydata
     *
     * @param prova_attempt $attemptobj
     * @param mod_prova_display_options $displayoptions
     */
    public function summary_table($attemptobj, $displayoptions) {
        // Prepare the summary table header.
        $table = new html_table();
        $table->attributes['class'] = 'generaltable quizsummaryofattempt boxaligncenter';
        $table->head = array(get_string('question', 'prova'), get_string('status', 'prova'));
        $table->align = array('left', 'left');
        $table->size = array('', '');
        $markscolumn = $displayoptions->marks >= question_display_options::MARK_AND_MAX;
        if ($markscolumn) {
            $table->head[] = get_string('marks', 'prova');
            $table->align[] = 'left';
            $table->size[] = '';
        }
        $tablewidth = count($table->align);
        $table->data = array();

        // Get the summary info for each question.
        $slots = $attemptobj->get_slots();
        foreach ($slots as $slot) {
            // Add a section headings if we need one here.
            $heading = $attemptobj->get_heading_before_slot($slot);
            if ($heading) {
                $cell = new html_table_cell(format_string($heading));
                $cell->header = true;
                $cell->colspan = $tablewidth;
                $table->data[] = array($cell);
                $table->rowclasses[] = 'quizsummaryheading';
            }

            // Don't display information items.
            if (!$attemptobj->is_real_question($slot)) {
                continue;
            }

            // Real question, show it.
            $flag = '';
            if ($attemptobj->can_navigate_to($slot)) {
                $row = array(html_writer::link($attemptobj->attempt_url($slot),
                        $attemptobj->get_question_number($slot) . $flag),
                        $attemptobj->get_question_status($slot, $displayoptions->correctness));
            } else {
                $row = array($attemptobj->get_question_number($slot) . $flag,
                                $attemptobj->get_question_status($slot, $displayoptions->correctness));
            }
            if ($markscolumn) {
                $row[] = $attemptobj->get_question_mark($slot);
            }
            $table->data[] = $row;
            $table->rowclasses[] = 'quizsummary' . $slot . ' ' . $attemptobj->get_question_state_class($slot, $displayoptions->correctness);
        }

        // Print the summary table.
        $output = html_writer::table($table);

        return $output;
    }

    /**
     * Creates any controls a the page should have.
     *
     * @param prova_attempt $attemptobj
     */
    public function summary_page_controls($attemptobj) {
        $output = '';

        // Return to place button.
        if ($attemptobj->get_state() == prova_attempt::IN_PROGRESS) {
            $button = new single_button(
                    new moodle_url($attemptobj->attempt_url(null, $attemptobj->get_currentpage())),
                    get_string('returnattempt', 'prova'));
            $output .= $this->container($this->container($this->render($button), 'controls'), 'submitbtns mdl-align');
        }

        // Finish attempt button.
        $options = array(
            'attempt' => $attemptobj->get_attemptid(),
            'finishattempt' => 1,
            'timeup' => 0,
            'slots' => '',
            'cmid' => $attemptobj->get_cmid(),
            'sesskey' => sesskey(),
        );

        $button = new single_button(
                new moodle_url($attemptobj->processattempt_url(), $options),
                get_string('submitallandfinish', 'prova'));
        $button->id = 'responseform';
        if ($attemptobj->get_state() == prova_attempt::IN_PROGRESS) {
            $button->add_action(new confirm_action(get_string('confirmclose', 'prova'), null,
                    get_string('submitallandfinish', 'prova')));
        }

        $duedate = $attemptobj->get_due_date();
        $message = '';
        if ($attemptobj->get_state() == prova_attempt::OVERDUE) {
            $message = get_string('overduemustbesubmittedby', 'prova', userdate($duedate));

        } else if ($duedate) {
            $message = get_string('mustbesubmittedby', 'prova', userdate($duedate));
        }

        $output .= $this->countdown_timer($attemptobj, time());
        $output .= $this->container($message . $this->container($this->render($button), 'controls'), 'submitbtns mdl-align');

        return $output;
    }

    /*
     * View Page
     */
    /**
     * Generates the view page
     *
     * @param int $course The id of the course
     * @param array $prova Array conting prova data
     * @param int $cm Course Module ID
     * @param int $context The page context ID
     * @param array $infomessages information about this prova
     * @param mod_prova_view_object $viewobj
     * @param string $buttontext text for the start/continue attempt button, if
     *      it should be shown.
     * @param array $infomessages further information about why the student cannot
     *      attempt this prova now, if appicable this prova
     */
    public function view_page($course, $prova, $cm, $context, $viewobj) {
        $output = '';
        $output .= $this->view_information($prova, $cm, $context, $viewobj->infomessages);
        $output .= $this->view_table($prova, $context, $viewobj);
        $output .= $this->view_result_info($prova, $context, $cm, $viewobj);
        $output .= $this->box($this->view_page_buttons($viewobj, $prova), 'quizattempt');
        return $output;
    }

        /**
     * Work out, and render, whatever buttons, and surrounding info, should appear
     * at the end of the review page.
     * @param mod_prova_view_object $viewobj the information required to display
     * the view page.
     * @return string HTML to output.
     */
    public function view_page_buttons(mod_prova_view_object $viewobj, $prova) {
        global $CFG;
        $output = '';

        if ($viewobj->teacherormanager) {
            return $this->goto_prova_analytics_linkbtn($prova);
        }

        if (!$viewobj->provahasquestions) {
            $output .= $this->no_questions_message($viewobj->canedit, $viewobj->editurl);
        }

        $output .= $this->access_messages($viewobj->preventmessages);

        if ($viewobj->buttontext) {
            $output .= $this->start_attempt_button($viewobj->buttontext,
                    $viewobj->startattempturl, $viewobj->preflightcheckform,
                    $viewobj->popuprequired, $viewobj->popupoptions);
        }

        if ($viewobj->showbacktocourse) {
            $output .= $this->single_button($viewobj->backtocourseurl,
                    get_string('backtocourse', 'prova'), 'get',
                    array('class' => 'continuebutton'));
        }

        return $output;
    }

    /**
     * Gera link direto para visualização de dos resultados das provas.
     *
     * @param array $prova Array conting prova data
     */ 
    public function goto_prova_analytics_linkbtn($prova) {

        $url = new moodle_url('/mod/prova/viewattempts.php', array('appid' => $prova->id, 'cm' => $prova->cmid));
        $button = new single_button($url, get_string('analyticsviewprova', 'prova'));
        $button->method = 'get';

        return $this->render($button);
    }

    /**
     * Output the page information
     *
     * @param object $prova the prova settings.
     * @param object $cm the course_module object.
     * @param object $context the prova context.
     * @param array $messages any access messages that should be described.
     * @return string HTML to output.
     */
    public function view_information($prova, $cm, $context, $messages) {
        global $CFG;

        $output = '';

        // Print prova name and description.
        $output .= $this->heading(format_string($prova->name));
        $output .= $this->prova_intro($prova, $cm);

        // Output any access messages.
        if ($messages) {
            $output .= $this->box($this->access_messages($messages), 'quizinfo');
        }

        // Show number of attempts summary to those who can view reports.
        if (has_capability('mod/prova:viewreports', $context)) {
            // TODO: implementar reports
            if (false /* $strattemptnum = $this->prova_attempt_summary_link_to_reports($prova, $cm, $context)*/) {
                $output .= html_writer::tag('div', $strattemptnum,
                        array('class' => 'quizattemptcounts'));
            }
        }
        return $output;
    }

    /**
     * Output the prova intro.
     * @param object $prova the prova settings.
     * @param object $cm the course_module object.
     * @return string HTML to output.
     */
    public function prova_intro($prova, $cm) {
        if (html_is_blank($prova->intro)) {
            return '';
        }

        return $this->box(format_module_intro('prova', $prova, $cm->id), 'generalbox', 'intro');
    }

    /**
     * Generates the table heading.
     */
    public function view_table_heading() {
        return $this->heading(get_string('summaryofattempts', 'prova'), 3);
    }

    /**
     * Generates the table of data
     *
     * @param array $prova Array contining prova data
     * @param int $context The page context ID
     * @param mod_prova_view_object $viewobj
     */
    public function view_table($prova, $context, $viewobj) {
        if (!$viewobj->attempts) {
            return '';
        }

        // Prepare table header.
        $table = new html_table();
        $table->attributes['class'] = 'generaltable quizattemptsummary';
        $table->head = array();
        $table->align = array();
        $table->size = array();
        if ($viewobj->attemptcolumn) {
            $table->head[] = get_string('attemptnumber', 'prova');
            $table->align[] = 'center';
            $table->size[] = '';
        }
        $table->head[] = get_string('attemptstate', 'prova');
        $table->align[] = 'left';
        $table->size[] = '';
        if ($viewobj->markcolumn) {
            $table->head[] = get_string('marks', 'prova') . ' / ' .
                    modprova_format_grade($prova, $prova->sumgrades);
            $table->align[] = 'center';
            $table->size[] = '';
        }
        // if ($viewobj->gradecolumn) {
        //     $table->head[] = get_string('grade') . ' / ' .
        //             modprova_format_grade($prova, $prova->grade);
        //     $table->align[] = 'center';
        //     $table->size[] = '';
        // }
        if ($viewobj->canreviewmine) {
            $table->head[] = get_string('review', 'prova');
            $table->align[] = 'center';
            $table->size[] = '';
        }
        if ($viewobj->feedbackcolumn) {
            $table->head[] = get_string('feedback', 'prova');
            $table->align[] = 'left';
            $table->size[] = '';
        }

        // One row for each attempt.
        foreach ($viewobj->attemptobjs as $attemptobj) {
            $attemptoptions = $attemptobj->get_display_options(true);
            $row = array();

            // Add the attempt number.
            if ($viewobj->attemptcolumn) {
                if ($attemptobj->is_preview()) {
                    $row[] = get_string('preview', 'prova');
                } else {
                    $row[] = $attemptobj->get_attempt_number();
                }
            }

            $row[] = $this->attempt_state($attemptobj);

            if ($viewobj->markcolumn) {
                if ($attemptoptions->marks >= question_display_options::MARK_AND_MAX &&
                        $attemptobj->is_finished()) {
                    $row[] = modprova_format_grade($prova, $attemptobj->get_sum_marks());
                } else {
                    $row[] = '';
                }
            }

            // Ouside the if because we may be showing feedback but not grades.
            $attemptgrade = prova_rescale_grade($attemptobj->get_sum_marks(), $prova, false);

            // if ($viewobj->gradecolumn) {
            //     if ($attemptoptions->marks >= question_display_options::MARK_AND_MAX &&
            //             $attemptobj->is_finished()) {

            //         // Highlight the highest grade if appropriate.
            //         if ($viewobj->overallstats && !$attemptobj->is_preview()
            //                 && $viewobj->numattempts > 1 && !is_null($viewobj->mygrade)
            //                 && $attemptobj->get_state() == prova_attempt::FINISHED
            //                 && $attemptgrade == $viewobj->mygrade
            //                 && $prova->grademethod == PROVA_GRADEHIGHEST) {
            //             $table->rowclasses[$attemptobj->get_attempt_number()] = 'bestrow';
            //         }

            //         $row[] = modprova_format_grade($prova, $attemptgrade);
            //     } else {
            //         $row[] = '';
            //     }
            // }

            if ($viewobj->canreviewmine) {
                $row[] = $viewobj->accessmanager->make_review_link($attemptobj->get_attempt(), $attemptoptions, $this);
            }

            // TODO: implementar feedback
            // if ($viewobj->feedbackcolumn && $attemptobj->is_finished()) {
            //     if ($attemptoptions->overallfeedback) {
            //         $row[] = prova_feedback_for_grade($attemptgrade, $prova, $context);
            //     } else {
            //         $row[] = '';
            //     }
            // }

            if ($attemptobj->is_preview()) {
                $table->data['preview'] = $row;
            } else {
                $table->data[$attemptobj->get_attempt_number()] = $row;
            }
        } // End of loop over attempts.

        $output = '';
        $output .= $this->view_table_heading();
        $output .= html_writer::table($table);
        return $output;
    }

    /**
     * Generates data pertaining to prova results
     *
     * @param array $prova Array containing prova data
     * @param int $context The page context ID
     * @param int $cm The Course Module Id
     * @param mod_prova_view_object $viewobj
     */
    public function view_result_info($prova, $context, $cm, $viewobj) {
        $output = '';
        if (!$viewobj->numattempts && !$viewobj->gradecolumn && is_null($viewobj->mygrade)) {
            return $output;
        }
        $resultinfo = '';

        if ($viewobj->overallstats) {
            if ($viewobj->moreattempts) {
                $a = new stdClass();
                $a->method = prova_get_grading_option_name($prova->grademethod);
                $a->mygrade = modprova_format_grade($prova, $viewobj->mygrade);
                $a->provagrade = modprova_format_grade($prova, $prova->sumgrades);
                $resultinfo .= $this->heading(get_string('gradesofar', 'prova', $a), 3);
            } else {
                $a = new stdClass();
                $a->grade = modprova_format_grade($prova, $viewobj->mygrade);
                $a->maxgrade = modprova_format_grade($prova, $prova->sumgrades);
                $a = get_string('outofshort', 'prova', $a);
                $resultinfo .= $this->heading(get_string('yourfinalgradeis', 'prova', $a), 3);
            }
        }

        if ($viewobj->mygradeoverridden) {

            $resultinfo .= html_writer::tag('p', get_string('overriddennotice', 'grades'),
                    array('class' => 'overriddennotice'))."\n";
        }
        if ($viewobj->gradebookfeedback) {
            $resultinfo .= $this->heading(get_string('comment', 'prova'), 3);
            $resultinfo .= html_writer::div($viewobj->gradebookfeedback, 'quizteacherfeedback') . "\n";
        }
        // TOOD: implementar feedback
        // if ($viewobj->feedbackcolumn) {
        //     $resultinfo .= $this->heading(get_string('overallfeedback', 'prova'), 3);
        //     $resultinfo .= html_writer::div(
        //             prova_feedback_for_grade($viewobj->mygrade, $prova, $context),
        //             'provagradefeedback') . "\n";
        // }

        if ($resultinfo) {
            $output .= $this->box($resultinfo, 'generalbox', 'feedback');
        }
        return $output;
    }


    /*
     * Attempt Page
     */
	/**
     * Cria estrutura visual do preview da prova
     *
     * @param attempt \prova_attempt_preview objeto da prova
     * @param page \int representa o número slot que será exibido
     * @param accessmanager TODO
     * @param messages TODO
     * @param slots \array(int) contém todos os ids dos slots de uma prova
     * @param id TODO
     * @param newxpage \int contém o número da próxima página que será exibida
     * @return \String html da tela de preview
     */
    public function attempt_page($attempt, $page, $messages, $slots,
            $nextpage) {
        $output = '';
        $output .= $this->header();
        $output .= $this->provacontrol_form($attempt, $page);
        $output .= $this->attempt_form($attempt, $page, $slots, $nextpage);
        $output .= $this->get_navigation_panel($attempt, 'prova_attempt_nav_panel', $page);
        $output .= $this->footer();
        return $output;
    }

    public function start_attempt_page(prova $provaobj, mod_prova_preflight_check_form $mform) {
        $output = '';
        $output .= $this->header();
        $output .= $this->heading(format_string($provaobj->get_prova_name(), true,
                                  array("context" => $provaobj->get_context())));
        $output .= $this->prova_intro($provaobj->get_prova(), $provaobj->get_cm());
        $output .= $mform->render();
        $output .= $this->footer();
        return $output;
    }

    public function provacontrol_form($attemptobj, $page) {

        if (!$attemptobj->get_prova()->has_provacontrol()) { return; }

        $url = new moodle_url('/mod/prova/view.php', array('id' => $attemptobj->get_cmid()));

        $this->page->requires->yui_module('moodle-mod_prova-provacontrol', 'M.mod_prova.provacontrol.init', [[
                'url' =>  $url->out(),
                'msg' =>  get_string('provacontrol_alert', 'prova'),
            ]]
        );

        $output = html_writer::start_tag(
            'form',
            array(
                'method' => 'post',
                'enctype' => 'multipart/form-data',
                'accept-charset' => 'utf-8',
                'id' => 'provacontrol'
            )
        );

        $output .= html_writer::empty_tag('input', array('type' => 'hidden', 'name' => 'attempt', 'value' => $attemptobj->get_attemptid()));
        $output .= html_writer::empty_tag('input', array('type' => 'hidden', 'name' => 'thispage', 'value' => $page));
         $output .= html_writer::empty_tag('input', array('type' => 'hidden', 'name' => 'sesskey', 'value' => sesskey()));

        $output .= html_writer::end_tag('form');

        return $output;
    }

    /**
     * Cria estrutura visual do preview da prova
     *
     * @param attempt \prova_attempt_preview objeto da prova
     * @param page \int representa o número slot que será exibido
     * @param slots \array(int) contém todos os ids dos slots de uma prova
     * @param newxpage \int contém o número da próxima página que será exibida
     * @return \String $output
     */
    public function attempt_form($attempt, $page, $slots, $nextpage) {
        $prova = $attempt->get_provaid();

        $output = '';

        // Start the form.
        $output .= html_writer::start_tag(
            'form',
            array(
                'action' => new moodle_url($attempt->processattempt_url(), array('cmid' => $attempt->get_cmid())),
                'method' => 'post',
                'enctype' => 'multipart/form-data',
                'accept-charset' => 'utf-8',
                'id' => 'responseform'
            )
        );
        $output .= html_writer::start_tag('div');

        // Print all the questions.
        foreach ($slots as $slot) {
            $output .= $attempt->render_question($slot, false, $this, $attempt->attempt_url($slot, $page), $this, $this);
        }

        $navmethod = $attempt->get_prova()->get_prova()->navmethod;
        $output .= $this->attempt_navigation_buttons($page, $attempt->is_last_page($page));

        // Some hidden fields to trach what is going on.
        $output .= html_writer::empty_tag('input', array('type' => 'hidden', 'name' => 'provaid',
                'value' => $prova));
        $output .= html_writer::empty_tag('input', array('type' => 'hidden', 'name' => 'thispage',
                'value' => $page, 'id' => 'followingpage'));
        $output .= html_writer::empty_tag('input', array('type' => 'hidden', 'name' => 'nextpage',
                'value' => $nextpage));
        $output .= html_writer::empty_tag('input', array('type' => 'hidden', 'name' => 'timeup',
                'value' => '0', 'id' => 'timeup'));
        $output .= html_writer::empty_tag('input', array('type' => 'hidden', 'name' => 'sesskey',
                'value' => sesskey()));
        $output .= html_writer::empty_tag('input', array('type' => 'hidden', 'name' => 'scrollpos',
                'value' => '', 'id' => 'scrollpos'));

        $output .= html_writer::end_tag('div');
        $output .= html_writer::end_tag('form');

        return $output;

    }

    /**
     * Function from /mod/prova/renderer.php
     */
    protected function attempt_navigation_buttons($page, $lastpage, $navmethod = 'free') {
        $output = '';

        $output .= html_writer::start_tag('div', array('class' => 'submitbtns'));
        if ($page > 1 && $navmethod == 'free') {
            $output .= html_writer::empty_tag('input', array('type' => 'submit', 'name' => 'previous',
                    'value' => get_string('navigateprevious', 'prova'), 'class' => 'mod_prova-prev-nav btn btn-secondary'));
        }
        if ($lastpage) {
            $nextlabel = get_string('endtest', 'prova');
        } else {
            $nextlabel = get_string('navigatenext', 'prova');
        }
        $output .= html_writer::empty_tag('input', array('type' => 'submit', 'name' => 'next',
                'value' => $nextlabel, 'class' => 'mod_prova-next-nav btn btn-primary'));
        $output .= html_writer::end_tag('div');

        return $output;
    }

    /**
     * Outputs the navigation block panel
     *
     * @param prova_nav_panel_base $panel instance of prova_nav_panel_base
     */
    public function navigation_panel(prova_nav_panel_base $panel, $title) {

        $output = '</div></section>';

        $output .= '<section id="mod_prova_navblock" data-region="blocks-column-seox" class="d-print-none" aria-label="Blocks"><div class="card-body p-3">';

        $output .= $title; 

        $output .= $panel->render_before_button_bits($this);

        $bcc = $panel->get_button_container_class();
        $output .= html_writer::start_tag('div', array('class' => "qn_buttons clearfix $bcc"));

        foreach ($panel->get_question_buttons() as $button) {
            $output .= $this->render($button);
        }
        $output .= html_writer::end_tag('div');

        $outputpanel = $panel->render_end_bits($this);

        $output .= html_writer::tag('div',$outputpanel, array('class' => 'othernav'));

        $this->page->requires->js_init_call('M.mod_prova.nav.init', null, false, prova_get_js_module());

        return $output;
    }

    /**
     * Create a preview link
     *
     * @param $url contains a url to the given page
     */
    public function restart_preview_button($url) {
        return $this->single_button($url, get_string('startnewpreview', 'prova'));
    }

    /**
     * Get the navigation panel object for this attempt.
     *
     * @param $prova_attemtp
     * @param $page the current page number.
     * @param $showall whether we are showing the whole prova on one page. (Used by review.php)
     * @return prova_nav_panel_base the requested object.
     */
    public function get_navigation_panel($attempt, $panelclass, $page, $showall = false) {
        $panel = new $panelclass($attempt, $attempt->get_display_options(true), $page, $showall);

        $title = html_writer::tag('h5', get_string('provanavigation', 'provacomp_addprovas'), array('id' => 'mod_prova_navblock_title'));
        $content = $this->navigation_panel($panel, $title);
        return $content;
    }

    /**
     * Get the navigation panel object for this attempt.
     *
     * @param $prova_attemtp
     * @param $page the current page number.
     * @param $showall whether we are showing the whole prova on one page. (Used by review.php)
     * @return prova_nav_panel_base the requested object.
     */
    public function get_navigation_panel_reviewquestion($attempt, $slot, $slots) {
        $panel = new prova_reviewquestion_nav_panel($attempt, $attempt->get_display_options(true), $slot, false);
        $panel->set_question_url_params($slots);

        $title = html_writer::tag('h5', get_string('provanavigation', 'provacomp_addprovas'), array('id' => 'mod_prova_navblock_title'));
        $content = $this->navigation_panel($panel, $title);
        return $content;
    }


    /**
     * Display a prova navigation button.
     *
     * @param prova_nav_question_button $button
     * @return string HTML fragment.
     */
    protected function render_prova_nav_question_button(prova_nav_question_button $button) {
        $classes = array('qnbutton', $button->stateclass, $button->navmethod, 'btn', 'btn-secondary');
        $extrainfo = array();

        if ($button->currentpage) {
            $classes[] = 'thispage';
            $extrainfo[] = get_string('onthispage', 'prova');
        }

        // Flagged?
        if ($button->flagged) {
            $classes[] = 'flagged';
            $flaglabel = get_string('flagged', 'question');
        } else {
            $flaglabel = '';
        }
        $extrainfo[] = html_writer::tag('span', $flaglabel, array('class' => 'flagstate'));

        if (is_numeric($button->number)) {
            $qnostring = 'questionnonav';
        } else {
            $qnostring = 'questionnonavinfo';
        }

        $a = new stdClass();
        $a->number = $button->number;
        $a->attributes = implode(' ', $extrainfo);
        $tagcontents = html_writer::tag('span', '', array('class' => 'thispageholder')) .
                        html_writer::tag('span', '', array('class' => 'trafficlight')) .
                        get_string($qnostring, 'prova', $a);
        $tagattributes = array('class' => implode(' ', $classes), 'id' => $button->id,
                                  'title' => $button->statestring, 'data-quiz-page' => $button->page);

        if ($button->url) {
            return html_writer::link($button->url, $tagcontents, $tagattributes);
        } else {
            return html_writer::tag('span', $tagcontents, $tagattributes);
        }
    }

    /**
     * Outputs an error message for any guests accessing the prova
     *
     * @param int $course The course ID
     * @param array $prova Array contingin prova data
     * @param int $cm Course Module ID
     * @param int $context The page contect ID
     * @param array $messages Array containing any messages
     */
    public function view_page_guest($course, $prova, $cm, $context, $messages) {
        $output = '';
        $output .= $this->view_information($prova, $cm, $context, $messages);
        $guestno = html_writer::tag('p', get_string('guestsno', 'prova'));
        $liketologin = html_writer::tag('p', get_string('liketologin'));
        $referer = get_local_referer(false);
        $output .= $this->confirm($guestno."\n\n".$liketologin."\n", get_login_url(), $referer);
        return $output;
    }

    /**
     * Outputs and error message for anyone who is not enrolle don the course
     *
     * @param int $course The course ID
     * @param array $prova Array contingin prova data
     * @param int $cm Course Module ID
     * @param int $context The page contect ID
     * @param array $messages Array containing any messages
     */
    public function view_page_notenrolled($course, $prova, $cm, $context, $messages) {
        global $CFG;
        $output = '';
        $output .= $this->view_information($prova, $cm, $context, $messages);
        $youneedtoenrol = html_writer::tag('p', get_string('youneedtoenrol', 'prova'));
        $button = html_writer::tag('p',
                $this->continue_button($CFG->wwwroot . '/course/view.php?id=' . $course->id));
        $output .= $this->box($youneedtoenrol."\n\n".$button."\n", 'generalbox', 'notice');
        return $output;
    }

    /**
     * Generate a brief textual desciption of the current state of an attempt.
     * @param prova_attempt $attemptobj the attempt
     * @param int $timenow the time to use as 'now'.
     * @return string the appropriate lang string to describe the state.
     */
    public function attempt_state($attemptobj) {
        switch ($attemptobj->get_state()) {
            case prova_attempt::IN_PROGRESS:
                return get_string('stateinprogress', 'prova');

            case prova_attempt::OVERDUE:
                return get_string('stateoverdue', 'prova') . html_writer::tag('span',
                        get_string('stateoverduedetails', 'prova',
                                userdate($attemptobj->get_due_date())),
                        array('class' => 'statedetails'));

            case prova_attempt::FINISHED:
                return get_string('statefinished', 'prova') . html_writer::tag('span',
                        get_string('statefinisheddetails', 'prova',
                                userdate($attemptobj->get_submitted_date())),
                        array('class' => 'statedetails'));

            case prova_attempt::ABANDONED:
                return get_string('stateabandoned', 'prova');
        }
    }

    /**
     * Displayed where there might normally be a review link, to explain why the
     * review is not available at this time.
     * @param string $message optional message explaining why the review is not possible.
     * @return string HTML to output.
     */
    public function no_review_message($message) {
        return html_writer::nonempty_tag('span', $message,
                array('class' => 'noreviewmessage'));
    }

    /**
     * Generates the view attempt button
     *
     * @param string $buttontext the label to display on the button.
     * @param moodle_url $url The URL to POST to in order to start the attempt.
     * @param mod_prova_preflight_check_form $preflightcheckform deprecated.
     * @param bool $popuprequired whether the attempt needs to be opened in a pop-up.
     * @param array $popupoptions the options to use if we are opening a popup.
     * @return string HTML fragment.
     */
    // TODO: resolver preflight
    public function start_attempt_button($buttontext, moodle_url $url,
            /*mod_prova_preflight_check_form*/ $preflightcheckform = null,
            $popuprequired = false, $popupoptions = null) {

        if (is_string($preflightcheckform)) {
            // Calling code was not updated since the API change.
            debugging('The third argument to start_attempt_button should now be the ' .
                    'mod_prova_preflight_check_form from ' .
                    'prova_access_manager::get_preflight_check_form, not a warning message string.');
        }

        $button = new single_button($url, $buttontext);
        $button->class .= ' provastartbuttondiv';
        if ($popuprequired) {
            $button->class .= ' quizsecuremoderequired';
        }

        $popupjsoptions = null;
        if ($popuprequired && $popupoptions) {
            $action = new popup_action('click', $url, 'popup', $popupoptions);

            $popupjsoptions = $action->get_js_options();
        }

        if ($preflightcheckform) {
            $checkform = $preflightcheckform->render();
        } else {
            $checkform = null;
        }

        $this->page->requires->js_call_amd('mod_prova/preflightcheck', 'init', array('.provastartbuttondiv [type=submit]', get_string('startattempt', 'prova'), '#mod_prova_preflight_form', $popupjsoptions));

        return $this->render($button) . $checkform;
    }

    /**
     * Output the JavaScript required to initialise the countdown timer.
     * @param int $timerstartvalue time remaining, in seconds.
     */
    public function initialise_timer($timerstartvalue, $ispreview) {
        $options = array($timerstartvalue, (bool)$ispreview);
        $this->page->requires->js_init_call('M.mod_prova.timer.init', $options, false, prova_get_js_module());
    }

    /**
     * Print each message in an array, surrounded by &lt;p>, &lt;/p> tags.
     *
     * @param array $messages the array of message strings.
     * @param bool $return if true, return a string, instead of outputting.
     *
     * @return string HTML to output.
     */
    public function access_messages($messages) {
        $output = '';
        foreach ($messages as $message) {
            $output .= html_writer::tag('p', $message) . "\n";
        }
        return $output;
    }

    /**
     * Outputs a chart.
     *
     * @param \core\chart_base $chart The chart.
     * @param string $title The title to display above the graph.
     * @return string HTML fragment for the graph.
     */
    public function chart(\core\chart_base $chart, $title) {
        return $this->heading($title, 3) . html_writer::tag('div', $this->render($chart), array('class' => 'graph'));
    }

}


class mod_prova_links_to_other_attempts implements renderable {
    /**
     * @var array string attempt number => url, or null for the current attempt.
     * url may be either a moodle_url, or a renderable.
     */
    public $links = array();
}

class mod_prova_view_object {
    /** @var array $infomessages of messages with information to display about the prova. */
    public $infomessages;
    /** @var array $attempts contains all the user's attempts at this prova. */
    public $attempts;
    /** @var array $attemptobjs prova_attempt objects corresponding to $attempts. */
    public $attemptobjs;
    /** @var prova_access_manager $accessmanager contains various access rules. */
    public $accessmanager;
    /** @var bool $canreviewmine whether the current user has the capability to
     *       review their own attempts. */
    public $canreviewmine;
    /** @var bool $canedit whether the current user has the capability to edit the prova. */
    public $canedit;
    /** @var moodle_url $editurl the URL for editing this prova. */
    public $editurl;
    /** @var int $attemptcolumn contains the number of attempts done. */
    public $attemptcolumn;
    /** @var int $gradecolumn contains the grades of any attempts. */
    public $gradecolumn;
    /** @var int $markcolumn contains the marks of any attempt. */
    public $markcolumn;
    /** @var int $overallstats contains all marks for any attempt. */
    public $overallstats;
    /** @var string $feedbackcolumn contains any feedback for and attempt. */
    public $feedbackcolumn;
    /** @var string $timenow contains a timestamp in string format. */
    public $timenow;
    /** @var int $numattempts contains the total number of attempts. */
    public $numattempts;
    /** @var float $mygrade contains the user's final grade for a prova. */
    public $mygrade;
    /** @var bool $moreattempts whether this user is allowed more attempts. */
    public $moreattempts;
    /** @var int $mygradeoverridden contains an overriden grade. */
    public $mygradeoverridden;
    /** @var string $gradebookfeedback contains any feedback for a gradebook. */
    public $gradebookfeedback;
    /** @var bool $unfinished contains 1 if an attempt is unfinished. */
    public $unfinished;
    /** @var object $lastfinishedattempt the last attempt from the attempts array. */
    public $lastfinishedattempt;
    /** @var array $preventmessages of messages telling the user why they can't
     *       attempt the prova now. */
    public $preventmessages;
    /** @var string $buttontext caption for the start attempt button. If this is null, show no
     *      button, or if it is '' show a back to the course button. */
    public $buttontext;
    /** @var moodle_url $startattempturl URL to start an attempt. */
    public $startattempturl;
    /** @var moodleform|null $preflightcheckform confirmation form that must be
     *       submitted before an attempt is started, if required. */
    public $preflightcheckform;
    /** @var moodle_url $startattempturl URL for any Back to the course button. */
    public $backtocourseurl;
    /** @var bool $showbacktocourse should we show a back to the course button? */
    public $showbacktocourse;
    /** @var bool whether the attempt must take place in a popup window. */
    public $popuprequired;
    /** @var array options to use for the popup window, if required. */
    public $popupoptions;
    /** @var bool $provahasquestions whether the prova has any questions. */
    public $provahasquestions;

    public function __get($field) {
        switch ($field) {
            case 'startattemptwarning':
                debugging('startattemptwarning has been deprecated. It is now always blank.');
                return '';

            default:
                debugging('Unknown property ' . $field);
                return null;
        }
    }
}
