<?php
/**
 * Inicia um attempt de uma aplicação de prova
 *
 * @package /mod/prova/screens/attempt
 */

require_once(__DIR__ . '/../config.php');
require_once($CFG->dirroot . '/mod/prova/locallib.php');

// Get submitted parameters.
$id = required_param('cmid', PARAM_INT); // Course module id
$forcenew = optional_param('forcenew', false, PARAM_BOOL); // Used to force a new preview
$page = optional_param('page', -1, PARAM_INT); // Page to jump to in the attempt.

if (!$cm = get_coursemodule_from_id('prova', $id)) {
    print_error('invalidcoursemodule');
}
if (!$course = $DB->get_record('course', array('id' => $cm->course))) {
    print_error("coursemisconf");
}

$provaobj = prova::create($cm->instance, $USER->id);
// This script should only ever be posted to, so set page URL to the view page.
$PAGE->set_url($provaobj->view_url());

// Check login and sesskey.
require_login($provaobj->get_course(), false, $provaobj->get_cm());
// TODO: add sesskey
// require_sesskey();
$PAGE->set_heading($provaobj->get_course()->fullname);

// If no questions have been set up yet redirect to edit.php or display an error.
// TODO: criar capabilities deste plugin 
if (!$provaobj->has_questions()) {
    // if ($provaobj->has_capability('mod/prova:manage')) {
    //     redirect($provaobj->edit_url());
    // } else {
    print_error('cannotstartnoquestions', 'mod_prova', $provaobj->view_url());
    // }
}

// Create an object to manage all the other (non-roles) access rules.
$timenow = time();
$accessmanager = $provaobj->get_access_manager($timenow);

// Validate permissions for creating a new attempt and start a new preview attempt if required.
list($currentattemptid, $attemptnumber, $lastattempt, $messages, $page) =
    prova_validate_new_attempt($provaobj, $accessmanager, $forcenew, $page, true);

// Check access.
if (!$provaobj->is_preview_user() && $messages) {
    $output = $PAGE->get_renderer('mod_prova');
    print_error('attempterror', 'prova', $provaobj->view_url(), $output->access_messages($messages));
}

// TODO: entender a necessidade de preflight
if ($accessmanager->is_preflight_check_required($currentattemptid)) {
    // Need to do some checks before allowing the user to continue.
    $mform = $accessmanager->get_preflight_check_form(
            $provaobj->start_attempt_url($page), $currentattemptid);

    if ($mform->is_cancelled()) {
        $accessmanager->back_to_view_page($PAGE->get_renderer('mod_prova'));

    } else if (!$mform->get_data()) {

        // Form not submitted successfully, re-display it and stop.
        $PAGE->set_url($provaobj->start_attempt_url($page));
        $PAGE->set_title($provaobj->get_prova_name());
        $accessmanager->setup_attempt_page($PAGE);
        $output = $PAGE->get_renderer('mod_prova');

        echo $output->start_attempt_page($provaobj, $mform);
        die();
    }

    // Pre-flight check passed.
    $accessmanager->notify_preflight_check_passed($currentattemptid);
}
if ($currentattemptid) {
    if ($lastattempt->state == prova_attempt::OVERDUE) {
        redirect($provaobj->summary_url($lastattempt->id));
    } else {
        redirect($provaobj->attempt_url($currentattemptid, $page));
    }
}

$attempt = prova_prepare_and_start_new_attempt($provaobj, $attemptnumber, $lastattempt);

// Redirect to the attempt page.
redirect($provaobj->attempt_url($attempt->id, $page));
