<?php
require_once(__DIR__ . '/../../../../config.php');
require_once($CFG->dirroot.'/mod/prova/locallib.php');
$PAGE->set_context(context_system::instance());

$appid = optional_param('appid', 0, PARAM_INT);
$unlock = optional_param('unlock', 0, PARAM_INT);
$return_url = optional_param('returnurl', 0, PARAM_URL);

if (!$provaobj = prova::create($appid)) {
    print_error('invalidprovaid', 'prova');
}
$prova = $provaobj->get_prova();

if (!$course = $DB->get_record('course', array('id' => $prova->course))) {
    print_error('invalidcourseid');
}
if (!$cm = get_coursemodule_from_instance("prova", $prova->id, $course->id)) {
    print_error('invalidcoursemodule');
}

require_login($course, false, $cm);

$PAGE->set_url('/mod/prova/viewattempts.php', array('appid' => $appid));
$PAGE->set_heading('Liberar Prova Para Aluno');
$PAGE->set_title(get_string('unlockedprovatitle', 'prova'));
$PAGE->navbar->add(get_string('form_return_title', 'prova'), new moodle_url($return_url));

\mod_prova\manager::has_permission_to_access();

echo $OUTPUT->header();
echo print_table_seox($appid);
if($unlock != null){
    if($unlock == 1){
        echo $OUTPUT->notification(get_string('unlock_prova_sucess', 'prova'), 'notifysuccess');
    }elseif($unlock == 2){
        echo $OUTPUT->notification(get_string('unlock_prova_error', 'prova'), 'notifyerros');
    }
}
echo $OUTPUT->footer();

function print_table_seox($appid)
    {
        global $DB, $OUTPUT, $PAGE;
        $urldata = null;
       
        $sql = "SELECT 
            prova_att.id AS attempt_id,
            prova_att.userid userid, 
            CONCAT(users.firstname,' ', users.lastname) AS nome_aluno,
            prova_app.name AS nome_prova, 
            prova_att.timefinish AS timefinish,
            prova_att.timestart AS timestart,
            prova_att.state state
            from {prova_attempts} AS prova_att
            INNER JOIN {prova} AS prova_app
            ON prova_app.id = prova_att.prova
            INNER JOIN {user} AS users
            ON users.id = prova_att.userid";

        $sql .= " WHERE prova_att.prova = $appid and prova_att.state = 'finished'";

        //Parametros para paginação
        $page = optional_param('page', 0, PARAM_INT);
        $perpage = 10;
        $start = $page * $perpage;

        //Query com dados paginados
        $results = $DB->get_records_sql($sql, null, $start, $perpage);
        $totalcount =  $DB->count_records('prova_attempts', array('prova'=> $appid,'state'=> 'finished'));

        $provaobj = prova::create($appid);
        $output = $PAGE->get_renderer('mod_prova');

        if (!empty($results)) {
            $table = new html_table();

            $table->head = array('Nome Usuario', 'Nome App Prova', 'Data e Hora da Entrega',  'T. Execução Prova (H:M:s)', 'Status', 'Liberar Prova');

            // Gera o body da tabela
            $cont = 1;
            foreach ($results as $prova) {

                $tempo_execucao     = calculate_date($prova->timestart, $prova->timefinish);
                $dt_hora_execucao   = format_date($prova->timefinish);
                $row = new html_table_row();
                foreach ($prova as $key => $field) {

                    if ($key == 'attempt_id') { continue; }
                    if ($key == 'userid') { continue; }
                    if ($key == 'timefinish') { continue; }
                    if ($key == 'timestart') { continue; }
                    if ($key == 'nota') { continue; }

                    if (/* $key == 'tempo_execucao' and  */$prova->state != 'finished') { 
                        $row->cells[] = ' - ';
                        continue;
                    }

                    $row->cells[] = $field;
                }

                //$nota_prova = number_format(round($prova->nota, 2), 2);
                $row->cells[2] = $dt_hora_execucao;
                //$row->cells[3] = $nota_prova;
                $row->cells[4] = $tempo_execucao;
                $row->cells[5] = $prova->state != 'finished' ? ' - ' : 'Finalizado';

                if ($prova->state == 'finished') {
                    // Actions Field
                    $last_column = '<div style="display:inline-block; width: 100px">';
                    // -- Edit
				$editurl = new moodle_url('/mod/prova/screens/liberarprova/processliberarprova.php', array(
					'prova_appid' => $appid,
					'prova_attid' => $prova->attempt_id,
                    'id'=> $prova->userid,
				));
				$editaction = $OUTPUT->action_icon($editurl, new pix_icon('t/lock', get_string('edit')), null, array('class' => 'seox_leanblocks-action'));
				$last_column .= $editaction;                
                
                    $row->cells[] = $last_column . '</div>';
                } else {
                    $row->cells[] = '';
                }

                $table->data[] = $row;
                $cont++;
            }
            $url = new moodle_url('/mod/prova/screens/liberarprova/view.php', array('appid' => $appid));
            $pagebar = $OUTPUT->paging_bar($totalcount, $page, $perpage, $url);
            echo $output = html_writer::table($table);
            echo $pagebar;
            
        } else {
            echo html_writer::tag('p', get_string('noresults_at_search_attempt', 'prova'));
        }
    }

    /* Função que subtrai duas datas em formato unix timestamp
    *  e converte em formato H:M:S */
    function calculate_date($dateinit, $datefinish){

        $date_init = new DateTime();
        $date_finish = new DateTime();
        
        $date_init->setTimestamp($dateinit);
        $date_finish->setTimestamp($datefinish);

        $interval = $date_init->diff($date_finish);

        $hours_exec    =   ($interval->h <= 9) ? ('0'.$interval->h) : ($interval->h);
        $minutes_exec  =   ($interval->i <= 9) ? ('0'.$interval->i) : ($interval->i);
        $seconds_exec  =   ($interval->s <= 9) ? ('0'.$interval->s) : ($interval->s);

        $date_exec_format = $hours_exec .':'. $minutes_exec .':'. $seconds_exec;

        return $date_exec_format;
    }

?>