<?php
/**
 * This page allows the teacher to enter a manual grade for a particular question.
 * This page is expected to only be used in a popup window.
 *
 * @package mod_prova
 */

require_once(__DIR__ . '/../config.php');
require_once($CFG->dirroot . '/mod/prova/locallib.php');

$attemptid = required_param('attempt', PARAM_INT);
$slot = required_param('slot', PARAM_INT); // The question number in the attempt.
$cmid = optional_param('cmid', null, PARAM_INT);
$qcancel = optional_param('qcancel', null, PARAM_INT);

$PAGE->set_url('/mod/prova/screens/report/comment.php', array('attempt' => $attemptid, 'slot' => $slot));

$attemptobj = prova_create_attempt_handling_errors($attemptid, $cmid);
$student = $DB->get_record('user', array('id' => $attemptobj->get_userid()));

// Can only grade finished attempts.
if (!$attemptobj->is_finished()) {
    print_error('attemptclosed', 'prova');
}

// Check login and permissions.
require_login($attemptobj->get_course(), false, $attemptobj->get_cm());
$attemptobj->require_capability('mod/prova:grade');

// Print the page header.
$PAGE->set_pagelayout('popup');
$PAGE->set_title(get_string('manualgradequestion', 'prova', array(
        'question' => format_string($attemptobj->get_question_name($slot)),
        'prova' => format_string($attemptobj->get_prova_name()), 'user' => fullname($student))));
$PAGE->set_heading($attemptobj->get_course()->fullname);
$output = $PAGE->get_renderer('mod_prova');
$PAGE->requires->css('/mod/prova/comment.css');
echo $output->header();


// Prepare summary information about this question attempt.
$summarydata = array();

// Student name.
$userpicture = new user_picture($student);
$userpicture->courseid = $attemptobj->get_courseid();
$summarydata['user'] = array(
    'title'   => $userpicture,
    'content' => new action_link(new moodle_url('/user/view.php', array(
            'id' => $student->id, 'course' => $attemptobj->get_courseid())),
            fullname($student, true)),
);

// Prova name.
$summarydata['provaname'] = array(
    'title'   => get_string('modulename', 'prova'),
    'content' => format_string($attemptobj->get_prova_name()),
);

// Question name.
$summarydata['questionname'] = array(
    'title'   => get_string('question', 'prova'),
    'content' => $attemptobj->get_question_name($slot),
);

// Process any data that was submitted.
if (data_submitted() && confirm_sesskey()) {
    if (optional_param('submit', false, PARAM_BOOL) && question_engine::is_manual_grade_in_range($attemptobj->get_uniqueid(), $slot)) {
        $transaction = $DB->start_delegated_transaction();
        $attemptobj->process_submitted_actions(time());
        $transaction->allow_commit();

        // Log this action.
        $params = array(
            'objectid' => $attemptobj->get_question_attempt($slot)->get_question()->id,
            'courseid' => $attemptobj->get_courseid(),
            'context' => context_module::instance($attemptobj->get_cmid()),
            'other' => array(
                'provaid' => $attemptobj->get_provaid(),
                'attemptid' => $attemptobj->get_attemptid(),
                'slot' => $slot
            )
        );
        $event = \mod_prova\event\question_manually_graded::create($params);
        $event->trigger();

        echo $output->notification(get_string('changessaved'), 'notifysuccess');
        close_window(2, true);
        die;
    }
}

// Print prova information.
echo $output->review_summary_table($summarydata, 0);

$dropbuttom = $PAGE->get_renderer('mod_prova', 'comment');

$warningtextform = html_writer::tag('p', get_string('warningcancelquestion', 'prova'));
$warningbody = html_writer::tag('span', $warningtextform, array('id' => 'warningselectprova'));

// Print the comment form.
echo '<form method="post" class="mform" id="manualgradingform" action="' .
        $CFG->wwwroot . '/mod/prova/screens/report/comment.php">';
echo $attemptobj->render_question_for_commenting($slot);
?>

<div>

    <input type="hidden" name="attempt" value="<?php echo $attemptobj->get_attemptid(); ?>" />
    <input type="hidden" name="slot" value="<?php echo $slot; ?>" />
    <input type="hidden" name="slots" value="<?php echo $slot; ?>" />
    <input type="hidden" name="sesskey" value="<?php echo sesskey(); ?>" />
    
</div>

<fieldset class="hidden">
    <div>
        <div class="fitem fitem_actionbuttons fitem_fsubmit">
            <fieldset class="felement fsubmit">
                <?php if($qcancel){echo $output->notification(get_string('warningcancelquestion', 'prova'), 'notifysuccess'); }?>
                <?php if($qcancel == 2){echo $output->notification(get_string('warningcancelquestionerror', 'prova'), 'notifyerros'); }?>
                <?php echo $button = $dropbuttom->seox_add_menu_actions($attemptid, $slot, $attemptobj->get_prova()->get_prova()); ?> 

                <input id="id_submitbutton" type="submit" name="submit" class="btn btn-primary" value="<?php print_string('save', 'prova'); ?>"/>
            </fieldset>
        </div>
    </div>
</fieldset>
<?php
echo '</form>';
$PAGE->requires->js_init_call('M.mod_prova.init_comment_popup', null, false, prova_get_js_module());
$PAGE->requires->event_handler('.dropdown-item', 'click', 'M.util.show_confirm_dialog',
        array('message' => get_string('cancel_question_message', 'prova')));
// End of the page.

echo $output->footer();

 	


