<?php
/**
 * This script controls the display of the prova reports.
 *
 * @package mod_prova
 */

define('NO_OUTPUT_BUFFERING', true);

require_once(__DIR__ . '/../config.php');
require_once($CFG->dirroot . '/mod/prova/locallib.php');
require_once($CFG->dirroot . '/mod/prova/src/attempt/lib.php');
require_once($CFG->dirroot . '/mod/prova/src/report/reportlib.php');
require_once($CFG->dirroot . '/mod/prova/src/report/default.php');

$return_url = optional_param('returnurl', 0, PARAM_URL);
$PAGE->navbar->add(get_string('form_return_title', 'prova'), new moodle_url($return_url));

$id = optional_param('id', 0, PARAM_INT);
$q = optional_param('q', 0, PARAM_INT);
$mode = optional_param('mode', '', PARAM_ALPHA);

if ($id) {
    if (!$cm = get_coursemodule_from_id('prova', $id)) {
        print_error('invalidcoursemodule');
    }
    if (!$course = $DB->get_record('course', array('id' => $cm->course))) {
        print_error('coursemisconf');
    }
    if (!$provaobj = prova::create($cm->instance)) {
        print_error('invalidcoursemodule');
    }
    $prova = $provaobj->get_prova();
} else {
    if (!$provaobj = prova::create($q)) {
        print_error('invalidprovaid', 'prova');
    }
    $prova = $provaobj->get_prova();
    
    if (!$course = $DB->get_record('course', array('id' => $prova->course))) {
        print_error('invalidcourseid');
    }
    if (!$cm = get_coursemodule_from_instance("prova", $prova->id, $course->id)) {
        print_error('invalidcoursemodule');
    }
}

$url = new moodle_url('/mod/prova/screens/report/report.php', array('id' => $cm->id));
if ($mode === '') {
    $mode = 'responses';
}
$url->param('mode', $mode);
$PAGE->set_url($url);
$PAGE->requires->css('/mod/prova/src/estatisticasquestao/report_form_buttom.css');

require_login($course, false, $cm);
$context = context_module::instance($cm->id);
$PAGE->set_pagelayout('report');

$reportlist = prova_report_list($context);
if (empty($reportlist)) {
    print_error('erroraccessingreport', 'prova');
}

// Validate the requested report name.
/*  if ($mode == '') { */
     // Default to first accessible report and redirect.
 /*     $url->param('mode', reset($reportlist));
     redirect($url);
} else if (!in_array($mode, $reportlist)) {
     print_error('erroraccessingreport', 'prova');
 }
if (!is_readable("report/$mode/report.php")) {
     print_error('reportnotfound', 'prova', '', $mode);
 } */

// Open the selected prova report and display it.
$file = $CFG->dirroot . '/mod/prova/src/report/' . $mode . '/report.php';
if (is_readable($file)) {
    include_once($file);
}
$reportclassname = 'prova_' . $mode . '_report';
if (!class_exists($reportclassname)) {
    print_error('preprocesserror', 'prova');
}

$report = new $reportclassname();

$report->display($prova, $cm, $course);

// Print footer.
echo $OUTPUT->footer();

// Log that this report was viewed.
$params = array(
    'context' => $context,
    'other' => array(
        'provaid' => $prova->id,
        'reportname' => $mode
    )
);
$event = \mod_prova\event\report_viewed::create($params);
$event->add_record_snapshot('course', $course);
$event->add_record_snapshot('prova', $prova);
$event->trigger();
