<?php
/**
 * This page prints a review of a particular question attempt.
 * This page is expected to only be used in a popup window.
 *
 * @package   mod_prova
 * @copyright 1999 onwards Martin Dougiamas  {@link http://moodle.com}
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */


require_once(__DIR__ . '/../config.php');
require_once($CFG->dirroot . '/mod/prova/locallib.php');

$attemptid = required_param('attempt', PARAM_INT);
$slot = required_param('slot', PARAM_INT);
$slots = optional_param('slots', null, PARAM_TEXT);
$page = optional_param('page', null, PARAM_INT);
$seq = optional_param('step', null, PARAM_INT);
$cmid = optional_param('cmid', null, PARAM_INT);

$baseurl = new moodle_url('/mod/prova/screens/report/reviewquestion.php',
        array('attempt' => $attemptid, 'slot' => $slot));
$currenturl = new moodle_url($baseurl);
if (!is_null($seq)) {
    $currenturl->param('step', $seq);
}
$PAGE->set_url($currenturl);

$attemptobj = prova_create_attempt_handling_errors($attemptid, $cmid);

// Check login.
require_login($attemptobj->get_course(), false, $attemptobj->get_cm());
$attemptobj->check_review_capability();
$student = $DB->get_record('user', array('id' => $attemptobj->get_userid()));

$accessmanager = $attemptobj->get_access_manager(time());
$options = $attemptobj->get_display_options(true, false, true);

$PAGE->set_pagelayout('popup');
$PAGE->set_title(get_string('reviewofquestion', 'prova', array(
        'question' => format_string($attemptobj->get_question_name($slot)),
        'prova' => format_string($attemptobj->get_prova_name()), 'user' => fullname($student))));
$PAGE->set_heading($attemptobj->get_course()->fullname);
$PAGE->requires->css('/mod/prova/components/addprovas/preview.scss');
$PAGE->requires->css('/mod/prova/src/report/responses/status.css');

$output = $PAGE->get_renderer('mod_prova');

// Check permissions - warning there is similar code in review.php and
// prova_attempt::check_file_access. If you change on, change them all.
if ($attemptobj->is_own_attempt()) {
    if (!$attemptobj->is_finished()) {
        echo $output->review_question_not_allowed($attemptobj, get_string('cannotreviewopen', 'prova'));
        die();
    } else if (!$options->attempt) {
        echo $output->review_question_not_allowed($attemptobj,
                $attemptobj->cannot_review_message());
        die();
    }

} else if (!$attemptobj->is_review_allowed()) {
    throw new moodle_prova_exception($attemptobj->get_provaobj(), 'noreviewattempt');
}

// Prepare summary informat about this question attempt.
$summarydata = array();

// Student name.
$userpicture = new user_picture($student);
$userpicture->courseid = $attemptobj->get_courseid();
$summarydata['user'] = array(
    'title'   => $userpicture,
    'content' => new action_link(new moodle_url('/user/view.php', array(
            'id' => $student->id, 'course' => $attemptobj->get_courseid())),
            fullname($student, true)),
);

// Prova name.
$summarydata['provaname'] = array(
    'title'   => get_string('modulename', 'prova'),
    'content' => format_string($attemptobj->get_prova_name()),
);

// Question name.
$summarydata['questionname'] = array(
    'title'   => get_string('question', 'prova'),
    'content' => $attemptobj->get_question_name($slot),
);

// Other attempts at the prova.
if ($attemptobj->has_capability('mod/prova:viewreports')) {
    $attemptlist = $attemptobj->links_to_other_attempts($baseurl);
    if ($attemptlist) {
        $summarydata['attemptlist'] = array(
            'title'   => get_string('attempts', 'prova'),
            'content' => $attemptlist,
        );
    }
}

// Timestamp of this action.
$timestamp = $attemptobj->get_question_action_time($slot);
if ($timestamp) {
    $summarydata['timestamp'] = array(
        'title'   => get_string('completedon', 'prova'),
        'content' => userdate($timestamp),
    );
}


echo $output->review_question_page($attemptobj, $slot, $seq, $attemptobj->get_display_options(true), $summarydata, $slots, $page);
