<?php
/**
 * Base class for the settings form for {@link prova_attempts_report}s.
 *
 * @package mod_prova
 */


defined('MOODLE_INTERNAL') || die();

require_once($CFG->libdir . '/formslib.php');


/**
 * Base class for the settings form for {@link prova_attempts_report}s.
 *
 * @copyright 2012 The Open University
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
abstract class mod_prova_attempts_report_form extends moodleform {

    protected function definition() {
        $mform = $this->_form;

        $mform->addElement('header', 'preferencespage',
                get_string('reportwhattoinclude', 'prova'));

        $this->standard_attempt_fields($mform);
        // $this->other_attempt_fields($mform);

        // $mform->addElement('header', 'preferencesuser',
        //         get_string('reportdisplayoptions', 'prova'));

        // $this->standard_preference_fields($mform);
        // Funcao estava quebrando botao de liberar notas.
        // $this->other_preference_fields($mform);

        $mform->addElement('submit', 'submitbutton',
                get_string('showreport', 'prova'));
    }

    protected function standard_attempt_fields(MoodleQuickForm $mform) {

        $mform->addElement('select', 'attempts', get_string('reportattemptsfrom', 'prova'), array(
                    prova_attempts_report::ENROLLED_WITH    => get_string('reportuserswith', 'prova'),
                    prova_attempts_report::ENROLLED_WITHOUT => get_string('reportuserswithout', 'prova'),
                    prova_attempts_report::ENROLLED_ALL     => get_string('reportuserswithorwithout', 'prova'),
                    prova_attempts_report::ALL_WITH        => get_string('reportusersall', 'prova'),
                 ));

        $stategroup = array(
            $mform->createElement('advcheckbox', 'stateinprogress', '',
                    get_string('stateinprogress', 'prova')),
            $mform->createElement('advcheckbox', 'stateoverdue', '',
                    get_string('stateoverdue', 'prova')),
            $mform->createElement('advcheckbox', 'statefinished', '',
                    get_string('statefinished', 'prova')),
            $mform->createElement('advcheckbox', 'stateabandoned', '',
                    get_string('stateabandoned', 'prova')),
        );
        $mform->addGroup($stategroup, 'stateoptions',
                get_string('reportattemptsthatare', 'prova'), array(' '), false);
        $mform->setDefault('stateinprogress', 1);
        $mform->setDefault('stateoverdue',    1);
        $mform->setDefault('statefinished',   1);
        $mform->setDefault('stateabandoned',  1);
        $mform->disabledIf('stateinprogress', 'attempts', 'eq', prova_attempts_report::ENROLLED_WITHOUT);
        $mform->disabledIf('stateoverdue',    'attempts', 'eq', prova_attempts_report::ENROLLED_WITHOUT);
        $mform->disabledIf('statefinished',   'attempts', 'eq', prova_attempts_report::ENROLLED_WITHOUT);
        $mform->disabledIf('stateabandoned',  'attempts', 'eq', prova_attempts_report::ENROLLED_WITHOUT);

        if (prova_report_can_filter_only_graded($this->_customdata['prova'])) {
            $gm = html_writer::tag('span',
                    prova_get_grading_option_name($this->_customdata['prova']->grademethod),
                    array('class' => 'highlight'));
            $mform->addElement('advcheckbox', 'onlygraded', '',
                    get_string('reportshowonlyfinished', 'prova', $gm));
            $mform->disabledIf('onlygraded', 'attempts', 'eq', prova_attempts_report::ENROLLED_WITHOUT);
            $mform->disabledIf('onlygraded', 'statefinished', 'notchecked');
        }
    }

    protected function other_attempt_fields(MoodleQuickForm $mform) {
    }

    protected function standard_preference_fields(MoodleQuickForm $mform) {
        $mform->addElement('text', 'pagesize', get_string('pagesize', 'prova'));
        $mform->setType('pagesize', PARAM_INT);
    }

    protected function other_preference_fields(MoodleQuickForm $mform) {
    }

    public function validation($data, $files) {
        $errors = parent::validation($data, $files);

        if ($data['attempts'] != prova_attempts_report::ENROLLED_WITHOUT && !(
                $data['stateinprogress'] || $data['stateoverdue'] || $data['statefinished'] || $data['stateabandoned'])) {
            $errors['stateoptions'] = get_string('reportmustselectstate', 'prova');
        }

        return $errors;
    }
}
