<?php
/**
 * This file defines the prova responses table for showing last try at question.
 *
 * @package prova_responses
 */


defined('MOODLE_INTERNAL') || die();

require_once($CFG->dirroot . '/mod/prova/src/report/attemptsreport_table.php');


/**
 * This is a table subclass for displaying the prova responses report.
 *
 * @copyright 2008 Jean-Michel Vedrine
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class prova_last_responses_table extends prova_attempts_report_table {

    /** @var int number of inprogress attempts **/
    public $inprogess_attempts = 0;

    /** @var int number of notcorrected attempts **/
    public $notcorrected_attempts = 0;

    /**
     * Constructor
     * @param object $prova
     * @param context $context
     * @param string $qmsubselect
     * @param prova_responses_options $options
     * @param \core\dml\sql_join $groupstudentsjoins
     * @param \core\dml\sql_join $studentsjoins
     * @param array $questions
     * @param moodle_url $reporturl
     */
    public function __construct($prova, $context, $qmsubselect, prova_responses_options $options,
            \core\dml\sql_join $groupstudentsjoins, \core\dml\sql_join $studentsjoins, $questions, $reporturl) {
        parent::__construct('mod-prova-report-responses-report', $prova, $context,
                $qmsubselect, $options, $groupstudentsjoins, $studentsjoins, $questions, $reporturl);
    }

    public function build_table() {
        if (!$this->rawdata) {
            return;
        }

        $this->strtimeformat = str_replace(',', ' ', get_string('strftimedatetime'));
        parent::build_table();
    }

    public function col_sumgrades($attempt) {
        if ($attempt->state != prova_attempt::FINISHED) {
            return '-';
        }

        $grade = prova_rescale_grade($attempt->sumgrades, $this->prova);
        if ($this->is_downloading()) {
            return $grade;
        }

        /* $gradehtml = '<a href="review.php?q=' . $this->prova->id . '&amp;attempt=' .
                $attempt->attempt . '">' . $grade . '</a>'; */

        $gradehtml =  $grade;
        return $gradehtml;
    }

    public function data_col($slot, $field, $attempt) {
        if ($attempt->usageid == 0) {
            return '-';
        }

        $value = $this->field_from_extra_data($attempt, $slot, $field);

        if (is_null($value)) {
            $summary = '-';
        } else {
            $summary = trim($value);
        }

        if ($this->is_downloading() && $this->is_downloading() != 'html') {
            return $summary;
        }
        $summary = s($summary);

        if ($this->is_downloading() || $field != 'responsesummary') {
            return $summary;
        }

        return $this->make_review_link($summary, $attempt, $slot);
    }

    public function status_col($attempt, $slots) {
        global $OUTPUT;
        $output = '';

        if ($attempt->state == prova_attempt::IN_PROGRESS) {
            $this->inprogess_attempts++;
        }

        $slottarget = $slots[0];
        $status = 'corrected';
        $complemento = ['slots' => implode('-',$slots), 'page' => 0];
        foreach ($slots as $key => $slot) {
            $state = $this->slot_state($attempt, $slot);
            if ($state->is_finished()) {
                $fraction = $this->slot_fraction($attempt, $slot);
                if ($state == question_state::$needsgrading) {
                    $this->notcorrected_attempts++;
                    $slottarget = $slot;
                    $status = 'notcorrected';
                    $complemento['page'] = $key;
                    break;
                }
            }
        }

        $reviewparams = array_merge(array('attempt' => $attempt->attempt, 'slot' => $slottarget), $complemento);
        $url = new moodle_url('/mod/prova/screens/report/reviewquestion.php', $reviewparams);
        $popup = new popup_action('click', $url, 'reviewquestion', array('height' => 700, 'width' => 1350));
        $link = $OUTPUT->action_link($url, get_string('reviewgeneralresponses_'.$status, 'prova_responses'), $popup, array('title' => get_string('reviewgeneralresponses_'.$status, 'prova_responses')));

        $output .= '<span class="prova-report-correction '.$status.'">';
        $output .= $link;
        $output .= '</span>';

        return $attempt->state == prova_attempt::IN_PROGRESS ? '-' : $output;
    }

    /**
     * Column text from the extra data loaded in load_extra_data(), before html formatting etc.
     *
     * @param object $attempt
     * @param int $slot
     * @param string $field
     * @return string
     */
    protected function field_from_extra_data($attempt, $slot, $field) {
        if (!isset($this->lateststeps[$attempt->usageid][$slot])) {
            return '-';
        }
        $stepdata = $this->lateststeps[$attempt->usageid][$slot];

        if (property_exists($stepdata, $field . 'full')) {
            $value = $stepdata->{$field . 'full'};
        } else {
            $value = $stepdata->$field;
        }
        return $value;
    }

    public function other_cols($colname, $attempt) {

        if (preg_match('/^status(\S+)$/', $colname, $matches)) {
            $slots = explode('-', $matches[1]);
            return $this->status_col($attempt, $slots);
        }

        if (preg_match('/^question(\d+)$/', $colname, $matches)) {
            return $this->data_col($matches[1], 'questionsummary', $attempt);

        } else if (preg_match('/^response(\d+)$/', $colname, $matches)) {
            return $this->data_col($matches[1], 'responsesummary', $attempt);

        } else if (preg_match('/^right(\d+)$/', $colname, $matches)) {
            return $this->data_col($matches[1], 'rightanswer', $attempt);

        } else {
            return null;
        }
    }

    protected function requires_extra_data() {
        return true;
    }

    protected function is_latest_step_column($column) {
        if (preg_match('/^(?:question|response|right)([0-9]+)/', $column, $matches)) {
            return $matches[1];
        }
        return false;
    }

    /**
     * Get any fields that might be needed when sorting on date for a particular slot.
     * @param int    $slot  the slot for the column we want.
     * @param string $alias the table alias for latest state information relating to that slot.
     * @return string sql fragment to alias fields.
     */
    protected function get_required_latest_state_fields($slot, $alias) {
        global $DB;
        $sortableresponse = $DB->sql_order_by_text("{$alias}.questionsummary");
        if ($sortableresponse === "{$alias}.questionsummary") {
            // Can just order by text columns. No complexity needed.
            return "{$alias}.questionsummary AS question{$slot},
                    {$alias}.rightanswer AS right{$slot},
                    {$alias}.responsesummary AS response{$slot}";
        } else {
            // Work-around required.
            return $DB->sql_order_by_text("{$alias}.questionsummary") . " AS question{$slot},
                    {$alias}.questionsummary AS question{$slot}full,
                    " . $DB->sql_order_by_text("{$alias}.rightanswer") . " AS right{$slot},
                    {$alias}.rightanswer AS right{$slot}full,
                    " . $DB->sql_order_by_text("{$alias}.responsesummary") . " AS response{$slot},
                    {$alias}.responsesummary AS response{$slot}full";
        }
    }

    /**
     * Gera em extra form data para registrar dificuldade das novas
     * questões corrigidas
     *
     * @return array, array no formato de parametros de url
     */
    public function calculate_questions_diff() {
        global $DB;

        $questions = [];
        foreach ($this->lateststeps as $step) {
            foreach ($step as $data) {
                if (!array_key_exists($data->questionid, $questions)) {
                    $questions[$data->questionid] = ['corrected' => 0, 'answered' => 0];
                }
                if ($data->fraction !== null) {
                    $data->fraction == $data->maxmark && $questions[$data->questionid]['corrected']++;   
                }
                $questions[$data->questionid]['answered']++;
            }
        }

        // D = (DA/3 + (1 - QA/QR)) / 2
        // DA dificuldade atual
        // QA quantidade de respondidas corretamente
        // QR quantidade de respondidas

        $qids = array_keys($questions);
        $calculated_qdiffs = [];

        if ($qids and !empty($qids)) {

            $current_diffs_clause = 'id_question IN (' . implode(',', $qids) .')';
            $current_diffs = $DB->get_records_select('seox_questions', $current_diffs_clause, null, null, 'id_question, dificuldade');

            $current_QA_QR_clause = 'questionid IN (' . implode(',', $qids) .')';
            $current_QA_QR = $DB->get_records_select('seox_questions_difficulty', $current_QA_QR_clause, null, null, 'id, questionid, qa, qr');

            foreach ($questions as $qid => $type) {
                if (!array_key_exists($qid, $current_diffs) or !$current_diffs[$qid]) { continue; }

                list($QA_old, $QR_old) = array_key_exists($qid, $current_QA_QR) ? [$current_QA_QR[$qid]->qa, $current_QA_QR[$qid]->qr] : [0, 0];
                $DA = $current_diffs[$qid]->dificuldade + 1;
                $QA = $QA_old + $type['corrected'];
                $QR = $QA_old + $type['answered'];
                $calculated_qdiffs['q'.$qid] = ($DA/3 + (1 - $QA/$QR)) / 2;
                $calculated_qdiffs['q'.$qid.'-qa'] = $QA;
                $calculated_qdiffs['q'.$qid.'-qr'] = $QR;
            }
        } else {
            $calculated_qdiffs = ['noatttempts' => 1];
        }

        return $calculated_qdiffs;
    }

    /**
     * Gera o botão para liberar notas, correções e comentários aos
     * alunos que realizaram a aplicação de uma prova
     *
     * @return html do botão, com um alert e javascript
     */
    public function allow_users_complete_review_btn($prova) {
        // Function deprecated.
        return;
        global $OUTPUT;

        $output = '';

        if (!$prova->corrected) {
            $provaid = $prova->id;
            // Finish attempt button.
            $options = array(
                'provaid' => $provaid,
                'sesskey' => sesskey()
            );

            $q_diffs = $this->calculate_questions_diff();
            $options = array_merge($options, $q_diffs);

            $button = new single_button(new moodle_url('/mod/prova/screens/report/processreport.php', $options), get_string('correctedformbtn', 'prova'));
            $button->id = 'correctedform';

            $message = '';
            if ($this->inprogess_attempts) {
                $message .= get_string('confirmclosecorrectedform_attemptstate', 'prova', '<strong>'.$this->inprogess_attempts.'</strong>');
            } 
            if ($this->notcorrected_attempts) {
                $message .= (($message != '') ? '<br>' : '') . get_string('confirmclosecorrectedform_correction', 'prova', '<strong>'.$this->notcorrected_attempts.'</strong>');
            }
            $message .= (($message != '') ? '<br>' : '') . get_string('confirmclosecorrectedform', 'prova');

            $button->add_action(new confirm_action($message, null, get_string('submitcorrectedform', 'prova')));

            $output .= '</form>';
            $output .= $OUTPUT->render($button);
            
        } else {
            $output .= '<span id="corrected-done-alert">' . get_string('submitcorrectedform_alert', 'prova') . '<span>';
        }

        return $OUTPUT->container($OUTPUT->container($output, 'controls'), 'submitbtns mdl-align');
    }
}
