<?php
/**
 * This file defines the setting form for the prova responses report.
 *
 * @package prova_responses
 */

defined('MOODLE_INTERNAL') || die();

require_once($CFG->dirroot . '/mod/prova/src/report/attemptsreport_form.php');


/**
 * Quiz responses report settings form.
 *
 * @copyright 2008 Jean-Michel Vedrine
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class prova_responses_settings_form extends mod_prova_attempts_report_form {

    protected function other_preference_fields(MoodleQuickForm $mform) {

        $generalname = array('method' => 'GET');

        $mform->setAttributes($generalname);

        $mform->addGroup(array(
            $mform->createElement('advcheckbox', 'resp', '',
                get_string('response', 'prova_responses')),
            $mform->createElement('advcheckbox', 'qtext', '',
                get_string('questiontext', 'prova_responses')),
            $mform->createElement('advcheckbox', 'right', '',
                get_string('rightanswer', 'prova_responses')),
            
        ), 'coloptions', get_string('showthe', 'prova_responses'), array(' '), false);
        $mform->disabledIf('resp',  'attempts', 'eq', prova_attempts_report::ENROLLED_WITHOUT);
        $mform->disabledIf('qtext', 'attempts', 'eq', prova_attempts_report::ENROLLED_WITHOUT);
        $mform->disabledIf('right', 'attempts', 'eq', prova_attempts_report::ENROLLED_WITHOUT);
    }

    public function validation($data, $files) {
        $errors = parent::validation($data, $files);

        if ($data['attempts'] != prova_attempts_report::ENROLLED_WITHOUT && !(
                $data['qtext'] || $data['resp'] || $data['right'])) {
            $errors['coloptions'] = get_string('reportmustselectstate', 'prova');
        }

        return $errors;
    }

    protected function other_attempt_fields(MoodleQuickForm $mform) {
        parent::other_attempt_fields($mform);
        if (prova_allows_multiple_tries($this->_customdata['prova'])) {
            $mform->addElement('select', 'whichtries', get_string('whichtries', 'question'), array(
                                           question_attempt::FIRST_TRY    => get_string('firsttry', 'question'),
                                           question_attempt::LAST_TRY     => get_string('lasttry', 'question'),
                                           question_attempt::ALL_TRIES    => get_string('alltries', 'question'))
            );
            $mform->setDefault('whichtries', question_attempt::LAST_TRY);
            $mform->disabledIf('whichtries', 'attempts', 'eq', prova_attempts_report::ENROLLED_WITHOUT);
        }
    }
}
