<?php
/**
 * Class to store the options for a {@link prova_responses_report}.
 *
 * @package   prova_responses
 * @copyright 2012 The Open University
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */


defined('MOODLE_INTERNAL') || die();

require_once($CFG->dirroot . '/mod/prova/src/report/attemptsreport_options.php');


/**
 * Class to store the options for a {@link prova_responses_report}.
 *
 * @copyright 2012 The Open University
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class prova_responses_options extends mod_prova_attempts_report_options {

    /** @var bool whether to show the students' reduced mode columns. */
    public $showreducedmode = true;

    /** @var bool whether to show the question text columns. */
    public $showqtext = false;

    /** @var bool whether to show the students' response columns. */
    public $showresponses = false;

    /** @var bool whether to show the correct response columns. */
    public $showright = false;

    /** @var bool which try/tries to show responses from. */
    public $whichtries = question_attempt::LAST_TRY;

    protected function get_url_params() {
        $params = parent::get_url_params();
        $params['general']       = $this->showreducedmode;
        $params['qtext']      = $this->showqtext;
        $params['resp']       = $this->showresponses;
        $params['right']      = $this->showright;
        if (prova_allows_multiple_tries($this->prova)) {
            $params['whichtries'] = $this->whichtries;
        }
        return $params;
    }

    public function get_initial_form_data() {
        $toform = parent::get_initial_form_data();
        $toform->general    = $this->showreducedmode;
        $toform->qtext      = $this->showqtext;
        $toform->resp       = $this->showresponses;
        $toform->right      = $this->showright;
        if (prova_allows_multiple_tries($this->prova)) {
            $toform->whichtries = $this->whichtries;
        }

        return $toform;
    }

    public function setup_from_form_data($fromform) {
        parent::setup_from_form_data($fromform);

        $this->showreducedmode  = $toform->general;
        $this->showqtext        = $fromform->qtext;
        $this->showresponses    = $fromform->resp;
        $this->showright        = $fromform->right;
        if (prova_allows_multiple_tries($this->prova)) {
            $this->whichtries = $fromform->whichtries;
        }
    }

    public function setup_from_params() {
        parent::setup_from_params();

        $this->showreducedmode  = optional_param('general', $this->showreducedmode,     PARAM_BOOL);
        $this->showqtext        = optional_param('qtext', $this->showqtext,     PARAM_BOOL);
        $this->showresponses    = optional_param('resp',  $this->showresponses, PARAM_BOOL);
        $this->showright        = optional_param('right', $this->showright,     PARAM_BOOL);
        if (prova_allows_multiple_tries($this->prova)) {
            $this->whichtries    = optional_param('whichtries', $this->whichtries, PARAM_ALPHA);
        }
    }

    public function setup_from_user_preferences() {
        parent::setup_from_user_preferences();

        $this->showreducedmode  = get_user_preferences('prova_report_responses_general', $this->showreducedmode);
        $this->showqtext        = get_user_preferences('prova_report_responses_qtext', $this->showqtext);
        $this->showresponses    = get_user_preferences('prova_report_responses_resp',  $this->showresponses);
        $this->showright        = get_user_preferences('prova_report_responses_right', $this->showright);
        if (prova_allows_multiple_tries($this->prova)) {
            $this->whichtries    = get_user_preferences('prova_report_responses_which_tries', $this->whichtries);
        }
    }

    public function update_user_preferences() {
        parent::update_user_preferences();

        set_user_preference('prova_report_responses_general', $this->showreducedmode);
        set_user_preference('prova_report_responses_qtext', $this->showqtext);
        set_user_preference('prova_report_responses_resp',  $this->showresponses);
        set_user_preference('prova_report_responses_right', $this->showright);
        if (prova_allows_multiple_tries($this->prova)) {
            set_user_preference('prova_report_responses_which_tries', $this->whichtries);
        }
    }

    public function resolve_dependencies() {
        parent::resolve_dependencies();

        if (!$this->showreducedmode && !$this->showqtext && !$this->showresponses && !$this->showright) {
            // We have to show at least something.
            $this->showreducedmode = true;
        }

        // We only want to show the checkbox to delete attempts
        // if the user has permissions and if the report mode is showing attempts.
        $this->checkboxcolumn = has_capability('mod/prova:deleteattempts', context_module::instance($this->cm->id))
                && ($this->attempts != prova_attempts_report::ENROLLED_WITHOUT);
    }
}
