<?php
require_once(__DIR__ . '/../../config.php');
require_once($CFG->dirroot.'/mod/prova/locallib.php');
$PAGE->set_context(context_system::instance());

$appid = optional_param('appid', 0, PARAM_INT);
$cmid = optional_param('cm', 0, PARAM_INT);
$return_url = optional_param('returnurl', 0, PARAM_URL);


$cm = get_coursemodule_from_id('prova', $cmid);
if (!$course = $DB->get_record('course', array('id' => $cm->course))) {
    print_error('coursemisconf');
}

$PAGE->set_url('/mod/prova/viewattempts.php', array('appid' => $appid));
$PAGE->set_heading('Listagem das Tentativas');
$PAGE->set_title(get_string('searchprovaaplicationtitle', 'prova'));
$PAGE->navbar->add(get_string('form_return_title', 'prova'), new moodle_url($return_url));

// \mod_prova\manager::has_permission_to_access();

require_login($course, false, $cm);
$context = context_module::instance($cm->id);
require_capability('mod/prova:view', $context);

echo $OUTPUT->header();
echo print_prova_attempt($appid);
echo print_table_seox($appid);
echo $OUTPUT->footer();

function print_prova_attempt($appid){

    global $DB;
    $sql = "SELECT prova_app.name AS nome_prova, 
                prova_app.timecreated AS dt_hora_criacao_encoded,
                /* to_char(to_timestamp(prova_app.timecreated),'DD-MM-YYYY HH24:MI:SS') AS dt_hora_criacao, */
                /* ROUND(AVG(prova_att.sumgrades)::numeric,2) AS media_notas, */
                AVG(prova_att.sumgrades) AS media
            FROM {prova} prova_app
            LEFT JOIN {prova_attempts} prova_att
            ON prova_app.id = prova_att.prova";

    $sql .= ' WHERE prova_app.id = '.$appid;
    $sql .= ' GROUP BY prova_app.id';

    $query = $DB->get_records_sql($sql);

    /* return $query; */

    if (!empty($query)) {
        $table = new html_table();

        $table->head = array('Nome da Prova', 'Data e hora da Criação', 'Média de Notas');

        // Gera o body da tabela
        $cont = 1;
        foreach ($query as $prova) {
         
            $row = new html_table_row();

            foreach ($prova as $key => $field) {

                if ($key == 'dt_hora_criacao_encoded') { continue; }
                if ($key == 'media') { continue; }

                if ($key == 'dt_hora_criacao' and $prova->dt_hora_criacao_encoded == 0) {
                    $row->cells[] = ' - ';
                    continue;
                }
               
                $row->cells[] = $field;
            }

            $media_notas        = number_format(round($prova->media, 2), 2);
            $dt_hora_criacao    = format_date($prova->dt_hora_criacao_encoded);
            $row->cells[]       = $dt_hora_criacao;
            $row->cells[]       = $media_notas;
            $table->data[]      = $row;
            $cont++;
        }
        $table_form = html_writer::table($table);
    } else {
        $table_form = html_writer::tag('p', get_string('noresults_at_search', 'prova'));
    }

    return $table_form;
}

function print_table_seox($appid)
    {
        global $DB, $OUTPUT, $PAGE;
        $urldata = null;
       
        $sql = "SELECT 
            prova_att.id AS attempt_id,
            prova_att.userid userid, 
            CONCAT(users.firstname,' ', users.lastname) AS nome_aluno,
            prova_att.sumgrades AS nota,
            prova_att.timefinish AS timefinish,
            prova_att.timestart AS timestart,
            prova_att.state state
            from mdl_prova_attempts AS prova_att
            INNER JOIN mdl_user AS users
            ON users.id = prova_att.userid";

        $sql .= ' WHERE  prova_att.prova = '.$appid;
        $results = $DB->get_records_sql($sql);

        $provaobj = prova::create($appid);
        $output = $PAGE->get_renderer('mod_prova');

        if (!empty($results)) {
            $table = new html_table();

            $table->head = array('ID Usuario', 'Nome Usuario', 'Data e Hora da Execução', 'Nota do Usuário ', 'T. Execução Prova (H:M:s)', 'Status', 'Imprimir Prova');

            // Gera o body da tabela
            $cont = 1;
            foreach ($results as $prova) {

                $tempo_execucao     = calculate_date($prova->timestart, $prova->timefinish);
                $dt_hora_execucao   = format_date($prova->timestart);

                $row = new html_table_row();

                foreach ($prova as $key => $field) {

                    if ($key == 'attempt_id') { continue; }
                    if ($key == 'timefinish') { continue; }
                    if ($key == 'timestart') { continue; }
                    if ($key == 'nota') { continue; }

                    if (/* $key == 'tempo_execucao' and  */$prova->state != 'finished') { 
                        $row->cells[] = ' - ';
                        continue;
                    }

                    $row->cells[] = $field;
                }

                $nota_prova = number_format(round($prova->nota, 2), 2);
                $row->cells[2] = $dt_hora_execucao;
                $row->cells[3] = $nota_prova;
                $row->cells[4] = $tempo_execucao;
                $row->cells[5] = $prova->state != 'finished' ? ' - ' : 'Finalizado';

                if ($prova->state == 'finished') {
                    // Actions Field
                    // -- Print
                    $last_column = '<div style="display:inline-block; width: 100px">';
                    $last_column .= $output->printter_link($provaobj->printter_url($prova->attempt_id), true, array(
                        'left' => 100,
                        'top' => 100,
                        'height' => 700,
                        'width' => 1100,
                    ));                   
                
                    $row->cells[] = $last_column . '</div>';
                } else {
                    $row->cells[] = '';
                }

                $table->data[] = $row;
                $cont++;
            }
            $table_form = html_writer::table($table);
        } else {
            $table_form = html_writer::tag('p', get_string('noresults_at_search_attempt', 'prova'));
        }

        return $table_form;
    }

    /* Função que subtrai duas datas em formato unix timestamp
    *  e converte em formato H:M:S */
    function calculate_date($dateinit, $datefinish){

        $date_init = new DateTime();
        $date_finish = new DateTime();
        
        $date_init->setTimestamp($dateinit);
        $date_finish->setTimestamp($datefinish);

        $interval = $date_init->diff($date_finish);

        $hours_exec    =   ($interval->h <= 9) ? ('0'.$interval->h) : ($interval->h);
        $minutes_exec  =   ($interval->i <= 9) ? ('0'.$interval->i) : ($interval->i);
        $seconds_exec  =   ($interval->s <= 9) ? ('0'.$interval->s) : ($interval->s);

        $date_exec_format = $hours_exec .':'. $minutes_exec .':'. $seconds_exec;

        return $date_exec_format;
    }

?>